@@ -2,7 +2,7 @@
  * @file gnome-cmd-state.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 
 #pragma once
 
-struct GnomeCmdState 
+struct GnomeCmdState
 {
-    GnomeVFSURI *active_dir_uri;
-    GnomeVFSURI *inactive_dir_uri;
+    GFile *activeDirGfile;
+    GFile *inactiveDirGfile;
     GList *active_dir_files;
     GList *inactive_dir_files;
     GList *active_dir_selected_files;
