@@ -2,7 +2,7 @@
  * @file libgcmd-widget-factory.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ GtkWidget *lookup_widget (GtkWidget *widget, const gchar *widget_name)
 
     for (;;)
     {
-        parent = GTK_IS_MENU (widget) ? gtk_menu_get_attach_widget (GTK_MENU (widget)) : widget->parent;
+        parent = GTK_IS_MENU (widget) ? gtk_menu_get_attach_widget (GTK_MENU (widget)) : gtk_widget_get_parent(widget);
         if (!parent)
             break;
         widget = parent;
@@ -204,7 +204,7 @@ GtkWidget *create_named_button_with_data (GtkWidget *parent, const gchar *label,
     GtkAccelGroup *accel_group = gtk_accel_group_new ();
     GtkWidget *w = gtk_button_new_with_label ("");
 
-    key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (w)->child), label);
+    key = gtk_label_parse_uline (GTK_LABEL (gtk_bin_get_child( GTK_BIN (w))), label);
     gtk_widget_add_accelerator (w, "clicked", accel_group, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
     gtk_window_add_accel_group (GTK_WINDOW (parent), accel_group);
     g_object_ref (w);
@@ -290,11 +290,25 @@ GtkWidget *create_radio_with_mnemonic (GtkWidget *parent, GSList *group, gchar *
 GtkWidget *create_spin (GtkWidget *parent, const gchar *name, gint min, gint max, gint value)
 {
     GtkObject *adj = gtk_adjustment_new (value, min, max, 1, 10, 0);
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     GtkWidget *spin = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
     g_object_ref (spin);
     g_object_set_data_full (G_OBJECT (parent), name, spin, g_object_unref);
     gtk_widget_show (spin);
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spin), TRUE);
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 
     return spin;
 }
@@ -404,7 +418,14 @@ GtkWidget *create_icon_button_widget (GtkWidget *parent, const gchar *name, cons
 
 GtkWidget *create_scale (GtkWidget *parent, const gchar *name, gint value, gint min, gint max)
 {
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     GtkWidget *scale = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (value, min, max, 0, 0, 0)));
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 
     g_object_ref (scale);
     g_object_set_data_full (G_OBJECT (parent), name, scale, g_object_unref);
