@@ -2,7 +2,7 @@
  * @file libgcmd.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include <libgcmd/libgcmd-widget-factory.h>
 #include <libgcmd/gnome-cmd-state.h>
 #include <libgcmd/gnome-cmd-plugin.h>
-#include <libgcmd/gnome-cmd-file-info.h>
+#include <libgcmd/gnome-cmd-file-base.h>
 #include <libgcmd/gnome-cmd-dialog.h>
 #include <libgcmd/gnome-cmd-string-dialog.h>
 #include <libgcmd/plugin-info.h>
