@@ -2,7 +2,7 @@
  * @file gnome-cmd-advrename-dialog.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -186,7 +186,7 @@ void GnomeCmdAdvrenameDialog::Private::manage_profiles(GnomeCmdAdvrenameDialog::
 
     if (GnomeCmd::ManageProfilesDialog<GnomeCmdData::AdvrenameConfig,GnomeCmdData::AdvrenameConfig::Profile,GnomeCmdAdvrenameProfileComponent> (GTK_WINDOW (dialog),cfg,new_profile,_("Profiles"),"gnome-commander-advanced-rename"))
     {
-        GtkWidget *menu = widget->parent;
+        GtkWidget *menu = gtk_widget_get_parent(widget);
 
         gnome_cmd_button_menu_disconnect_handler (priv->profile_menu_button, menu);
         g_object_unref (gtk_item_factory_from_widget (menu));
@@ -439,7 +439,7 @@ void GnomeCmdAdvrenameDialog::Private::on_dialog_size_allocate (GtkWidget *widge
 void GnomeCmdAdvrenameDialog::Private::on_dialog_response (GnomeCmdAdvrenameDialog *dialog, int response_id, gpointer unused)
 {
     GtkTreeIter i;
-    gchar *old_focused_file_name = NULL;
+    const gchar *old_focused_file_name = NULL;
     gchar *new_focused_file_name = NULL;
 
     switch (response_id)
@@ -459,22 +459,21 @@ void GnomeCmdAdvrenameDialog::Private::on_dialog_response (GnomeCmdAdvrenameDial
                                     COL_NEW_NAME, &new_name,
                                     -1);
 
-                gchar *old_name = g_strdup (f->info->name);
-                GnomeVFSResult result = GNOME_VFS_OK;
+                gboolean result = FALSE;
 
-                if (strcmp (f->info->name, new_name) != 0)
-                    result = f->rename(new_name);
+                if (strcmp (g_file_info_get_display_name(f->gFileInfo), new_name) != 0)
+                    result = f->rename(new_name, nullptr);
 
                 gtk_list_store_set (GTK_LIST_STORE (dialog->files), &i,
                                     COL_NAME, f->get_name(),
-                                    COL_RENAME_FAILED, result!=GNOME_VFS_OK,
+                                    COL_RENAME_FAILED, !result,
                                     -1);
 
-                if (!new_focused_file_name && (result == GNOME_VFS_OK) && !strcmp(old_focused_file_name, old_name))
+                if (!new_focused_file_name && result
+                    && !strcmp(old_focused_file_name, g_file_info_get_display_name(f->gFileInfo)))
                     new_focused_file_name = g_strdup(new_name);
 
                 g_free (new_name);
-                g_free (old_name);
             }
             if (new_focused_file_name)
             {
@@ -524,11 +523,11 @@ static void gnome_cmd_advrename_dialog_init (GnomeCmdAdvrenameDialog *dialog)
     gtk_window_set_resizable (*dialog, TRUE);
     gtk_dialog_set_has_separator (*dialog, FALSE);
     gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-    gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+    gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dialog))), 2);
 
     GtkWidget *vbox = dialog->priv->vbox = gtk_vbox_new (FALSE, 6);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG (dialog))), vbox, TRUE, TRUE, 0);
 
     // Results
     gchar *str = g_strdup_printf ("<b>%s</b>", _("Results"));
