@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-chmod-dialog.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ struct GnomeCmdChmodDialogPrivate
 {
     GList *files;
     GnomeCmdFile *f;
-    GnomeVFSFilePermissions perms;
+    guint32 permissions;
 
     GtkWidget *chmod_component;
     GtkWidget *recurse_check;
@@ -61,23 +61,31 @@ struct GnomeCmdChmodDialogPrivate
 G_DEFINE_TYPE (GnomeCmdChmodDialog, gnome_cmd_chmod_dialog, GNOME_CMD_TYPE_DIALOG)
 
 
-static void do_chmod (GnomeCmdFile *in, GnomeVFSFilePermissions perm, gboolean recursive, ChmodRecursiveMode mode)
+static void do_chmod (GnomeCmdFile *in, guint32 permissions, gboolean recursive, ChmodRecursiveMode mode)
 {
     g_return_if_fail (in != NULL);
-    g_return_if_fail (in->info != NULL);
+    GError *error = nullptr;
 
-    if (!(recursive && mode == CHMOD_DIRS_ONLY && in->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY))
+    if (!(recursive
+          && mode == CHMOD_DIRS_ONLY
+          && in->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) != G_FILE_TYPE_DIRECTORY))
     {
-        GnomeVFSResult ret = in->chmod(perm);
+        if (!in->chmod(permissions, &error))
+        {
+            auto message = g_strdup_printf (_("Could not chmod %s"), in->get_name());
+            gnome_cmd_show_message (nullptr, message, error->message);
+            g_error_free(error);
+            g_free(message);
+            return;
+        }
 
-        if (ret != GNOME_VFS_OK)
-            gnome_cmd_show_message (NULL, in->get_name(), gnome_vfs_result_to_string (ret));
-        else
-            if (!recursive)
-                return;
+        if (!recursive)
+        {
+            return;
+        }
     }
 
-    if (in->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+    if (in->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
     {
         GnomeCmdDir *dir = gnome_cmd_dir_ref (GNOME_CMD_DIR (in));
 
@@ -86,11 +94,13 @@ static void do_chmod (GnomeCmdFile *in, GnomeVFSFilePermissions perm, gboolean r
         for (GList *i = gnome_cmd_dir_get_files (dir); i; i = i->next)
         {
             GnomeCmdFile *f = (GnomeCmdFile *) i->data;
-            if (!f->is_dotdot && strcmp (f->info->name, ".") != 0
-                && !GNOME_VFS_FILE_INFO_SYMLINK(f->info))
+            auto filename = get_gfile_attribute_string(f->gFile, G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME);
+            if (!f->is_dotdot && strcmp (filename, ".") != 0
+                && !g_file_info_get_is_symlink(f->gFileInfo))
             {
-                do_chmod (f, perm, TRUE, mode);
+                do_chmod (f, permissions, TRUE, mode);
             }
+            g_free(filename);
         }
         gnome_cmd_dir_unref (dir);
     }
@@ -104,10 +114,11 @@ inline void do_chmod_files (GnomeCmdChmodDialog *dialog)
         GnomeCmdFile *f = (GnomeCmdFile *) i->data;
         gboolean recursive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->recurse_check));
         const gchar *mode_text = get_combo_text (dialog->priv->recurse_combo);
+        //ToDo: This needs a fix. It does not work with Gcmd working in non-english language.
         ChmodRecursiveMode mode = strcmp (mode_text, recurse_opts[CHMOD_ALL_FILES]) == 0 ? CHMOD_ALL_FILES :
                                                                                            CHMOD_DIRS_ONLY;
 
-        do_chmod (f, dialog->priv->perms, recursive, mode);
+        do_chmod (f, dialog->priv->permissions, recursive, mode);
         view_refresh (NULL, NULL);
     }
 }
@@ -115,7 +126,7 @@ inline void do_chmod_files (GnomeCmdChmodDialog *dialog)
 
 inline void show_perms (GnomeCmdChmodDialog *dialog)
 {
-    gnome_cmd_chmod_component_set_perms (GNOME_CMD_CHMOD_COMPONENT (dialog->priv->chmod_component), dialog->priv->perms);
+    gnome_cmd_chmod_component_set_perms (GNOME_CMD_CHMOD_COMPONENT (dialog->priv->chmod_component), dialog->priv->permissions);
 }
 
 
@@ -142,7 +153,7 @@ static void on_toggle_recurse (GtkToggleButton *togglebutton, GnomeCmdChmodDialo
 
 static void on_perms_changed (GnomeCmdChmodComponent *component, GnomeCmdChmodDialog *dialog)
 {
-    dialog->priv->perms =
+    dialog->priv->permissions =
         gnome_cmd_chmod_component_get_perms (GNOME_CMD_CHMOD_COMPONENT (dialog->priv->chmod_component));
 }
 
@@ -181,7 +192,7 @@ static void gnome_cmd_chmod_dialog_init (GnomeCmdChmodDialog *dialog)
     gnome_cmd_dialog_add_expanding_category (GNOME_CMD_DIALOG (dialog), vbox);
 
 
-    dialog->priv->chmod_component = gnome_cmd_chmod_component_new ((GnomeVFSFilePermissions) 0);
+    dialog->priv->chmod_component = gnome_cmd_chmod_component_new (0);
     g_object_ref (dialog->priv->chmod_component);
     g_object_set_data_full (G_OBJECT (chmod_dialog), "chmod_component", dialog->priv->chmod_component, g_object_unref);
     gtk_widget_show (dialog->priv->chmod_component);
@@ -223,7 +234,7 @@ GtkWidget *gnome_cmd_chmod_dialog_new (GList *files)
     dialog->priv->f = (GnomeCmdFile *) dialog->priv->files->data;
     g_return_val_if_fail (dialog->priv->f != NULL, NULL);
 
-    dialog->priv->perms = dialog->priv->f->info->permissions;
+    dialog->priv->permissions = get_gfile_attribute_uint32(dialog->priv->f->gFile, G_FILE_ATTRIBUTE_UNIX_MODE) & 0xFFF;
 
     show_perms (dialog);
 
