@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-con-dialog.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "gnome-cmd-data.h"
 #include "utils.h"
 #include "dialogs/gnome-cmd-con-dialog.h"
+#include "gnome-cmd-plain-path.h"
 
 using namespace std;
 
@@ -62,8 +63,6 @@ struct GnomeCmdConnectDialog::Private
     string *alias {nullptr};
     string uri_str;
 
-    GnomeCmdCon::Authentication auth {GnomeCmdCon::SAVE_PERMANENTLY};
-
     GtkWidget *required_table {nullptr};
     GtkWidget *optional_table {nullptr};
 
@@ -72,11 +71,9 @@ struct GnomeCmdConnectDialog::Private
     GtkWidget *alias_entry;
     GtkWidget *uri_entry;
     GtkWidget *server_entry;
-    GtkWidget *share_entry;
     GtkWidget *port_entry;
     GtkWidget *folder_entry;
     GtkWidget *domain_entry;
-    GtkWidget *user_entry;
 
     Private();
     ~Private();
@@ -91,29 +88,23 @@ inline GnomeCmdConnectDialog::Private::Private()
     alias_entry = gtk_entry_new ();
     uri_entry = gtk_entry_new ();
     server_entry = gtk_entry_new ();
-    share_entry = gtk_entry_new ();
     port_entry = gtk_entry_new ();
     folder_entry = gtk_entry_new ();
     domain_entry = gtk_entry_new ();
-    user_entry = gtk_entry_new ();
 
     gtk_entry_set_activates_default (GTK_ENTRY (alias_entry), TRUE);
     gtk_entry_set_activates_default (GTK_ENTRY (uri_entry), TRUE);
     gtk_entry_set_activates_default (GTK_ENTRY (server_entry), TRUE);
-    gtk_entry_set_activates_default (GTK_ENTRY (share_entry), TRUE);
     gtk_entry_set_activates_default (GTK_ENTRY (port_entry), TRUE);
     gtk_entry_set_activates_default (GTK_ENTRY (folder_entry), TRUE);
-    gtk_entry_set_activates_default (GTK_ENTRY (user_entry), TRUE);
 
     // We need an extra ref so we can remove them from the table
     g_object_ref (alias_entry);
     g_object_ref (uri_entry);
     g_object_ref (server_entry);
-    g_object_ref (share_entry);
     g_object_ref (port_entry);
     g_object_ref (folder_entry);
     g_object_ref (domain_entry);
-    g_object_ref (user_entry);
 }
 
 
@@ -122,11 +113,9 @@ inline GnomeCmdConnectDialog::Private::~Private()
     g_object_unref (alias_entry);
     g_object_unref (uri_entry);
     g_object_unref (server_entry);
-    g_object_unref (share_entry);
     g_object_unref (port_entry);
     g_object_unref (folder_entry);
     g_object_unref (domain_entry);
-    g_object_unref (user_entry);
 
     delete alias;
 }
@@ -145,18 +134,12 @@ void GnomeCmdConnectDialog::Private::setup_for_type()
     if (server_entry->parent)
         gtk_container_remove (GTK_CONTAINER (required_table), server_entry);
 
-    if (share_entry->parent)
-        gtk_container_remove (GTK_CONTAINER (optional_table), share_entry);
-
     if (port_entry->parent)
         gtk_container_remove (GTK_CONTAINER (optional_table), port_entry);
 
     if (folder_entry->parent)
         gtk_container_remove (GTK_CONTAINER (optional_table), folder_entry);
 
-    if (user_entry->parent)
-        gtk_container_remove (GTK_CONTAINER (optional_table), user_entry);
-
     if (domain_entry->parent)
         gtk_container_remove (GTK_CONTAINER (optional_table), domain_entry);
 
@@ -166,7 +149,7 @@ void GnomeCmdConnectDialog::Private::setup_for_type()
     gint i = 1;
     GtkWidget *table = required_table;
 
-    gboolean show_share, show_port, show_user, show_domain;
+    gboolean show_port, show_domain;
 
     show_entry (table, alias_entry, _("_Alias:"), i);
 
@@ -181,24 +164,18 @@ void GnomeCmdConnectDialog::Private::setup_for_type()
         case CON_FTP:
         case CON_DAV:
         case CON_DAVS:
-            show_share = FALSE;
             show_port = TRUE;
-            show_user = TRUE;
             show_domain = FALSE;
             break;
 
        case CON_ANON_FTP:
-            show_share = FALSE;
             show_port = TRUE;
-            show_user = FALSE;
             show_domain = FALSE;
             break;
 
 #ifdef HAVE_SAMBA
         case CON_SMB:
-            show_share = TRUE;
             show_port = FALSE;
-            show_user = TRUE;
             show_domain = TRUE;
             break;
 #endif
@@ -238,17 +215,11 @@ void GnomeCmdConnectDialog::Private::setup_for_type()
 
     i = 0;
 
-    if (show_share)
-        show_entry (table, share_entry, _("S_hare:"), i);
-
     if (show_port)
         show_entry (table, port_entry, _("_Port:"), i);
 
     show_entry (table, folder_entry, _("_Folder:"), i);
 
-    if (show_user)
-        show_entry (table, user_entry, _("_User name:"), i);
-
     if (show_domain)
         show_entry (table, domain_entry, _("_Domain name:"), i);
 }
@@ -268,17 +239,16 @@ inline void GnomeCmdConnectDialog::Private::show_entry(GtkWidget *table, GtkWidg
     ++i;
 }
 
-
+/**
+ * This method sets priv->uri_str to the URI created from values in the dialog if the URI is valid.
+ */
 gboolean GnomeCmdConnectDialog::verify_uri()
 {
     string uri;
     string server;
-    string share;
     string port;
     string folder;
     string domain;
-    string user;
-    string password;
 
     if (priv->uri_entry->parent)
         stringify (uri, gtk_editable_get_chars (GTK_EDITABLE (priv->uri_entry), 0, -1));
@@ -286,9 +256,6 @@ gboolean GnomeCmdConnectDialog::verify_uri()
     if (priv->server_entry->parent)
         stringify (server, gtk_editable_get_chars (GTK_EDITABLE (priv->server_entry), 0, -1));
 
-    if (priv->share_entry->parent)
-        stringify (share, gtk_editable_get_chars (GTK_EDITABLE (priv->share_entry), 0, -1));
-
     if (priv->port_entry->parent)
         stringify (port, gtk_editable_get_chars (GTK_EDITABLE (priv->port_entry), 0, -1));
 
@@ -298,9 +265,6 @@ gboolean GnomeCmdConnectDialog::verify_uri()
     if (priv->domain_entry->parent)
         stringify (domain, gtk_editable_get_chars (GTK_EDITABLE (priv->domain_entry), 0, -1));
 
-    if (priv->user_entry->parent)
-        stringify (user, gtk_editable_get_chars (GTK_EDITABLE (priv->user_entry), 0, -1));
-
     int type = gtk_combo_box_get_active (GTK_COMBO_BOX (priv->type_combo));
 
     if (type!=CON_URI && server.empty())
@@ -310,14 +274,9 @@ gboolean GnomeCmdConnectDialog::verify_uri()
         return FALSE;
     }
 
-    priv->auth = type==CON_ANON_FTP ? GnomeCmdCon::NOT_REQUIRED : GnomeCmdCon::SAVE_PERMANENTLY;
+    gnome_cmd_con_make_uri (uri, (ConnectionMethodID) type, uri, server, port, folder, domain);
 
-    if (type==CON_ANON_FTP)
-        user = "anonymous";
-
-    gnome_cmd_con_make_uri (uri, (ConnectionMethodID) type, priv->auth==GnomeCmdCon::SAVE_PERMANENTLY, uri, server, share, port, folder, domain, user, password);
-
-    if (type==CON_URI && uri.empty())
+    if (type==CON_URI && !uri_is_valid(uri.c_str()))
     {
         gnome_cmd_show_message (*this,
                                 stringify(g_strdup_printf (_("“%s” is not a valid location"), uri.c_str())),
@@ -512,8 +471,6 @@ GnomeCmdConRemote *gnome_cmd_connect_dialog_new (gboolean has_alias)
 
     gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->type_combo), CON_SSH);
 
-    dialog->priv->auth = GnomeCmdCon::SAVE_PERMANENTLY;
-
     gint response = gtk_dialog_run (*dialog);
 
     GnomeCmdConRemote *server = nullptr;
@@ -527,7 +484,6 @@ GnomeCmdConRemote *gnome_cmd_connect_dialog_new (gboolean has_alias)
         GnomeCmdCon *con = GNOME_CMD_CON (server);
 
         con->method = (ConnectionMethodID) gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->priv->type_combo));
-        con->auth = dialog->priv->auth;
     }
 
     gtk_widget_destroy (*dialog);
@@ -541,15 +497,12 @@ gboolean gnome_cmd_connect_dialog_edit (GnomeCmdConRemote *server)
     g_return_val_if_fail (server != nullptr, FALSE);
 
     auto *dialog = reinterpret_cast<GnomeCmdConnectDialog*> (gtk_widget_new (GNOME_CMD_TYPE_CONNECT_DIALOG, nullptr));
-
     g_return_val_if_fail (dialog != nullptr, FALSE);
 
     auto *con = GNOME_CMD_CON (server);
 
     gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->priv->type_combo), con->method);
 
-    dialog->priv->auth = con->auth;
-
     if (con->alias)
     {
         dialog->priv->alias = new string(con->alias);
@@ -558,83 +511,84 @@ gboolean gnome_cmd_connect_dialog_edit (GnomeCmdConRemote *server)
     else
         gtk_widget_set_sensitive (dialog->priv->alias_entry, FALSE);
 
+     auto host = g_strdup(gnome_cmd_con_get_host_name(con));
+     gint port = gnome_cmd_con_get_port (con);
+     auto path = gnome_cmd_con_get_root_path(con);
+
     if (con->uri)
     {
         dialog->priv->uri_str = con->uri;
 
-        GnomeVFSURI *uri = gnome_vfs_uri_new (con->uri);
+        gtk_entry_set_text (GTK_ENTRY (dialog->priv->uri_entry), con->uri);
 
-        if (uri)
-        {
-            gtk_entry_set_text (GTK_ENTRY (dialog->priv->uri_entry), con->uri);
+        gtk_entry_set_text (GTK_ENTRY (dialog->priv->server_entry), host);
 
-            gtk_entry_set_text (GTK_ENTRY (dialog->priv->server_entry), gnome_vfs_uri_get_host_name (uri));
+        if (path)
+            gtk_entry_set_text (GTK_ENTRY (dialog->priv->folder_entry), path);
 
-            const gchar *path = gnome_vfs_uri_get_path (uri);
-            const gchar *user_name = gnome_vfs_uri_get_user_name (uri);
-            guint port = gnome_vfs_uri_get_host_port (uri);
-
-#ifdef HAVE_SAMBA
-            if (con->method==CON_SMB)
-            {
-                gchar **a = g_strsplit (path, "/", 3);
-
-                if (g_strv_length (a) > 2)
-                {
-                    gtk_entry_set_text (GTK_ENTRY (dialog->priv->share_entry), a[1]);
-                    gtk_entry_set_text (GTK_ENTRY (dialog->priv->folder_entry), a[2]);
-                }
-                else
-                    gtk_entry_set_text (GTK_ENTRY (dialog->priv->folder_entry), path);
-
-                g_strfreev (a);
-
-                a = g_strsplit (user_name, ";", 2);
-
-                if (g_strv_length (a) > 1)
-                {
-                    gtk_entry_set_text (GTK_ENTRY (dialog->priv->domain_entry), a[0]);
-                    gtk_entry_set_text (GTK_ENTRY (dialog->priv->user_entry), a[1]);
-                }
-                else
-                    gtk_entry_set_text (GTK_ENTRY (dialog->priv->user_entry), user_name);
-
-                g_strfreev (a);
-            }
-            else
-            {
-#endif
-                gtk_entry_set_text (GTK_ENTRY (dialog->priv->folder_entry), path);
-                gtk_entry_set_text (GTK_ENTRY (dialog->priv->user_entry), user_name);
-#ifdef HAVE_SAMBA
-            }
-#endif
-
-            if (port)
-                gtk_entry_set_text (GTK_ENTRY (dialog->priv->port_entry), stringify(port).c_str());
-
-            gnome_vfs_uri_unref (uri);
-        }
+        if (port != -1)
+            gtk_entry_set_text (GTK_ENTRY (dialog->priv->port_entry), stringify(port).c_str());
     }
 
     gint response = gtk_dialog_run (*dialog);
 
     if (response == GTK_RESPONSE_OK)
     {
-        GnomeVFSURI *uri = gnome_vfs_uri_new (dialog->priv->uri_str.c_str());
+        g_free(host);
+        GError *error = nullptr;
 
-        const gchar *alias = dialog->priv->alias ? dialog->priv->alias->c_str() : nullptr;
-        const gchar *host = gnome_vfs_uri_get_host_name (uri);
+        if (dialog->priv->uri_str.c_str())
+        {
+            auto uri = g_uri_parse(dialog->priv->uri_str.c_str(), G_URI_FLAGS_NONE, &error);
+            if (error)
+            {
+                g_warning("gnome_cmd_connect_dialog_edit - g_uri_parse error: %s", error->message);
+                g_error_free(error);
+                return FALSE;
+            }
+            auto uriScheme = g_uri_get_scheme(uri);
+            auto uriHost = g_uri_get_host(uri);
+            auto uriPath = g_uri_get_path(uri);
+            auto uriPort = g_uri_get_port(uri);
+            auto uriString = g_uri_to_string(uri);
+            gnome_cmd_con_set_uri (con, uriString);
+            g_free(uriString);
 
+            gnome_cmd_con_set_scheme(con, uriScheme);
+            gnome_cmd_con_set_base_path(con, uriPath && strlen(uriPath) > 0
+                ? new GnomeCmdPlainPath(uriPath)
+                : new GnomeCmdPlainPath(G_DIR_SEPARATOR_S));
+            gnome_cmd_con_set_root_path(con, uriPath);
+            gnome_cmd_con_set_host_name (con, uriHost);
+            if (uriPort != -1)
+                gnome_cmd_con_set_port(con, uriPort);
+            // let GIO handle the port of the SMB connection
+            if (!strcmp(uriScheme, "smb") && uriPort != -1)
+                gnome_cmd_con_set_port(con, -1);
+        }
+        else
+        {
+            host = g_strdup(gtk_entry_get_text (GTK_ENTRY (dialog->priv->server_entry)));
+            path = g_strdup(gtk_entry_get_text (GTK_ENTRY (dialog->priv->folder_entry)));
+            auto portChar = gtk_entry_get_text (GTK_ENTRY (dialog->priv->port_entry));
+            port = portChar ? atoi(portChar) : -1;
+
+            gnome_cmd_con_set_base_path(con, path && strlen(path) > 0
+                ? new GnomeCmdPlainPath(path)
+                : new GnomeCmdPlainPath(G_DIR_SEPARATOR_S));
+            gnome_cmd_con_set_root_path(con, path);
+            gnome_cmd_con_set_host_name (con, host);
+            if (port != -1)
+                gnome_cmd_con_set_port(con, port);
+
+            g_free(host);
+            g_free(path);
+        }
+
+        auto alias = dialog->priv->alias ? dialog->priv->alias->c_str() : nullptr;
         gnome_cmd_con_set_alias (con, alias);
-        gnome_cmd_con_set_uri (con, dialog->priv->uri_str);
-        gnome_cmd_con_set_host_name (con, host);
         con->method = (ConnectionMethodID) gtk_combo_box_get_active (GTK_COMBO_BOX (dialog->priv->type_combo));
-        con->auth = dialog->priv->auth;
-
-        gnome_cmd_con_remote_set_host_name (server, host);
-
-        gnome_vfs_uri_unref (uri);
+        gnome_cmd_con_remote_set_tooltips (server, host);
     }
 
     gtk_widget_destroy (*dialog);
