@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-mkdir-dialog.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,5 +22,8 @@
 #pragma once
 
 #include "gnome-cmd-dir.h"
+using namespace std;
+
+GSList *make_gfile_list (GnomeCmdDir *dir, string filename);
 
 gboolean gnome_cmd_mkdir_dialog_new (GnomeCmdDir *dir, GnomeCmdFile *selected_file=NULL);
