@@ -2,7 +2,7 @@
  * @file gnome-cmd-prepare-xfer-dialog.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 using namespace std;
 
 
-static GnomeCmdDialogClass *parent_class = NULL;
+static GnomeCmdDialogClass *parent_class = nullptr;
 
 
 inline gboolean con_device_has_path (FileSelectorID fsID, GnomeCmdCon *&dev, const gchar *user_path)
@@ -41,7 +41,7 @@ inline gboolean con_device_has_path (FileSelectorID fsID, GnomeCmdCon *&dev, con
     dev = main_win->fs(fsID)->get_connection();
 
     return GNOME_CMD_IS_CON_DEVICE (dev) &&
-           g_str_has_prefix (user_path, gnome_cmd_con_device_get_mountp (GNOME_CMD_CON_DEVICE (dev)));
+           g_str_has_prefix (user_path, gnome_cmd_con_device_get_mountp_string (GNOME_CMD_CON_DEVICE (dev)));
 }
 
 
@@ -50,12 +50,15 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
     GnomeCmdCon *con = gnome_cmd_dir_get_connection (dialog->default_dest_dir);
     gchar *user_path = g_strstrip (g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->dest_dir_entry))));
 
-    gchar *dest_path = NULL;
-    gchar *dest_fn = NULL;
-    gint user_path_len;
-
     if (!user_path)
-        goto bailout;
+    {
+        gtk_widget_destroy (GTK_WIDGET (dialog));
+        return;
+    }
+
+    gchar *dest_path = nullptr;
+    gchar *dest_fn = nullptr;
+    gint user_path_len;
 
     user_path_len = strlen (user_path);
     dest_path = user_path;
@@ -74,7 +77,7 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
             // if LEFT or RIGHT device (connection) points to user_path than adjust user_path and set con to the found device
             if (con_device_has_path (INACTIVE, dev, user_path) || con_device_has_path (ACTIVE, dev, user_path))
             {
-                dest_path = g_strdup (user_path + strlen (gnome_cmd_con_device_get_mountp (GNOME_CMD_CON_DEVICE (dev))));
+                dest_path = g_strdup (user_path + strlen (gnome_cmd_con_device_get_mountp_string (GNOME_CMD_CON_DEVICE (dev))));
                 con = dev;
             }
             else    // otherwise connection not present in any pane, use home connection instead
@@ -86,38 +89,45 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
         if (!gnome_cmd_dir_is_local (dialog->default_dest_dir))
         {
             const gchar *t = gnome_cmd_dir_get_path (dialog->default_dest_dir)->get_path();
-            dest_path = g_build_filename (t, user_path, NULL);
+            auto destPathTmp = g_build_filename (t, user_path, nullptr);
+            if (*destPathTmp != '/')
+            {
+                dest_path = g_strdup_printf("%s%s", G_DIR_SEPARATOR_S, destPathTmp);
+                g_free(destPathTmp);
+            }
+            else
+            {
+                dest_path = destPathTmp;
+            }
+
         }
         else
         {
-            gchar *t = GNOME_CMD_FILE (dialog->src_fs->get_directory())->get_path();
-            dest_path = g_build_filename (t, user_path, NULL);
+            gchar *t = GNOME_CMD_FILE (dialog->src_fs->get_directory())->GetPathStringThroughParent();
+            dest_path = g_build_filename (t, user_path, nullptr);
             g_free (t);
         }
         g_free (user_path);
     }
 
     // Check if something exists at the given path and find out what it is
-    GnomeVFSFileType type;
-    GnomeVFSResult   res;
+    GFileType gFileType;
+    gboolean  fileExists;
 
-    res = gnome_cmd_con_get_path_target_type (con, dest_path, &type);
-
-    if (res != GNOME_VFS_OK && res != GNOME_VFS_ERROR_NOT_FOUND)
-        goto bailout;
+    fileExists = gnome_cmd_con_get_path_target_type (con, dest_path, &gFileType);
 
     if (g_list_length (dialog->src_files) == 1)
     {
-        GnomeCmdFile *f = GNOME_CMD_FILE (dialog->src_files->data);
+        GnomeCmdFile *gFile = GNOME_CMD_FILE (dialog->src_files->data);
 
-        if (res == GNOME_VFS_OK && type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+        if (fileExists && gFileType == G_FILE_TYPE_DIRECTORY)
         {
             // There exists a directory, copy into it using the original filename
             dest_dir = gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, dest_path));
-            dest_fn = g_strdup (f->get_name());
+            dest_fn = g_strdup (gFile->get_name());
         }
         else
-            if (res == GNOME_VFS_OK)
+            if (fileExists)
             {
                 // There exists something else, asume that the user wants to overwrite it for now
                 gchar *t = g_path_get_dirname (dest_path);
@@ -129,8 +139,8 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
             {
                 // Nothing found, check if the parent dir exists
                 gchar *parent_dir = g_path_get_dirname (dest_path);
-                res = gnome_cmd_con_get_path_target_type (con, parent_dir, &type);
-                if (res == GNOME_VFS_OK && type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+                fileExists = gnome_cmd_con_get_path_target_type (con, parent_dir, &gFileType);
+                if (fileExists && gFileType == G_FILE_TYPE_DIRECTORY)
                 {
                     // yup, xfer to it
                     dest_dir = gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, parent_dir));
@@ -138,11 +148,13 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
                     dest_fn = g_path_get_basename (dest_path);
                 }
                 else
-                    if (res == GNOME_VFS_OK)
+                    if (fileExists)
                     {
                         // the parent dir was a file, abort!
                         g_free (parent_dir);
-                        goto bailout;
+                        g_free (dest_path);
+                        gtk_widget_destroy (GTK_WIDGET (dialog));
+                        return;
                     }
                     else
                     {
@@ -150,20 +162,27 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
                         gchar *msg = g_strdup_printf (_("The directory “%s” doesn’t exist, do you want to create it?"),
                                                       g_path_get_basename (parent_dir));
                         gint choice = run_simple_dialog (GTK_WIDGET (dialog), TRUE, GTK_MESSAGE_QUESTION, msg, "",
-                                                         -1, _("No"), _("Yes"), NULL);
+                                                         -1, _("No"), _("Yes"), nullptr);
                         g_free (msg);
 
                         if (choice == 1)
                         {
-                            GnomeVFSResult mkdir_result = gnome_cmd_con_mkdir (con, parent_dir);
-                            if (mkdir_result != GNOME_VFS_OK)
+                            GError *error = nullptr;
+                            if (!gnome_cmd_con_mkdir (con, parent_dir, error))
                             {
-                                gnome_cmd_show_message (*main_win, gnome_vfs_result_to_string (mkdir_result));
-                                goto bailout;
+                                gnome_cmd_show_message (*main_win, error ? error->message : _("Unknown error"));
+                                g_error_free(error);
+                                g_free (dest_path);
+                                gtk_widget_destroy (GTK_WIDGET (dialog));
+                                return;
                             }
                         }
                         else
-                            goto bailout;
+                        {
+                            g_free (dest_path);
+                            gtk_widget_destroy (GTK_WIDGET (dialog));
+                            return;
+                        }
 
                         dest_dir = gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, parent_dir));
                         g_free (parent_dir);
@@ -173,16 +192,18 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
     }
     else
     {
-        if (res == GNOME_VFS_OK && type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+        if (fileExists && gFileType == G_FILE_TYPE_DIRECTORY)
         {
             // There exists a directory, copy to it
             dest_dir = gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, dest_path));
         }
         else
-            if (res == GNOME_VFS_OK)
+            if (fileExists)
             {
                 // There exists something which is not a directory, abort!
-                goto bailout;
+                g_free (dest_path);
+                gtk_widget_destroy (GTK_WIDGET (dialog));
+                return;
             }
             else
             {
@@ -201,39 +222,66 @@ static void on_ok (GtkButton *button, GnomeCmdPrepareXferDialog *dialog)
 
                 if (choice == GTK_RESPONSE_OK)
                 {
-                    GnomeVFSResult mkdir_result = gnome_cmd_con_mkdir (con, dest_path);
-                    if (mkdir_result != GNOME_VFS_OK)
+                    GError *error = nullptr;
+                    if (!gnome_cmd_con_mkdir (con, dest_path, error))
                     {
-                        gnome_cmd_show_message (*main_win, gnome_vfs_result_to_string (mkdir_result));
-                        goto bailout;
+                        gnome_cmd_show_message (*main_win, error->message);
+                        g_error_free(error);
+                        g_free (dest_path);
+                        gtk_widget_destroy (GTK_WIDGET (dialog));
+                        return;
                     }
                 }
                 else
-                    goto bailout;
-
+                {
+                    g_free (dest_path);
+                    gtk_widget_destroy (GTK_WIDGET (dialog));
+                    return;
+                }
                 dest_dir = gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, dest_path));
             }
     }
 
     if (!GNOME_CMD_IS_DIR (dest_dir))
-        goto bailout;
+    {
+        g_free (dest_path);
+        gtk_widget_destroy (GTK_WIDGET (dialog));
+        return;
+    }
 
     if (g_list_length (dialog->src_files) == 1)
-        DEBUG ('x', "Starting xfer the file '%s' to '%s'\n", dest_fn, GNOME_CMD_FILE (dest_dir)->get_real_path());
+        DEBUG ('x', "Starting xfer the file '%s' to '%s'\n", dest_fn, GNOME_CMD_FILE (dest_dir)->get_uri_str());
     else
-        DEBUG ('x', "Starting xfer %d files to '%s'\n", g_list_length (dialog->src_files), GNOME_CMD_FILE (dest_dir)->get_real_path());
+        DEBUG ('x', "Starting xfer %d files to '%s'\n", g_list_length (dialog->src_files), GNOME_CMD_FILE (dest_dir)->get_uri_str());
 
 
     gnome_cmd_dir_ref (dest_dir);
-    gnome_cmd_xfer_start (dialog->src_files,
-                          dest_dir,
-                          dialog->src_fs->file_list(),
-                          dest_fn,
-                          dialog->xferOptions,
-                          dialog->xferOverwriteMode,
-                          NULL, NULL);
 
-bailout:
+    switch (dialog->gnomeCmdTransferType)
+    {
+        case COPY:
+            gnome_cmd_copy_start (dialog->src_files,
+                                  dest_dir,
+                                  dialog->src_fs->file_list(),
+                                  dest_fn,
+                                  dialog->gFileCopyFlags,
+                                  dialog->overwriteMode,
+                                  NULL, NULL);
+            break;
+        case MOVE:
+            gnome_cmd_move_start (dialog->src_files,
+                      dest_dir,
+                      dialog->src_fs->file_list(),
+                      dest_fn,
+                      dialog->gFileCopyFlags,
+                      dialog->overwriteMode,
+                      NULL, NULL);
+            break;
+        case LINK:
+        default:
+            break;
+    }
+
     g_free (dest_path);
     gtk_widget_destroy (GTK_WIDGET (dialog));
 }
@@ -373,21 +421,21 @@ GtkType gnome_cmd_prepare_xfer_dialog_get_type ()
 
 inline gboolean path_points_at_directory (GnomeCmdFileSelector *to, const gchar *dest_path)
 {
-    GnomeVFSFileType type;
+    GFileType type;
 
-    GnomeVFSResult res = gnome_cmd_con_get_path_target_type (to->get_connection(), dest_path, &type);
+    gboolean fileExists = gnome_cmd_con_get_path_target_type (to->get_connection(), dest_path, &type);
 
-    return res == GNOME_VFS_OK && type == GNOME_VFS_FILE_TYPE_DIRECTORY;
+    return fileExists && type == G_FILE_TYPE_DIRECTORY;
 }
 
 
 GtkWidget *gnome_cmd_prepare_xfer_dialog_new (GnomeCmdFileSelector *from, GnomeCmdFileSelector *to)
 {
-    g_return_val_if_fail (from!=NULL, NULL);
-    g_return_val_if_fail (to!=NULL, NULL);
+    g_return_val_if_fail (from!=nullptr, nullptr);
+    g_return_val_if_fail (to!=nullptr, nullptr);
 
-    gchar *dest_str = NULL;
-    GnomeCmdPrepareXferDialog *dialog = (GnomeCmdPrepareXferDialog *) g_object_new (GNOME_CMD_TYPE_PREPARE_XFER_DIALOG, NULL);
+    gchar *destString = nullptr;
+    GnomeCmdPrepareXferDialog *dialog = (GnomeCmdPrepareXferDialog *) g_object_new (GNOME_CMD_TYPE_PREPARE_XFER_DIALOG, nullptr);
 
     dialog->src_files = from->file_list()->get_selected_files();
     gnome_cmd_file_list_ref (dialog->src_files);
@@ -400,41 +448,40 @@ GtkWidget *gnome_cmd_prepare_xfer_dialog_new (GnomeCmdFileSelector *from, GnomeC
     {
         if (num_files == 1)
         {
-            GnomeCmdFile *f = (GnomeCmdFile *) dialog->src_files->data;
-            dest_str = get_utf8 (f->info->name);
+            auto gFile = static_cast<GnomeCmdFile*> (dialog->src_files->data);
+            destString = g_strdup (gFile->get_name());
         }
     }
     else
+    {
         if (num_files == 1)
         {
-            GnomeCmdFile *f = (GnomeCmdFile *) dialog->src_files->data;
+            auto gFile = static_cast<GnomeCmdFile*> (dialog->src_files->data);
 
-            gchar *t = GNOME_CMD_FILE (dialog->default_dest_dir)->get_real_path();
-            gchar *path = get_utf8 (t);
-            gchar *fname = get_utf8 (f->info->name);
-            g_free (t);
+            auto tempPathString = GNOME_CMD_FILE (dialog->default_dest_dir)->get_real_path();
+            auto path = get_utf8 (tempPathString);
+            g_free (tempPathString);
 
-            dest_str = g_build_filename (path, fname, NULL);
-            if (path_points_at_directory (to, dest_str))
+            destString = g_build_filename (path, gFile->get_name(), nullptr);
+            if (path_points_at_directory (to, destString))
             {
-                g_free (dest_str);
-                dest_str = g_strdup (path);
+                g_free (destString);
+                destString = g_strdup (path);
             }
 
             g_free (path);
-            g_free (fname);
         }
         else
         {
-            gchar *t = GNOME_CMD_FILE (dialog->default_dest_dir)->get_real_path();
-            dest_str = get_utf8 (t);
-            g_free (t);
+            auto tempPathString = GNOME_CMD_FILE (dialog->default_dest_dir)->get_real_path();
+            destString = get_utf8 (tempPathString);
+            g_free (tempPathString);
         }
-
-    if (dest_str)
+    }
+    if (destString)
     {
-        gtk_entry_set_text (GTK_ENTRY (dialog->dest_dir_entry), dest_str);
-        g_free (dest_str);
+        gtk_entry_set_text (GTK_ENTRY (dialog->dest_dir_entry), destString);
+        g_free (destString);
     }
 
     gtk_widget_grab_focus (GTK_WIDGET (dialog->dest_dir_entry));
