@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-prepare-xfer-dialog.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,8 +44,9 @@ struct GnomeCmdPrepareXferDialog
     GtkWidget *ok_button;
     GtkWidget *cancel_button;
 
-    GnomeVFSXferOptions xferOptions;
-    GnomeVFSXferOverwriteMode xferOverwriteMode;
+    GFileCopyFlags gFileCopyFlags;
+    GnomeCmdConfirmOverwriteMode overwriteMode;
+    GnomeCmdTransferType gnomeCmdTransferType;
 
     GList *src_files;
     GnomeCmdFileSelector *src_fs;
