@@ -2,7 +2,7 @@
  * @file gnome-cmd-rename-dialog.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,17 +59,21 @@ static gboolean on_dialog_keypressed (GtkWidget *widget, GdkEventKey *event, gpo
         case GDK_Return:
         case GDK_KP_Enter:
             {
+                GError *error = nullptr;
                 gchar *new_fname = g_strdup (gtk_entry_get_text (dialog->priv->textbox));
-                GnomeVFSResult result = dialog->priv->f->rename(new_fname);
+                gboolean result = dialog->priv->f->rename(new_fname, &error);
 
-                if (result==GNOME_VFS_OK)
+                if (result)
                     main_win->fs(ACTIVE)->file_list()->focus_file(new_fname, TRUE);
 
                 dialog->priv->f->unref();
                 gtk_widget_destroy (widget);
 
-                if (result!=GNOME_VFS_OK)
-                    gnome_cmd_show_message (NULL, new_fname, gnome_vfs_result_to_string (result));
+                if (!result)
+                {
+                    gnome_cmd_show_message (NULL, new_fname, error->message);
+                    g_error_free(error);
+                }
 
                 g_free (new_fname);
             }
@@ -144,16 +148,12 @@ GtkWidget *gnome_cmd_rename_dialog_new (GnomeCmdFile *f, gint x, gint y, gint wi
     gtk_container_add (GTK_CONTAINER (dialog), GTK_WIDGET (dialog->priv->textbox));
     gtk_widget_set_style (GTK_WIDGET (dialog->priv->textbox), list_style);
 
-    gchar *fname = get_utf8 (f->get_name());
-
-    gtk_entry_set_text (dialog->priv->textbox, fname);
+    gtk_entry_set_text (dialog->priv->textbox, f->get_name());
 
     gtk_window_set_focus (GTK_WINDOW (dialog), GTK_WIDGET (dialog->priv->textbox));
     gtk_widget_grab_focus (GTK_WIDGET (dialog->priv->textbox));
     gtk_editable_select_region (GTK_EDITABLE (dialog->priv->textbox), 0, -1);
     gtk_widget_show (GTK_WIDGET (dialog->priv->textbox));
 
-    g_free (fname);
-
     return GTK_WIDGET (dialog);
 }
