@@ -2,7 +2,7 @@
  * @file eggcellrendererkeys.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -136,9 +136,9 @@ marshal_VOID__STRING_UINT_FLAGS_UINT (GClosure     *closure,
                                                                int          arg_3,
                                                                guint        arg_4,
                                                                gpointer     data2);
-    register GMarshalFunc_VOID__STRING_UINT_FLAGS_UINT callback;
-    register GCClosure *cc = (GCClosure*) closure;
-    register gpointer data1, data2;
+    GMarshalFunc_VOID__STRING_UINT_FLAGS_UINT callback;
+    GCClosure *cc = (GCClosure*) closure;
+    gpointer data1, data2;
 
     if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -339,7 +339,14 @@ static void egg_cell_renderer_keys_get_property  (GObject *object, guint param_i
 {
     g_return_if_fail (EGG_IS_CELL_RENDERER_KEYS (object));
 
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     EggCellRendererKeys *keys = EGG_CELL_RENDERER_KEYS (object);
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 
     switch (param_id)
     {
@@ -366,7 +373,14 @@ static void egg_cell_renderer_keys_set_property (GObject *object, guint param_id
 {
     g_return_if_fail (EGG_IS_CELL_RENDERER_KEYS (object));
 
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     EggCellRendererKeys *keys = EGG_CELL_RENDERER_KEYS (object);
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 
     switch (param_id)
     {
@@ -418,7 +432,14 @@ void egg_cell_renderer_keys_get_size (GtkCellRenderer *cell,
                                       gint            *width,
                                       gint            *height)
 {
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     auto keys = reinterpret_cast <EggCellRendererKeys*> (cell);
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
     GtkRequisition requisition;
 
     if (keys->sizing_label == nullptr)
@@ -597,8 +618,15 @@ egg_cell_renderer_keys_start_editing (GtkCellRenderer      *cell,
     GtkWidget *label;
     GtkWidget *eventbox;
 
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-align"
+#endif
     GtkCellRendererText *celltext = GTK_CELL_RENDERER_TEXT (cell);
     EggCellRendererKeys *keys = EGG_CELL_RENDERER_KEYS (cell);
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
 
     // If the cell isn't editable we return NULL
     if (celltext->editable == FALSE)
