@@ -2,7 +2,7 @@
  * @file gnome-cmd-app.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-app.h"
+#include "imageloader.h"
 
 using namespace std;
 
@@ -52,7 +53,8 @@ GnomeCmdApp *gnome_cmd_app_new_with_values (const gchar *name,
                                             const gchar *pattern_string,
                                             gboolean handles_uris,
                                             gboolean handles_multiple,
-                                            gboolean requires_terminal)
+                                            gboolean requires_terminal,
+                                            GAppInfo *gAppInfo)
 {
     GnomeCmdApp *app = gnome_cmd_app_new ();
 
@@ -65,6 +67,7 @@ GnomeCmdApp *gnome_cmd_app_new_with_values (const gchar *name,
     gnome_cmd_app_set_handles_uris (app, handles_uris);
     gnome_cmd_app_set_handles_multiple (app, handles_multiple);
     gnome_cmd_app_set_requires_terminal (app, requires_terminal);
+    app->gAppInfo = gAppInfo;
 
     return app;
 }
@@ -119,23 +122,19 @@ static char* panel_find_icon (GtkIconTheme *icon_theme, const char *icon_name, g
 }
 
 
-GnomeCmdApp *gnome_cmd_app_new_from_vfs_app (GnomeVFSMimeApplication *vfs_app)
+GnomeCmdApp *gnome_cmd_app_new_from_app_info (GAppInfo *gAppInfo)
 {
-    g_return_val_if_fail (vfs_app != nullptr, nullptr);
+    g_return_val_if_fail (gAppInfo != nullptr, nullptr);
 
-    GtkIconTheme *theme = gtk_icon_theme_get_default ();
-    char *icon = panel_find_icon (theme, gnome_vfs_mime_application_get_icon (vfs_app), 16);
-    
-    GnomeCmdApp *rel_value = gnome_cmd_app_new_with_values (vfs_app->name,
-                                          vfs_app->command,
-                                          icon,
+    return gnome_cmd_app_new_with_values (g_app_info_get_name (gAppInfo),
+                                          g_app_info_get_commandline (gAppInfo),
+                                          get_default_application_icon_path(gAppInfo),
                                           APP_TARGET_ALL_FILES,
                                           nullptr,
-                                          vfs_app->expects_uris == GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS,
-                                          vfs_app->can_open_multiple_files,
-                                          vfs_app->requires_terminal);
-    g_free (icon);
-    return rel_value;
+                                          g_app_info_supports_uris (gAppInfo),
+                                          true,
+                                          false,
+                                          gAppInfo);
 }
 
 
@@ -148,7 +147,8 @@ GnomeCmdApp *gnome_cmd_app_dup (GnomeCmdApp *app)
                                           app->pattern_string,
                                           app->handles_uris,
                                           app->handles_multiple,
-                                          app->requires_terminal);
+                                          app->requires_terminal,
+                                          app->gAppInfo);
 }
 
 
