@@ -2,7 +2,7 @@
  * @file gnome-cmd-app.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@ struct GnomeCmdApp
     gchar *name;
     gchar *cmd;
     gchar *icon_path;
+    GAppInfo *gAppInfo;
 
     AppTarget target;
     gchar *pattern_string;
@@ -51,7 +52,7 @@ struct GnomeCmdApp
 
 GnomeCmdApp *gnome_cmd_app_new ();
 
-GnomeCmdApp *gnome_cmd_app_new_from_vfs_app (GnomeVFSMimeApplication *vfs_app);
+GnomeCmdApp *gnome_cmd_app_new_from_app_info (GAppInfo *gAppInfo);
 
 GnomeCmdApp *gnome_cmd_app_new_with_values (const gchar *name,
                                             const gchar *cmd,
@@ -60,7 +61,8 @@ GnomeCmdApp *gnome_cmd_app_new_with_values (const gchar *name,
                                             const gchar *pattern_string,
                                             gboolean handles_uris,
                                             gboolean handles_multiple,
-                                            gboolean requires_terminal);
+                                            gboolean requires_terminal,
+                                            GAppInfo *gAppInfo);
 
 GnomeCmdApp *gnome_cmd_app_dup (GnomeCmdApp *app);
 
