@@ -2,7 +2,7 @@
  * @file gnome-cmd-chmod-component.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,9 +49,9 @@ struct GnomeCmdChmodComponentClass
 };
 
 
-GtkWidget *gnome_cmd_chmod_component_new (GnomeVFSFilePermissions perms);
+GtkWidget *gnome_cmd_chmod_component_new (guint32 perms);
 
 GtkType gnome_cmd_chmod_component_get_type ();
 
-GnomeVFSFilePermissions gnome_cmd_chmod_component_get_perms (GnomeCmdChmodComponent *component);
-void gnome_cmd_chmod_component_set_perms (GnomeCmdChmodComponent *component, GnomeVFSFilePermissions perms);
+guint32 gnome_cmd_chmod_component_get_perms (GnomeCmdChmodComponent *component);
+void gnome_cmd_chmod_component_set_perms (GnomeCmdChmodComponent *component, guint32 permissions);
