@@ -2,7 +2,7 @@
  * @file gnome-cmd-con-device.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <libgnomevfs/gnome-vfs-volume.h>
 
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-data.h"
@@ -40,21 +39,20 @@ using namespace std;
 struct GnomeCmdConDevicePrivate
 {
     gchar *alias {nullptr};
-    gchar *device_fn {nullptr};
+    gchar *device_fn {nullptr}; // The device identifier (either a linux device string or a uuid)
     gchar *mountp {nullptr};
     gchar *icon_path {nullptr};
     gboolean autovolume;
-    GnomeVFSVolume *vfsvol;
+    GMount *gMount;
+    GVolume *gVolume;
 };
 
 
 static GnomeCmdConClass *parent_class = nullptr;
 
 
-static gboolean is_mounted (GnomeCmdCon *con)
+static gboolean is_mounted (GnomeCmdConDevice *dev_con)
 {
-    g_return_val_if_fail (GNOME_CMD_IS_CON_DEVICE (con), FALSE);
-
     FILE *fd = fopen ("/etc/mtab", "r");
 
     if (!fd)
@@ -62,7 +60,6 @@ static gboolean is_mounted (GnomeCmdCon *con)
 
     gchar tmp[512];
     gboolean ret = FALSE;
-    GnomeCmdConDevice *dev_con = GNOME_CMD_CON_DEVICE (con);
 
     gchar *s;
     while ((s=fgets (tmp, sizeof(tmp), fd))!=nullptr)
@@ -89,25 +86,31 @@ static gboolean is_mounted (GnomeCmdCon *con)
 }
 
 
-static void do_mount_thread_func (GnomeCmdCon *con)
+static void do_legacy_mount(GnomeCmdCon *con)
 {
     g_return_if_fail (GNOME_CMD_IS_CON_DEVICE (con));
 
-    gint ret, estatus;
-    GnomeVFSFileInfoOptions infoOpts = (GnomeVFSFileInfoOptions) (GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE);
+    GnomeCmdConDevice *dev_con = GNOME_CMD_CON_DEVICE (con);
 
-    if (!is_mounted (con))
+    gint ret, estatus;
+
+    if (!is_mounted (dev_con))
     {
-        gchar *cmd;
+        gchar *cmd = nullptr;
         gchar *emsg = nullptr;
 
-        GnomeCmdConDevice *dev_con = GNOME_CMD_CON_DEVICE (con);
+        if (dev_con->priv->device_fn != nullptr && dev_con->priv->mountp != nullptr)
+        {
+            DEBUG ('m', "mounting %s\n", dev_con->priv->mountp);
+            if (dev_con->priv->device_fn)
+                cmd = g_strdup_printf ("mount %s %s", dev_con->priv->device_fn, dev_con->priv->mountp);
+            else
+                cmd = g_strdup_printf ("mount %s", dev_con->priv->mountp);
+        }
+
+        if (!cmd)
+            return;
 
-        DEBUG ('m', "mounting %s\n", dev_con->priv->mountp);
-        if (dev_con->priv->device_fn)
-            cmd = g_strdup_printf ("mount %s %s", dev_con->priv->device_fn, dev_con->priv->mountp);
-        else
-            cmd = g_strdup_printf ("mount %s", dev_con->priv->mountp);
         DEBUG ('m', "Mount command: %s\n", cmd);
         ret = system (cmd);
         estatus = WEXITSTATUS (ret);
@@ -143,31 +146,127 @@ static void do_mount_thread_func (GnomeCmdCon *con)
     }
     else
         DEBUG('m', "The device was already mounted\n");
+}
 
-    GnomeVFSURI *uri = gnome_cmd_con_create_uri (con, con->base_path);
-    if (!uri)
+
+static void set_con_mount_succeed(GnomeCmdCon *con)
+{
+    con->state = GnomeCmdCon::STATE_OPEN;
+    con->open_result = GnomeCmdCon::OPEN_OK;
+}
+
+
+static void set_con_mount_failed(GnomeCmdCon *con)
+{
+    g_object_unref (con->base_gFileInfo);
+    con->base_gFileInfo = nullptr;
+    con->open_result = GnomeCmdCon::OPEN_FAILED;
+    con->state = GnomeCmdCon::STATE_CLOSED;
+    con->open_failed_msg = con->open_failed_error->message;
+}
+
+
+static void mount_finish_callback(GObject *gVol, GAsyncResult *result, gpointer user_data)
+{
+    GnomeCmdCon *con = GNOME_CMD_CON (user_data);
+    GnomeCmdConDevice *dev_con = GNOME_CMD_CON_DEVICE (con);
+    auto gVolume = G_VOLUME(gVol);
+    GError *error = nullptr;
+
+    if(!g_volume_mount_finish(gVolume, result, &error))
+    {
+        g_critical("Unable to mount the volume, error: %s", error->message);
+        con->open_failed_error = g_error_copy(error);
+        set_con_mount_failed(con);
+        g_error_free(error);
         return;
-
-    gchar *uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
-    con->base_info = gnome_vfs_file_info_new ();
-    GnomeVFSResult result = gnome_vfs_get_file_info_uri (uri, con->base_info, infoOpts);
-
-    gnome_vfs_uri_unref (uri);
-    g_free (uri_str);
-
-    if (result == GNOME_VFS_OK)
-    {
-        con->state = GnomeCmdCon::STATE_OPEN;
-        con->open_result = GnomeCmdCon::OPEN_OK;
     }
-    else
+    dev_con->priv->gMount = g_volume_get_mount (gVolume);
+    if (!con->base_path)
     {
-        gnome_vfs_file_info_unref (con->base_info);
-        con->base_info = nullptr;
-        con->open_failed_reason = result;
-        con->open_result = GnomeCmdCon::OPEN_FAILED;
-        con->state = GnomeCmdCon::STATE_CLOSED;
+        set_con_base_path_for_gmount(con, dev_con->priv->gMount);
     }
+    if (con->base_path)
+    {
+        gnome_cmd_con_set_root_path(con, con->base_path->get_path());
+    }
+    set_con_base_gfileinfo(con);
+    set_con_mount_succeed(con);
+}
+
+static void do_legacy_mount_thread_func(GnomeCmdCon *con)
+{
+    g_return_if_fail (GNOME_CMD_IS_CON_DEVICE (con));
+
+    GError *error = nullptr;
+
+    if (!con->base_path)
+        con->base_path = new GnomeCmdPlainPath(G_DIR_SEPARATOR_S);
+
+    do_legacy_mount(con);
+
+    auto gFile = gnome_cmd_con_create_gfile(con, con->base_path);
+
+    if (!gFile)
+    {
+        con->open_failed_error = g_error_new(G_IO_ERROR, G_IO_ERROR_FAILED, "Unable to create a GFile object for the given path.");
+        set_con_mount_failed(con);
+        g_warning("Unable to create a GFile object for the given path.");
+        return;
+    }
+    con->base_gFileInfo = g_file_query_info(gFile,
+                              "*",
+                              G_FILE_QUERY_INFO_NONE,
+                              nullptr,
+                              &error);
+    g_object_unref(gFile);
+    if (error)
+    {
+        con->open_failed_error = g_error_copy(error);
+        set_con_mount_failed(con);
+        g_warning("Unable to mount the volume via legacy mount, error: %s", error->message);
+        g_error_free(error);
+        return;
+    }
+
+    set_con_mount_succeed(con);
+    return;
+}
+
+
+static void do_mount (GnomeCmdCon *con)
+{
+    g_return_if_fail (GNOME_CMD_IS_CON_DEVICE (con));
+
+    GnomeCmdConDevice *dev_con = GNOME_CMD_CON_DEVICE (con);
+
+    // This is a legacy-mount: If mount point is given, we mount the device with system calls ('mount')
+    if (dev_con->priv->mountp)
+    {
+        auto gThread = g_thread_new (nullptr, (GThreadFunc) do_legacy_mount_thread_func, con);
+        g_thread_unref(gThread);
+    }
+
+    // Check if the volume is already mounted:
+    auto gMount = g_volume_get_mount (dev_con->priv->gVolume);
+    if (gMount)
+    {
+        dev_con->priv->gMount = gMount;
+        set_con_base_path_for_gmount(con, dev_con->priv->gMount);
+        set_con_base_gfileinfo(con);
+        set_con_mount_succeed(con);
+        return;
+    }
+
+    auto gMountOperation = gtk_mount_operation_new ((GtkWindow*) main_win);
+
+    g_volume_mount (dev_con->priv->gVolume,
+        G_MOUNT_MOUNT_NONE,
+        gMountOperation,
+        nullptr,
+        mount_finish_callback,
+        con);
+
 }
 
 
@@ -177,38 +276,47 @@ static void dev_open (GnomeCmdCon *con)
 
     DEBUG ('m', "Mounting device\n");
 
-    if (!con->base_path)
-        con->base_path = new GnomeCmdPlainPath(G_DIR_SEPARATOR_S);
-
     con->state = GnomeCmdCon::STATE_OPENING;
     con->open_result = GnomeCmdCon::OPEN_IN_PROGRESS;
 
-    g_thread_new (nullptr, (GThreadFunc) do_mount_thread_func, con);
+    do_mount(con);
 }
 
 
-static void dev_vfs_umount_callback (gboolean succeeded, char *error, char *detailed_error, gpointer data)
+static void unmount_callback(GObject *gMnt, GAsyncResult *result, gpointer user_data)
 {
+    GnomeCmdCon *con = GNOME_CMD_CON (user_data);
+    auto gMount = G_MOUNT(gMnt);
+    GError *error = nullptr;
     GtkWidget *msgbox;
 
-    DEBUG('m', "VFS umount callback: %s %s %s\n", succeeded ? "succeeded" : "failed",
-               error ? error : "",
-               detailed_error ? detailed_error : "");
+    if (!g_mount_unmount_with_operation_finish (gMount, result, &error))
+    {
+        DEBUG('m', "unmount_callback: failed %s\n", error->message);
 
-    if (succeeded)
-        msgbox = gtk_message_dialog_new (*main_win,
-                                         GTK_DIALOG_MODAL,
-                                         GTK_MESSAGE_INFO,
-                                         GTK_BUTTONS_OK,
-                                         _("Device is now safe to remove"));
-    else
         msgbox = gtk_message_dialog_new (*main_win,
                                          GTK_DIALOG_MODAL,
                                          GTK_MESSAGE_ERROR,
                                          GTK_BUTTONS_OK,
-                                         _("Cannot unmount the volume:\n%s %s"),
-                                         error ? error : _("Unknown error"),
-                                         detailed_error ? detailed_error: "");
+                                         _("Cannot unmount the volume:\n%s\nError code: %d"),
+                                         error->message,
+                                         error->code);
+        gtk_dialog_run (GTK_DIALOG (msgbox));
+        gtk_widget_destroy (msgbox);
+
+        g_error_free(error);
+        return;
+    }
+
+    con->state = GnomeCmdCon::STATE_CLOSED;
+
+    DEBUG('m', "unmount_callback: succeeded\n");
+
+    msgbox = gtk_message_dialog_new (*main_win,
+                                     GTK_DIALOG_MODAL,
+                                     GTK_MESSAGE_INFO,
+                                     GTK_BUTTONS_OK,
+                                     _("Volume successfully unmounted"));
 
     gtk_dialog_run (GTK_DIALOG (msgbox));
     gtk_widget_destroy (msgbox);
@@ -232,27 +340,34 @@ static gboolean dev_close (GnomeCmdCon *con)
 
     if (dev_con->priv->autovolume)
     {
-        if (dev_con->priv->vfsvol)
-        {
-            gchar *name;
-            name = gnome_vfs_volume_get_display_name (dev_con->priv->vfsvol);
-            DEBUG ('m', "umounting VFS volume \"%s\"\n", name);
-            g_free (name);
+        if (!g_mount_can_unmount(dev_con->priv->gMount))
+            return ret == 0;
 
-            gnome_vfs_volume_unmount (dev_con->priv->vfsvol, dev_vfs_umount_callback, nullptr);
-        }
+        auto gMountName = g_mount_get_name(dev_con->priv->gMount);
+        DEBUG ('m', "umounting GIO mount \"%s\"\n", gMountName);
+        g_free(gMountName);
+        g_mount_unmount_with_operation (dev_con->priv->gMount,
+                            G_MOUNT_UNMOUNT_NONE,
+                            nullptr,
+                            nullptr,
+                            unmount_callback,
+                            con);
     }
     else
     {
-        DEBUG ('m', "umounting %s\n", dev_con->priv->mountp);
-        gchar *cmd = g_strdup_printf ("umount %s", dev_con->priv->mountp);
-        ret = system (cmd);
-        DEBUG ('m', "umount returned %d\n", ret);
-        g_free (cmd);
-    }
+        // Legacy unmount
+        if(dev_con->priv->mountp)
+        {
+            DEBUG ('m', "umounting %s\n", dev_con->priv->mountp);
+            gchar *cmd = g_strdup_printf ("umount %s", dev_con->priv->mountp);
+            ret = system (cmd);
+            DEBUG ('m', "umount returned %d\n", ret);
+            g_free (cmd);
 
-    if (ret == 0)
-        con->state = GnomeCmdCon::STATE_CLOSED;
+            if (ret == 0)
+                con->state = GnomeCmdCon::STATE_CLOSED;
+        }
+    }
 
     return ret == 0;
 }
@@ -272,19 +387,26 @@ static gboolean dev_open_is_needed (GnomeCmdCon *con)
 }
 
 
-static GnomeVFSURI *dev_create_uri (GnomeCmdCon *con, GnomeCmdPath *path)
+static GFile *dev_create_gfile (GnomeCmdCon *con, GnomeCmdPath *gnomeCmdPath)
 {
     g_return_val_if_fail (GNOME_CMD_IS_CON_DEVICE (con), nullptr);
-    g_return_val_if_fail (path!=nullptr, nullptr);
 
+    GFile *newGFile = nullptr;
     GnomeCmdConDevice *dev_con = GNOME_CMD_CON_DEVICE (con);
 
-    gchar *p = g_build_filename (dev_con->priv->mountp, path->get_path(), nullptr);
-    GnomeVFSURI *u1 = gnome_vfs_uri_new ("file:");
-    GnomeVFSURI *u2 = gnome_vfs_uri_append_path (u1, p);
-    gnome_vfs_uri_unref (u1);
+    g_return_val_if_fail (dev_con->priv->gMount != nullptr, nullptr);
 
-    return u2;
+    if (gnomeCmdPath)
+    {
+        auto gMountGFile = g_mount_get_default_location (dev_con->priv->gMount);
+        newGFile = g_file_get_child (gMountGFile, gnomeCmdPath->get_path());
+        g_object_unref(gMountGFile);
+    }
+    else
+    {
+        newGFile = g_mount_get_default_location (dev_con->priv->gMount);
+    }
+    return newGFile;
 }
 
 
@@ -305,10 +427,15 @@ static void destroy (GtkObject *object)
 {
     GnomeCmdConDevice *con = GNOME_CMD_CON_DEVICE (object);
 
-    if (con->priv->vfsvol)
+    if (con->priv->gMount)
     {
-        gnome_vfs_volume_unref (con->priv->vfsvol);
-        con->priv->vfsvol = nullptr;
+        g_object_unref(con->priv->gMount);
+        con->priv->gMount = nullptr;
+    }
+    if (con->priv->gVolume)
+    {
+        g_object_unref(con->priv->gVolume);
+        con->priv->gVolume = nullptr;
     }
 
     g_free (con->priv);
@@ -333,7 +460,7 @@ static void class_init (GnomeCmdConDeviceClass *klass)
     con_class->close = dev_close;
     con_class->cancel_open = dev_cancel_open;
     con_class->open_is_needed = dev_open_is_needed;
-    con_class->create_uri = dev_create_uri;
+    con_class->create_gfile = dev_create_gfile;
     con_class->create_path = dev_create_path;
 }
 
@@ -344,7 +471,7 @@ static void init (GnomeCmdConDevice *dev_con)
 
     dev_con->priv = g_new0 (GnomeCmdConDevicePrivate, 1);
 
-    con->method = CON_LOCAL;
+    con->method = CON_FILE;
     con->should_remember_dir = TRUE;
     con->needs_open_visprog = FALSE;
     con->needs_list_visprog = FALSE;
@@ -394,10 +521,12 @@ GnomeCmdConDevice *gnome_cmd_con_device_new (const gchar *alias, const gchar *de
     gnome_cmd_con_device_set_mountp (dev, mountp);
     gnome_cmd_con_device_set_icon_path (dev, icon_path);
     gnome_cmd_con_device_set_autovol(dev, FALSE);
-    gnome_cmd_con_device_set_vfs_volume(dev, nullptr);
+    gnome_cmd_con_device_set_gmount(dev, nullptr);
+    gnome_cmd_con_device_set_gvolume(dev, nullptr);
     gnome_cmd_con_device_set_alias (dev, alias);
 
-    gnome_cmd_con_set_root_path (con, mountp);
+    if (mountp)
+        gnome_cmd_con_set_root_path (con, mountp);
 
     con->open_msg = g_strdup_printf (_("Mounting %s"), alias);
 
@@ -415,7 +544,10 @@ void gnome_cmd_con_device_set_alias (GnomeCmdConDevice *dev, const gchar *alias)
 
     dev->priv->alias = g_strdup (alias);
     GNOME_CMD_CON (dev)->alias = g_strdup (alias);
-    GNOME_CMD_CON (dev)->go_text = g_strdup_printf (_("Go to: %s (%s)"), alias, dev->priv->mountp);
+    if (dev->priv->mountp)
+        GNOME_CMD_CON (dev)->go_text = g_strdup_printf (_("Go to: %s (%s)"), alias, dev->priv->mountp);
+    else
+        GNOME_CMD_CON (dev)->go_text = g_strdup_printf (_("Go to: %s"), alias);
     GNOME_CMD_CON (dev)->open_text = g_strdup_printf (_("Mount: %s"), alias);
     GNOME_CMD_CON (dev)->close_text = g_strdup_printf (_("Unmount: %s"), alias);
 }
@@ -510,20 +642,37 @@ void gnome_cmd_con_device_set_autovol (GnomeCmdConDevice *dev, const gboolean au
 }
 
 
-void gnome_cmd_con_device_set_vfs_volume (GnomeCmdConDevice *dev, GnomeVFSVolume *vfsvol)
+void gnome_cmd_con_device_set_gmount (GnomeCmdConDevice *dev, GMount *gMount)
 {
     g_return_if_fail (dev != nullptr);
     g_return_if_fail (dev->priv != nullptr);
 
-    if (dev->priv->vfsvol)
+    if (dev->priv->gMount)
     {
-        gnome_vfs_volume_unref (dev->priv->vfsvol);
-        dev->priv->vfsvol = nullptr;
+        g_object_unref (dev->priv->gMount);
+        dev->priv->gMount = nullptr;
     }
 
-    dev->priv->vfsvol = vfsvol;
-    if (dev->priv->vfsvol)
-        gnome_vfs_volume_ref(dev->priv->vfsvol);
+    dev->priv->gMount = gMount;
+    if (dev->priv->gMount)
+        g_object_ref(dev->priv->gMount);
+}
+
+
+void gnome_cmd_con_device_set_gvolume (GnomeCmdConDevice *dev, GVolume *gVolume)
+{
+    g_return_if_fail (dev != nullptr);
+    g_return_if_fail (dev->priv != nullptr);
+
+    if (dev->priv->gVolume)
+    {
+        g_object_unref (dev->priv->gVolume);
+        dev->priv->gVolume = nullptr;
+    }
+
+    dev->priv->gVolume = gVolume;
+    if (dev->priv->gVolume)
+        g_object_ref(dev->priv->gVolume);
 }
 
 
@@ -545,7 +694,7 @@ const gchar *gnome_cmd_con_device_get_device_fn (GnomeCmdConDevice *dev)
 }
 
 
-const gchar *gnome_cmd_con_device_get_mountp (GnomeCmdConDevice *dev)
+const gchar *gnome_cmd_con_device_get_mountp_string (GnomeCmdConDevice *dev)
 {
     g_return_val_if_fail (dev != nullptr, nullptr);
     g_return_val_if_fail (dev->priv != nullptr, nullptr);
@@ -572,10 +721,18 @@ gboolean gnome_cmd_con_device_get_autovol (GnomeCmdConDevice *dev)
 }
 
 
-GnomeVFSVolume *gnome_cmd_con_device_get_vfs_volume (GnomeCmdConDevice *dev)
+GMount *gnome_cmd_con_device_get_gmount (GnomeCmdConDevice *dev)
 {
     g_return_val_if_fail (dev != nullptr, nullptr);
     g_return_val_if_fail (dev->priv != nullptr, nullptr);
 
-    return dev->priv->vfsvol;
+    return dev->priv->gMount;
+}
+
+GVolume *gnome_cmd_con_device_get_gvolume (GnomeCmdConDevice *dev)
+{
+    g_return_val_if_fail (dev != nullptr, nullptr);
+    g_return_val_if_fail (dev->priv != nullptr, nullptr);
+
+    return dev->priv->gVolume;
 }
