@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-con-device.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ void gnome_cmd_con_device_set_alias (GnomeCmdConDevice *dev, const gchar *alias)
 const gchar *gnome_cmd_con_device_get_device_fn (GnomeCmdConDevice *dev);
 void gnome_cmd_con_device_set_device_fn (GnomeCmdConDevice *dev, const gchar *device_fn);
 
-const gchar *gnome_cmd_con_device_get_mountp (GnomeCmdConDevice *dev);
+const gchar *gnome_cmd_con_device_get_mountp_string (GnomeCmdConDevice *dev);
 void gnome_cmd_con_device_set_mountp (GnomeCmdConDevice *dev, const gchar *mountp);
 
 const gchar *gnome_cmd_con_device_get_icon_path (GnomeCmdConDevice *dev);
@@ -68,5 +68,8 @@ void gnome_cmd_con_device_set_icon_path (GnomeCmdConDevice *dev, const gchar *ic
 gboolean gnome_cmd_con_device_get_autovol (GnomeCmdConDevice *dev);
 void gnome_cmd_con_device_set_autovol (GnomeCmdConDevice *dev, const gboolean autovol);
 
-GnomeVFSVolume *gnome_cmd_con_device_get_vfs_volume (GnomeCmdConDevice *dev);
-void gnome_cmd_con_device_set_vfs_volume (GnomeCmdConDevice *dev, GnomeVFSVolume *vfsvol);
+GMount *gnome_cmd_con_device_get_gmount (GnomeCmdConDevice *dev);
+void gnome_cmd_con_device_set_gmount (GnomeCmdConDevice *dev, GMount *gMount);
+
+GVolume *gnome_cmd_con_device_get_gvolume (GnomeCmdConDevice *dev);
+void gnome_cmd_con_device_set_gvolume (GnomeCmdConDevice *dev, GVolume *gVolume);
