@@ -2,7 +2,7 @@
  * @file gnome-cmd-con-home.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,13 +56,9 @@ static gboolean home_open_is_needed (GnomeCmdCon *con)
 }
 
 
-static GnomeVFSURI *home_create_uri (GnomeCmdCon *con, GnomeCmdPath *path)
+static GFile *home_create_gfile (GnomeCmdCon *con, GnomeCmdPath *path)
 {
-    GnomeVFSURI *u1 = gnome_vfs_uri_new ("file:");
-    GnomeVFSURI *u2 = gnome_vfs_uri_append_path (u1, path->get_path());
-    gnome_vfs_uri_unref (u1);
-
-    return u2;
+    return g_file_new_for_path(path->get_path());
 }
 
 
@@ -103,7 +99,7 @@ static void class_init (GnomeCmdConHomeClass *klass)
     con_class->close = home_close;
     con_class->cancel_open = home_cancel_open;
     con_class->open_is_needed = home_open_is_needed;
-    con_class->create_uri = home_create_uri;
+    con_class->create_gfile = home_create_gfile;
     con_class->create_path = home_create_path;
 }
 
@@ -116,7 +112,7 @@ static void init (GnomeCmdConHome *home_con)
 
     con->state = GnomeCmdCon::STATE_OPEN;
     con->alias = g_strdup (_("Home"));
-    con->method = CON_LOCAL;
+    con->method = CON_FILE;
     con->open_msg = g_strdup ("This should not be visible anywhere");
     con->should_remember_dir = TRUE;
     con->needs_open_visprog = FALSE;
@@ -125,9 +121,9 @@ static void init (GnomeCmdConHome *home_con)
     con->is_local = TRUE;
     con->is_closeable = FALSE;
     con->go_text = g_strdup (_("Go to: Home"));
-    con->go_pixmap = gnome_cmd_pixmap_new_from_icon ("gnome-fs-home", dev_icon_size);
-    con->open_pixmap = gnome_cmd_pixmap_new_from_icon ("gnome-fs-home", dev_icon_size);
-    con->close_pixmap = gnome_cmd_pixmap_new_from_icon ("gnome-fs-home", dev_icon_size);
+    con->go_pixmap = gnome_cmd_pixmap_new_from_icon ("user-home", dev_icon_size);
+    con->open_pixmap = gnome_cmd_pixmap_new_from_icon ("user-home", dev_icon_size);
+    con->close_pixmap = gnome_cmd_pixmap_new_from_icon ("user-home", dev_icon_size);
 
     GnomeCmdDir *dir = gnome_cmd_dir_new (con, new GnomeCmdPlainPath(g_get_home_dir ()));
 
