@@ -2,7 +2,7 @@
  * @file gnome-cmd-con-list.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -149,12 +149,17 @@ static void init (GnomeCmdConList *con_list)
     con_list->priv->update_lock = FALSE;
 
     con_list->priv->home_con = gnome_cmd_con_home_new ();
+
 #ifdef HAVE_SAMBA
-    con_list->priv->smb_con = gnome_cmd_con_smb_new ();
+    if (gnome_cmd_data.options.show_samba_workgroups_button)
+        con_list->priv->smb_con = gnome_cmd_con_smb_new ();
 #endif
+
     con_list->priv->all_cons = g_list_append (nullptr, con_list->priv->home_con);
+
 #ifdef HAVE_SAMBA
-    con_list->priv->all_cons = g_list_append (con_list->priv->all_cons, con_list->priv->smb_con);
+    if (gnome_cmd_data.options.show_samba_workgroups_button)
+        con_list->priv->all_cons = g_list_append (con_list->priv->all_cons, con_list->priv->smb_con);
 #endif
 }
 
@@ -380,3 +385,25 @@ GnomeCmdCon *GnomeCmdConList::get_smb()
     return priv->smb_con;
 }
 #endif
+
+
+GnomeCmdCon *get_remote_con_for_gfile(GFile *gFile)
+{
+    GnomeCmdCon *gnomeCmdCon = nullptr;
+    auto remoteCons = get_remote_cons();
+
+    for(auto remoteConEntry = remoteCons; remoteConEntry; remoteConEntry = remoteConEntry->next)
+    {
+        auto remoteCon = static_cast<GnomeCmdConRemote*>(remoteConEntry->data);
+        auto gnomeCmdConParent = &remoteCon->parent;
+        auto gFileSrcUri = g_file_get_uri(gFile);
+        if (strstr(gFileSrcUri, gnomeCmdConParent->uri))
+        {
+            gnomeCmdCon = gnomeCmdConParent;
+            g_free(gFileSrcUri);
+            break;
+        }
+        g_free(gFileSrcUri);
+    }
+    return gnomeCmdCon;
+}
\ No newline at end of file
