@@ -2,7 +2,7 @@
  * @file gnome-cmd-con-remote.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-con-remote.h"
+#include "gnome-cmd-main-win.h"
 #include "gnome-cmd-plain-path.h"
 #include "imageloader.h"
 #include "utils.h"
@@ -34,46 +35,64 @@ using namespace std;
 static GnomeCmdConClass *parent_class = nullptr;
 
 
-static void get_file_info_func (GnomeCmdCon *con)
+static void set_con_mount_failed(GnomeCmdCon *con)
 {
-    GnomeVFSURI *uri = gnome_cmd_con_create_uri (con, con->base_path);
+    g_return_if_fail(GNOME_CMD_IS_CON(con));
+    con->base_gFileInfo = nullptr;
+    con->open_result = GnomeCmdCon::OPEN_FAILED;
+    con->state = GnomeCmdCon::STATE_CLOSED;
+    con->open_failed_msg = con->open_failed_error->message;
+}
 
-    GnomeVFSFileInfoOptions infoOpts = (GnomeVFSFileInfoOptions) (GNOME_VFS_FILE_INFO_FOLLOW_LINKS | GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE);
+static void mount_remote_finish_callback(GObject *gobj, GAsyncResult *result, gpointer user_data)
+{
+    auto con = GNOME_CMD_CON(user_data);
+    g_return_if_fail(GNOME_CMD_IS_CON(user_data));
+    g_return_if_fail(gobj != nullptr);
+    auto gFile = G_FILE(gobj);
+    g_return_if_fail(G_IS_FILE(gFile));
 
-    DEBUG('m', "Connecting to %s\n", gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD));
+    GError *error = nullptr;
 
-    // Get basic file info - opens gnome-keyring dialog via libgnome-keyring for password input if needed
-    con->base_info = gnome_vfs_file_info_new ();
-    GnomeVFSResult res = gnome_vfs_get_file_info_uri (uri, con->base_info, infoOpts);
-
-    gnome_vfs_uri_unref (uri);
-
-    if (con->state == GnomeCmdCon::STATE_OPENING)
+    g_file_mount_enclosing_volume_finish(gFile, result, &error);
+    if (error && !g_error_matches(error, G_IO_ERROR, G_IO_ERROR_ALREADY_MOUNTED))
     {
-        DEBUG('m', "State was OPENING, setting flags\n");
-        if (res == GNOME_VFS_OK)
-        {
-            con->state = GnomeCmdCon::STATE_OPEN;
-            con->open_result = GnomeCmdCon::OPEN_OK;
-        }
-        else
-        {
-            con->state = GnomeCmdCon::STATE_CLOSED;
-            con->open_failed_reason = res;
-            con->open_result = GnomeCmdCon::OPEN_FAILED;
-        }
+        DEBUG('m', "Unable to mount enclosing volume: %s\n", error->message);
+        con->open_failed_error = g_error_copy(error);
+        g_error_free(error);
+        set_con_mount_failed(con);
+        g_object_unref(gFile);
+        return;
     }
-    else
-        if (con->state == GnomeCmdCon::STATE_CANCELLING)
-            DEBUG('m', "The open operation was cancelled, doing nothing\n");
-        else
-            DEBUG('m', "Strange ConState %d\n", con->state);
+    set_con_base_gfileinfo(con);
+    con->state = GnomeCmdCon::STATE_OPEN;
+    con->open_result = GnomeCmdCon::OPEN_OK;
+    g_object_unref(gFile);
 }
 
 
-static gboolean start_get_file_info (GnomeCmdCon *con)
+static void mount_func (GnomeCmdCon *con)
 {
-    g_thread_new (nullptr, (GThreadFunc) get_file_info_func, con);
+    g_return_if_fail(GNOME_CMD_IS_CON(con));
+
+    auto gFile = gnome_cmd_con_create_gfile(con);
+
+    auto uri = g_file_get_uri(gFile);
+    DEBUG('m', "Connecting to %s\n", uri);
+
+    auto gMountOperation = gtk_mount_operation_new ((GtkWindow*) main_win);
+
+    g_file_mount_enclosing_volume (gFile,
+                                   G_MOUNT_MOUNT_NONE,
+                                   gMountOperation,
+                                   nullptr,
+                                   mount_remote_finish_callback,
+                                   con);
+}
+
+static gboolean start_mount_func (GnomeCmdCon *con)
+{
+    g_thread_new (nullptr, (GThreadFunc) mount_func, con);
 
     return FALSE;
 }
@@ -91,17 +110,62 @@ static void remote_open (GnomeCmdCon *con)
     if (!con->base_path)
         con->base_path = new GnomeCmdPlainPath(G_DIR_SEPARATOR_S);
 
-    g_timeout_add (1, (GSourceFunc) start_get_file_info, con);
+    g_timeout_add (1, (GSourceFunc) start_mount_func, con);
 }
 
 
+static void remote_close_callback(GObject *gobj, GAsyncResult *result, gpointer user_data)
+{
+    auto gMount = (GMount*) gobj;
+    auto con = (GnomeCmdCon*) user_data;
+    GError *error = nullptr;
+
+    g_mount_unmount_with_operation_finish(gMount, result, &error);
+
+    if (error && !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CLOSED))
+    {
+        gnome_cmd_error_message(_("Disconnect error"), error);
+        g_error_free(error);
+        return;
+    }
+    if (error)
+        g_error_free(error);
+
+    con->state = GnomeCmdCon::STATE_CLOSED;
+    con->open_result = GnomeCmdCon::OPEN_NOT_STARTED;
+}
+
 static gboolean remote_close (GnomeCmdCon *con)
 {
+    GError *error = nullptr;
+
     gnome_cmd_con_set_default_dir (con, nullptr);
     delete con->base_path;
     con->base_path = nullptr;
-    con->state = GnomeCmdCon::STATE_CLOSED;
-    con->open_result = GnomeCmdCon::OPEN_NOT_STARTED;
+
+    auto uri = gnome_cmd_con_get_uri(con);
+    auto gFileTmp = g_file_new_for_uri(uri);
+    DEBUG ('m', "Closing connection to %s\n", uri);
+
+    auto gMount = g_file_find_enclosing_mount (gFileTmp, nullptr, &error);
+    if (error)
+    {
+        g_warning("remote_close - g_file_find_enclosing_mount error: %s - %s", uri, error->message);
+        g_error_free(error);
+        g_object_unref(gFileTmp);
+        return false;
+    }
+
+    g_mount_unmount_with_operation (
+        gMount,
+        G_MOUNT_UNMOUNT_NONE,
+        nullptr,
+        nullptr,
+        remote_close_callback,
+        con
+    );
+
+    g_object_unref(gFileTmp);
 
     return TRUE;
 }
@@ -120,16 +184,29 @@ static gboolean remote_open_is_needed (GnomeCmdCon *con)
 }
 
 
-static GnomeVFSURI *remote_create_uri (GnomeCmdCon *con, GnomeCmdPath *path)
+static GFile *create_remote_gfile_with_path(GnomeCmdCon *con, GnomeCmdPath *path)
+{
+    auto gUri = g_uri_build(G_URI_FLAGS_NONE, con->scheme, nullptr, con->hostname, con->port, path->get_path(), nullptr, nullptr);
+    auto gFilePathUriString = g_uri_to_string(gUri);
+    auto gFile = g_file_new_for_uri(gFilePathUriString);
+    g_free(gFilePathUriString);
+    return gFile;
+}
+
+
+static GFile *remote_create_gfile (GnomeCmdCon *con, GnomeCmdPath *path)
 {
     g_return_val_if_fail (con->uri != nullptr, nullptr);
 
-    GnomeVFSURI *u0 = gnome_vfs_uri_new (con->uri);
-    GnomeVFSURI *u1 = gnome_vfs_uri_append_path (u0, path->get_path());
-
-    gnome_vfs_uri_unref (u0);
-
-    return u1;
+    if (path)
+    {
+        return create_remote_gfile_with_path(con, path);
+    }
+    else
+    {
+        auto gFile = g_file_new_for_uri (con->uri);
+        return gFile;
+    }
 }
 
 
@@ -145,7 +222,7 @@ static GnomeCmdPath *remote_create_path (GnomeCmdCon *con, const gchar *path_str
 
 static void destroy (GtkObject *object)
 {
-    GnomeCmdConRemote *con_remote = GNOME_CMD_CON_REMOTE (object);
+    auto con_remote = GNOME_CMD_CON_REMOTE (object);
 
     gnome_cmd_pixmap_free (con_remote->parent.go_pixmap);
     gnome_cmd_pixmap_free (con_remote->parent.open_pixmap);
@@ -169,7 +246,7 @@ static void class_init (GnomeCmdConRemoteClass *klass)
     con_class->close = remote_close;
     con_class->cancel_open = remote_cancel_open;
     con_class->open_is_needed = remote_open_is_needed;
-    con_class->create_uri = remote_create_uri;
+    con_class->create_gfile = remote_create_gfile;
     con_class->create_path = remote_create_path;
 }
 
@@ -249,45 +326,71 @@ GtkType gnome_cmd_con_remote_get_type ()
 /**
  * Logic for setting up a new remote connection accordingly to the given uri_str.
  */
-GnomeCmdConRemote *gnome_cmd_con_remote_new (const gchar *alias, const string &uri_str, GnomeCmdCon::Authentication auth)
+GnomeCmdConRemote *gnome_cmd_con_remote_new (const gchar *alias, const string &uri_str)
 {
-    gchar *canonical_uri = gnome_vfs_make_uri_canonical (uri_str.c_str());
+    auto gnomeCmdConRemote = static_cast<GnomeCmdConRemote*> (g_object_new (GNOME_CMD_TYPE_CON_REMOTE, nullptr));
 
-    GnomeVFSURI *uri = gnome_vfs_uri_new (canonical_uri);
+    g_return_val_if_fail (gnomeCmdConRemote != nullptr, nullptr);
 
-    g_return_val_if_fail (uri != nullptr, nullptr);
+    GError *error = nullptr;
 
-    auto server = static_cast<GnomeCmdConRemote*> (g_object_new (GNOME_CMD_TYPE_CON_REMOTE, nullptr));
+    gchar *scheme = nullptr;
+    gchar *user = nullptr;
+    gchar *host = nullptr;
+    gint port = -1;
+    gchar *path = nullptr;
 
-    g_return_val_if_fail (server != nullptr, nullptr);
+    g_uri_split_with_user (
+        uri_str.c_str(),
+        G_URI_FLAGS_HAS_PASSWORD,
+        &scheme,
+        &user,
+        nullptr, //password
+        nullptr, //auth_params
+        &host, //host
+        &port, //port
+        &path, //path
+        nullptr, //query
+        nullptr, //fragment
+        &error
+    );
+    if (error)
+    {
+        g_warning("gnome_cmd_con_remote_new - g_uri_split error: %s", error->message);
+        g_error_free(error);
+        return nullptr;
+    }
 
-    const gchar *host = gnome_vfs_uri_get_host_name (uri);      // do not g_free
-    const gchar *password = gnome_vfs_uri_get_password (uri);   // do not g_free
-    gchar *path = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri), nullptr);
-
-    GnomeCmdCon *con = GNOME_CMD_CON (server);
+    GnomeCmdCon *con = GNOME_CMD_CON (gnomeCmdConRemote);
 
     gnome_cmd_con_set_alias (con, alias);
-    gnome_cmd_con_set_uri (con, canonical_uri);
+    gnome_cmd_con_set_uri (con, uri_str.c_str());
+    gnome_cmd_con_set_scheme(con, scheme);
+    gnome_cmd_con_set_user_name (con, user);
     gnome_cmd_con_set_host_name (con, host);
+    gnome_cmd_con_set_port (con, port);
     gnome_cmd_con_set_root_path (con, path);
 
-    gnome_cmd_con_remote_set_host_name (server, host);
+    gnome_cmd_con_remote_set_tooltips (gnomeCmdConRemote, host);
 
-    con->method = gnome_cmd_con_get_scheme (uri);
-    con->auth = con->method==CON_ANON_FTP ? GnomeCmdCon::NOT_REQUIRED : password ? GnomeCmdCon::SAVE_FOR_SESSION : GnomeCmdCon::SAVE_PERMANENTLY;
+    con->method = gnome_cmd_con_get_scheme (uri_str.c_str());
 
+    g_free (scheme);
+    g_free(host);
     g_free (path);
-    gnome_vfs_uri_unref (uri);
+    g_free (user);
 
-    return server;
+    return gnomeCmdConRemote;
 }
 
 
-void gnome_cmd_con_remote_set_host_name (GnomeCmdConRemote *con, const gchar *host_name)
+void gnome_cmd_con_remote_set_tooltips (GnomeCmdConRemote *con, const gchar *host_name)
 {
     g_return_if_fail (con != nullptr);
-    g_return_if_fail (host_name != nullptr);
+    if (!host_name)
+    {
+        return;
+    }
 
     GNOME_CMD_CON (con)->open_tooltip = g_strdup_printf (_("Opens remote connection to %s"), host_name);
     GNOME_CMD_CON (con)->close_tooltip = g_strdup_printf (_("Closes remote connection to %s"), host_name);
