@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-con-remote.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,6 @@ struct GnomeCmdConRemoteClass
 
 GtkType gnome_cmd_con_remote_get_type ();
 
-GnomeCmdConRemote *gnome_cmd_con_remote_new (const gchar *alias, const std::string &uri_str, GnomeCmdCon::Authentication auth=GnomeCmdCon::SAVE_PERMANENTLY);
+GnomeCmdConRemote *gnome_cmd_con_remote_new (const gchar *alias, const std::string &uri_str);
 
-void gnome_cmd_con_remote_set_host_name (GnomeCmdConRemote *con, const gchar *host_name);
+void gnome_cmd_con_remote_set_tooltips (GnomeCmdConRemote *con, const gchar *host_name);
