@@ -2,7 +2,7 @@
  * @file gnome-cmd-con.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@
 
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-con.h"
+#include "gnome-cmd-plain-path.h"
+#include "gnome-cmd-main-win.h"
+#include "gnome-cmd-con-list.h"
 
 using namespace std;
 
@@ -51,16 +54,16 @@ static GtkObjectClass *parent_class = nullptr;
 
 
 // Keep this in sync with enum ConnectionMethodID in gnome-cmd-con.h
-const gchar *icon_name[] = {"gnome-fs-ssh",            // CON_SSH
-                            "gnome-fs-ftp",            // CON_FTP
-                            "gnome-fs-ftp",            // CON_ANON_FTP
+const gchar *icon_name[] = {"folder-remote",           // CON_SSH
+                            "folder-remote",           // CON_FTP
+                            "folder-remote",           // CON_ANON_FTP
 #ifdef HAVE_SAMBA
-                            "gnome-fs-smb",            // CON_SMB
+                            "folder-remote",           // CON_SMB
 #endif
-                            "gnome-fs-web",            // CON_DAV
-                            "gnome-fs-web",            // CON_DAVS
-                            "gnome-fs-network",        // CON_URI
-                            "gnome-fs-directory"};     // CON_LOCAL
+                            "folder-remote",           // CON_DAV
+                            "folder-remote",           // CON_DAVS
+                            "network-workgroup",       // CON_URI
+                            "folder"};                 // CON_FILE
 
 
 static void on_open_done (GnomeCmdCon *con)
@@ -69,9 +72,12 @@ static void on_open_done (GnomeCmdCon *con)
 }
 
 
-static void on_open_failed (GnomeCmdCon *con, const gchar *msg, GnomeVFSResult result)
+static void on_open_failed (GnomeCmdCon *con)
 {
     // gnome_cmd_con_updated (con);
+    // Free the error because the error handling is done now. (Logging happened already.)
+    g_error_free(con->open_failed_error);
+    con->open_failed_msg = nullptr;
 }
 
 
@@ -85,6 +91,7 @@ static void destroy (GtkObject *object)
 
     g_free (con->alias);
     g_free (con->uri);
+    g_free (con->scheme);
 
     delete con->base_path;
     g_string_free (con->root_path, TRUE);
@@ -93,6 +100,9 @@ static void destroy (GtkObject *object)
     gnome_cmd_pixmap_free (con->open_pixmap);
     g_free (con->close_text);
     g_free (con->close_tooltip);
+    g_object_unref(con->base_gFileInfo);
+    g_error_free(con->open_failed_error);
+    con->open_failed_msg = nullptr;
     gnome_cmd_pixmap_free (con->close_pixmap);
     gnome_cmd_pixmap_free (con->go_pixmap);
 
@@ -147,9 +157,9 @@ static void class_init (GnomeCmdConClass *klass)
             GTK_RUN_LAST,
             G_OBJECT_CLASS_TYPE (object_class),
             GTK_SIGNAL_OFFSET (GnomeCmdConClass, open_failed),
-            gtk_marshal_NONE__POINTER_INT,
+            gtk_marshal_NONE__NONE,
             GTK_TYPE_NONE,
-            2, GTK_TYPE_POINTER, GTK_TYPE_INT);
+            0);
 
     object_class->destroy = destroy;
 
@@ -161,7 +171,7 @@ static void class_init (GnomeCmdConClass *klass)
     klass->close = nullptr;
     klass->cancel_open = nullptr;
     klass->open_is_needed = nullptr;
-    klass->create_uri = nullptr;
+    klass->create_gfile = nullptr;
     klass->create_path = nullptr;
 }
 
@@ -170,11 +180,10 @@ static void init (GnomeCmdCon *con)
 {
     con->alias = nullptr;
     con->uri = nullptr;
+    con->scheme = nullptr;
     con->method = CON_URI;
-    con->auth = GnomeCmdCon::NOT_REQUIRED;
 
     con->base_path = nullptr;
-    con->base_info = nullptr;
     con->root_path = g_string_sized_new (128);
     con->open_msg = nullptr;
     con->should_remember_dir = FALSE;
@@ -195,8 +204,8 @@ static void init (GnomeCmdCon *con)
 
     con->state = GnomeCmdCon::STATE_CLOSED;
     con->open_result = GnomeCmdCon::OPEN_NOT_STARTED;
-    con->open_failed_reason = GNOME_VFS_OK;
     con->open_failed_msg = nullptr;
+    con->open_failed_error = nullptr;
 
     con->priv = g_new0 (GnomeCmdConPrivate, 1);
     con->priv->default_dir = nullptr;
@@ -270,7 +279,7 @@ static gboolean check_con_open_progress (GnomeCmdCon *con)
             {
                 DEBUG ('m', "GnomeCmdCon::OPEN_FAILED detected\n");
                 DEBUG ('m', "Emitting 'open-failed' signal\n");
-                gtk_signal_emit (GTK_OBJECT (con), signals[OPEN_FAILED], con->open_failed_msg, con->open_failed_reason);
+                gtk_signal_emit (GTK_OBJECT (con), signals[OPEN_FAILED]);
             }
             return FALSE;
 
@@ -283,6 +292,61 @@ static gboolean check_con_open_progress (GnomeCmdCon *con)
 }
 
 
+void gnome_cmd_con_set_base_path(GnomeCmdCon *con, GnomeCmdPath *path)
+{
+    g_return_if_fail (con != nullptr);
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+    g_return_if_fail (path != nullptr);
+
+    if (con->base_path)
+        delete con->base_path;
+
+    con->base_path = path;
+}
+
+
+void set_con_base_path_for_gmount(GnomeCmdCon *con, GMount *gMount)
+{
+    g_return_if_fail (con != nullptr);
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+    g_return_if_fail (gMount != nullptr);
+    g_return_if_fail (G_IS_MOUNT(gMount));
+
+    auto gFile = g_mount_get_default_location(gMount);
+    auto pathString = g_file_get_path(gFile);
+    g_object_unref(gFile);
+
+    gnome_cmd_con_set_base_path(con, new GnomeCmdPlainPath(pathString));
+
+    g_free(pathString);
+}
+
+gboolean set_con_base_gfileinfo(GnomeCmdCon *con)
+{
+    g_return_val_if_fail (con != nullptr, FALSE);
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), FALSE);
+    GError *error = nullptr;
+
+    if (con->base_gFileInfo)
+    {
+        g_object_unref(con->base_gFileInfo);
+        con->base_gFileInfo = nullptr;
+    }
+
+    auto gFile = con->is_local
+        ? gnome_cmd_con_create_gfile (con, con->base_path)
+        : gnome_cmd_con_create_gfile (con, nullptr);
+    con->base_gFileInfo = g_file_query_info(gFile, "*", G_FILE_QUERY_INFO_NONE, nullptr, &error);
+    g_object_unref(gFile);
+    if (error)
+    {
+        g_critical("set_con_base_gfileinfo: error: %s", error->message);
+        g_error_free(error);
+        return FALSE;
+    }
+    return TRUE;
+}
+
 void gnome_cmd_con_open (GnomeCmdCon *con)
 {
     g_return_if_fail (GNOME_CMD_IS_CON (con));
@@ -313,7 +377,7 @@ gboolean gnome_cmd_con_close (GnomeCmdCon *con)
 {
     g_return_val_if_fail (GNOME_CMD_IS_CON (con), FALSE);
 
-    if (gnome_cmd_con_is_closeable (con))
+    if (gnome_cmd_con_is_closeable (con) && gnome_cmd_con_is_open(con))
     {
         gtk_signal_emit (GTK_OBJECT (con), signals[CLOSE]);
         gtk_signal_emit (GTK_OBJECT (con), signals[UPDATED]);
@@ -323,13 +387,13 @@ gboolean gnome_cmd_con_close (GnomeCmdCon *con)
 }
 
 
-GnomeVFSURI *gnome_cmd_con_create_uri (GnomeCmdCon *con, GnomeCmdPath *path)
+GFile *gnome_cmd_con_create_gfile (GnomeCmdCon *con, GnomeCmdPath *path)
 {
     g_return_val_if_fail (GNOME_CMD_IS_CON (con), nullptr);
 
     GnomeCmdConClass *klass = GNOME_CMD_CON_GET_CLASS (con);
 
-    return klass->create_uri (con, path);
+    return klass->create_gfile (con, path);
 }
 
 
@@ -395,6 +459,10 @@ void gnome_cmd_con_add_bookmark (GnomeCmdCon *con, gchar *name, gchar *path)
 
 void gnome_cmd_con_erase_bookmark (GnomeCmdCon *con)
 {
+    if (!con)
+        return;
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+
     GnomeCmdBookmarkGroup *group = con->priv->bookmarks;
     for(GList *l = group->bookmarks; l; l = l->next)
     {
@@ -417,66 +485,79 @@ void gnome_cmd_con_updated (GnomeCmdCon *con)
 }
 
 
-// Get the type of the file at the specified path.
-// If the operation succeeds GNOME_VFS_OK is returned and type is set
-GnomeVFSResult gnome_cmd_con_get_path_target_type (GnomeCmdCon *con, const gchar *path_str, GnomeVFSFileType *type)
+/**
+ *  Get the type of the file at the specified path.
+ *  If the file does not exists, the function will return false.
+ *  If the operation succeeds, true is returned and type is set.
+ */
+gboolean gnome_cmd_con_get_path_target_type (GnomeCmdCon *con, const gchar *path_str, GFileType *gFileType)
 {
-    g_return_val_if_fail (GNOME_CMD_IS_CON (con), GNOME_VFS_ERROR_BAD_PARAMETERS);
-    g_return_val_if_fail (path_str != nullptr, GNOME_VFS_ERROR_BAD_PARAMETERS);
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), false);
+    g_return_val_if_fail (path_str != nullptr, false);
 
     GnomeCmdPath *path = gnome_cmd_con_create_path (con, path_str);
-    GnomeVFSURI *uri = gnome_cmd_con_create_uri (con, path);
-    GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
-    GnomeVFSResult res = gnome_vfs_get_file_info_uri (uri, info, GNOME_VFS_FILE_INFO_DEFAULT);
+    auto gFile = gnome_cmd_con_create_gfile(con, path);
 
-    if (res == GNOME_VFS_OK && info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK)         // resolve the symlink to get the real type of it
-        res = gnome_vfs_get_file_info_uri (uri, info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+    if (!gFile || !g_file_query_exists(gFile, nullptr))
+    {
+        if (gFile)
+            g_object_unref(gFile);
+        *gFileType = G_FILE_TYPE_UNKNOWN;
+        delete path;
+        return false;
+    }
 
-    if (res == GNOME_VFS_OK)
-        *type = info->type;
-
-    gnome_vfs_uri_unref (uri);
+    *gFileType = g_file_query_file_type(gFile, G_FILE_QUERY_INFO_NONE, nullptr);
+    g_object_unref(gFile);
     delete path;
-    gnome_vfs_file_info_unref (info);
 
-    return res;
+    return true;
 }
 
 
-GnomeVFSResult gnome_cmd_con_mkdir (GnomeCmdCon *con, const gchar *path_str)
+gboolean gnome_cmd_con_mkdir (GnomeCmdCon *con, const gchar *path_str, GError *error)
 {
-    GnomeVFSResult result;
+    GError *tmpError = nullptr;
 
-    g_return_val_if_fail (GNOME_CMD_IS_CON (con), GNOME_VFS_ERROR_BAD_PARAMETERS);
-    g_return_val_if_fail (path_str != nullptr, GNOME_VFS_ERROR_BAD_PARAMETERS);
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), false);
+    g_return_val_if_fail (path_str != nullptr, false);
 
-    GnomeCmdPath *path = gnome_cmd_con_create_path (con, path_str);
-    GnomeVFSURI *uri = gnome_cmd_con_create_uri (con, path);
+    auto path = gnome_cmd_con_create_path (con, path_str);
+    auto gFile = gnome_cmd_con_create_gfile (con, path);
 
-    result = gnome_vfs_make_directory_for_uri (
-        uri,
-        GNOME_VFS_PERM_USER_READ|GNOME_VFS_PERM_USER_WRITE|GNOME_VFS_PERM_USER_EXEC|
-        GNOME_VFS_PERM_GROUP_READ|GNOME_VFS_PERM_GROUP_EXEC|
-        GNOME_VFS_PERM_OTHER_READ|GNOME_VFS_PERM_OTHER_EXEC);
+    if (!g_file_make_directory (gFile, nullptr, &tmpError))
+    {
+        g_warning("g_file_make_directory error: %s\n", tmpError ? tmpError->message : "unknown error");
+        g_propagate_error(&error, tmpError);
+        return false;
+    }
 
-    gnome_vfs_uri_unref (uri);
+    g_object_unref (gFile);
     delete path;
 
-    return result;
+    return true;
 }
 
 
-void gnome_cmd_con_add_to_cache (GnomeCmdCon *con, GnomeCmdDir *dir)
+void gnome_cmd_con_add_to_cache (GnomeCmdCon *con, GnomeCmdDir *dir, gchar *uri_str)
 {
     g_return_if_fail (GNOME_CMD_IS_CON (con));
     g_return_if_fail (GNOME_CMD_IS_DIR (dir));
 
-    gchar *uri_str = GNOME_CMD_FILE (dir)->get_uri_str();
+    if (!uri_str)
+    {
+        uri_str = GNOME_CMD_FILE (dir)->get_uri_str();
+    }
+
+    if (!uri_str)
+    {
+        return;
+    }
 
     if (!con->priv->all_dirs_map)
         con->priv->all_dirs_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, nullptr);
 
-    DEBUG ('k', "ADDING 0x%p %s to the cache\n", dir, uri_str);
+    DEBUG ('k', "ADDING %p %s to the cache\n", dir, uri_str);
     g_hash_table_insert (con->priv->all_dirs_map, uri_str, dir);
 }
 
@@ -488,7 +569,7 @@ void gnome_cmd_con_remove_from_cache (GnomeCmdCon *con, GnomeCmdDir *dir)
 
     gchar *uri_str = GNOME_CMD_FILE (dir)->get_uri_str();
 
-    DEBUG ('k', "REMOVING 0x%p %s from the cache\n", dir, uri_str);
+    DEBUG ('k', "REMOVING %p %s from the cache\n", dir, uri_str);
     g_hash_table_remove (con->priv->all_dirs_map, uri_str);
     g_free (uri_str);
 }
@@ -517,7 +598,7 @@ GnomeCmdDir *gnome_cmd_con_cache_lookup (GnomeCmdCon *con, const gchar *uri_str)
     }
 
     if (dir)
-        DEBUG ('k', "FOUND 0x%p %s in the hash-table, reusing it!\n", dir, uri_str);
+        DEBUG ('k', "FOUND %p %s in the hash-table, reusing it!\n", dir, uri_str);
     else
         DEBUG ('k', "FAILED to find %s in the hash-table\n", uri_str);
 
@@ -531,65 +612,84 @@ const gchar *gnome_cmd_con_get_icon_name (ConnectionMethodID method)
 }
 
 
-string &__gnome_cmd_con_make_uri (string &s, const gchar *method, gboolean use_auth, string &server, string &port, string &folder, string &user, string &password)
+string &__gnome_cmd_con_make_uri (string &s, const gchar *method, string &server, string &port, string &folder)
 {
-    user = stringify (g_strescape (user.c_str(), nullptr));
-    password = stringify (g_strescape (password.c_str(), nullptr));
-
-    if (!password.empty() && !use_auth)
-    {
-        user += ':';
-        user += password;
-    }
-
-    folder = stringify (gnome_vfs_escape_path_string (folder.c_str()));
+    folder = stringify (g_uri_escape_string (folder.c_str(), nullptr, true));
 
     s = method;
 
-    if (!user.empty())
-        s += user + '@';
-
     s += server;
 
     if (!port.empty())
         s += ':' + port;
 
     if (!folder.empty())
-        s += '/' + folder;
+    {
+        if (folder[0] != '/')
+        {
+            s += '/';
+        }
+        s += folder;
+    }
 
     return s;
 }
 
 #ifdef HAVE_SAMBA
-std::string &gnome_cmd_con_make_smb_uri (std::string &s, gboolean use_auth, std::string &server, std::string &share, std::string &folder, std::string &domain, std::string &user, std::string &password)
+std::string &gnome_cmd_con_make_smb_uri (std::string &uriString, std::string &server, std::string &folder, std::string &domain)
 {
-    share = '/' + share;
-
-    user = stringify (g_strescape (user.c_str(), nullptr));
-    password = stringify (g_strescape (password.c_str(), nullptr));
-
-    if (!password.empty() && !use_auth)
-    {
-        user += ':';
-        user += password;
-    }
-
-    if (!domain.empty())
-        user = domain + ';' + user;
-
     const gchar *joinSign = !folder.empty() && folder[0] != '/' ? "/" : "";
 
-    folder = share + joinSign + folder;
-    folder = stringify (gnome_vfs_escape_path_string (folder.c_str()));
+    folder = joinSign + folder;
+    // remove initial '/' character
+    if (folder.length() > 0 && folder[0] == '/')
+    {
+        folder = folder.length() == 1 ? folder.erase() : folder.substr(1);
+    }
 
-    s = "smb://";
+    uriString = "smb://";
 
-    if (!user.empty())
-        s += user + '@';
+    if (!domain.empty())
+        uriString = domain + ';';
 
-    s += server;
-    s += folder;
+    uriString += server;
 
-    return s;
+    uriString += "/";
+    uriString += folder;
+
+    return uriString;
 }
 #endif
+
+
+/**
+ * This function checks if the active or inactive file pane is showing
+ * files from the given GMount. If yes, close the connection to this GMount
+ * and open the home connection instead.
+ */
+void gnome_cmd_con_close_active_or_inactive_connection (GMount *gMount)
+{
+    g_return_if_fail(gMount != nullptr);
+    g_return_if_fail(G_IS_MOUNT(gMount));
+    auto gFile = g_mount_get_root(gMount);
+    auto uriString = g_file_get_uri(gFile);
+
+    auto activeConUri = gnome_cmd_con_get_uri(main_win->fs(ACTIVE)->get_connection());
+    auto activeConGFile = activeConUri ? g_file_new_for_uri(activeConUri) : nullptr;
+    auto inactiveConUri = gnome_cmd_con_get_uri(main_win->fs(INACTIVE)->get_connection());
+    auto inactiveConGFile = inactiveConUri ? g_file_new_for_uri(inactiveConUri) : nullptr;
+
+    if (activeConUri && g_file_equal(gFile, activeConGFile))
+    {
+        gnome_cmd_con_close (main_win->fs(ACTIVE)->get_connection());
+        main_win->fs(ACTIVE)->set_connection(get_home_con());
+    }
+    if (inactiveConUri && g_file_equal(gFile, inactiveConGFile))
+    {
+        gnome_cmd_con_close (main_win->fs(INACTIVE)->get_connection());
+        main_win->fs(INACTIVE)->set_connection(get_home_con());
+    }
+
+    g_free(uriString);
+    g_object_unref(gFile);
+}
\ No newline at end of file
