@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-con.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ enum ConnectionMethodID        // Keep this order in sync with strings in gnome-
     CON_DAV,
     CON_DAVS,
     CON_URI,
-    CON_LOCAL      // CON_FILE ???
+    CON_FILE
 };
 
 struct GnomeCmdCon
@@ -75,26 +75,21 @@ struct GnomeCmdCon
         OPEN_NOT_STARTED
     };
 
-    enum Authentication
-    {
-        SAVE_NEVER,
-        SAVE_FOR_SESSION,
-        SAVE_PERMANENTLY,
-        NOT_REQUIRED
-    };
-
     gchar               *alias;                 // coded as UTF-8
     gchar               *uri;
+    gchar               *scheme;
     ConnectionMethodID  method;
-    Authentication      auth;
 
+    gchar               *username;
+    gchar               *hostname;
+    gint16              port{-1};
     gchar               *open_msg;
     GnomeCmdPath        *base_path;
-    GnomeVFSFileInfo    *base_info;
-    GString             *root_path;             // root path of the connection, used for calculation of relative paths
+    GFileInfo           *base_gFileInfo;
+    GString             *root_path;             // Root path of the connection, used for calculation of relative paths
     gboolean            should_remember_dir;
     gboolean            needs_open_visprog;
-    gboolean            needs_list_visprog;
+    gboolean            needs_list_visprog;     // Defines if a graphical progress bar should be drawn when opening a folder
     gboolean            can_show_free_space;
     State               state;
     gboolean            is_local;
@@ -110,7 +105,7 @@ struct GnomeCmdCon
     GnomeCmdPixmap      *close_pixmap;
 
     OpenResult          open_result;
-    GnomeVFSResult      open_failed_reason;
+    GError              *open_failed_error;
     gchar               *open_failed_msg;
 
     GnomeCmdConPrivate  *priv;
@@ -123,20 +118,23 @@ struct GnomeCmdConClass
     /* signals */
     void (* updated) (GnomeCmdCon *con);
     void (* open_done) (GnomeCmdCon *con);
-    void (* open_failed) (GnomeCmdCon *con, const gchar *msg, GnomeVFSResult result);
+    void (* open_failed) (GnomeCmdCon *con);
 
     /* virtual functions */
     void (* open) (GnomeCmdCon *con);
     void (* cancel_open) (GnomeCmdCon *con);
     gboolean (* close) (GnomeCmdCon *con);
     gboolean (* open_is_needed) (GnomeCmdCon *con);
-    GnomeVFSURI *(* create_uri) (GnomeCmdCon *con, GnomeCmdPath *path);
+    GFile *(* create_gfile) (GnomeCmdCon *con, GnomeCmdPath *path);
     GnomeCmdPath *(* create_path) (GnomeCmdCon *con, const gchar *path_str);
 };
 
 
 GtkType gnome_cmd_con_get_type ();
 
+void gnome_cmd_con_set_base_path(GnomeCmdCon *con, GnomeCmdPath *path);
+void set_con_base_path_for_gmount(GnomeCmdCon *con, GMount *gMount);
+gboolean set_con_base_gfileinfo(GnomeCmdCon *con);
 
 void gnome_cmd_con_open (GnomeCmdCon *con);
 
@@ -163,11 +161,11 @@ inline const gchar *gnome_cmd_con_get_uri (GnomeCmdCon *con)
     return con->uri;
 }
 
-inline void gnome_cmd_con_set_uri (GnomeCmdCon *con, const gchar *uri=NULL)
+inline void gnome_cmd_con_set_uri (GnomeCmdCon *con, const gchar *uri = nullptr)
 {
     g_return_if_fail (GNOME_CMD_IS_CON (con));
     g_free (con->uri);
-    con->uri = (gchar *) uri;
+    con->uri = g_strdup(uri);
 }
 
 inline void gnome_cmd_con_set_uri (GnomeCmdCon *con, const std::string &uri)
@@ -177,7 +175,27 @@ inline void gnome_cmd_con_set_uri (GnomeCmdCon *con, const std::string &uri)
     con->uri = uri.empty() ? NULL : g_strdup (uri.c_str());
 }
 
-GnomeVFSURI *gnome_cmd_con_create_uri (GnomeCmdCon *con, GnomeCmdPath *path);
+inline const gchar *gnome_cmd_con_get_scheme (GnomeCmdCon *con)
+{
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), NULL);
+    return con->scheme;
+}
+
+inline void gnome_cmd_con_set_scheme (GnomeCmdCon *con, const gchar *scheme = nullptr)
+{
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+    g_free (con->scheme);
+    con->scheme = g_strdup(scheme);
+}
+
+inline void gnome_cmd_con_set_scheme (GnomeCmdCon *con, const std::string &scheme)
+{
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+    g_free (con->scheme);
+    con->scheme = scheme.empty() ? NULL : g_strdup (scheme.c_str());
+}
+
+GFile *gnome_cmd_con_create_gfile (GnomeCmdCon *con, GnomeCmdPath *path = nullptr);
 
 GnomeCmdPath *gnome_cmd_con_create_path (GnomeCmdCon *con, const gchar *path_str);
 
@@ -207,13 +225,46 @@ inline void gnome_cmd_con_set_alias (GnomeCmdCon *con, const gchar *alias=NULL)
     con->close_text = g_strdup_printf (_("Disconnect from: %s"), alias);
 }
 
+inline const char *gnome_cmd_con_get_user_name (GnomeCmdCon *con)
+{
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), nullptr);
+    return con->username;
+}
+
+inline void gnome_cmd_con_set_user_name (GnomeCmdCon *con, const gchar *username)
+{
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+    g_free (con->username);
+    con->username = g_strdup(username);
+}
+
+inline const char *gnome_cmd_con_get_host_name (GnomeCmdCon *con)
+{
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), nullptr);
+    return con->hostname;
+}
+
 inline void gnome_cmd_con_set_host_name (GnomeCmdCon *con, const gchar *host)
 {
     g_return_if_fail (GNOME_CMD_IS_CON (con));
     g_free (con->open_msg);
+    g_free (con->hostname);
+    con->hostname = g_strdup(host);
     con->open_msg = g_strdup_printf (_("Connecting to %s\n"), host ? host : "<?>");
 }
 
+inline void gnome_cmd_con_set_port (GnomeCmdCon *con, gint16 port)
+{
+    g_return_if_fail (GNOME_CMD_IS_CON (con));
+    con->port = port;
+}
+
+inline gint16 gnome_cmd_con_get_port (GnomeCmdCon *con)
+{
+    g_return_val_if_fail (GNOME_CMD_IS_CON (con), 0);
+    return con->port;
+}
+
 inline void gnome_cmd_con_set_host_name (GnomeCmdCon *con, const std::string &host)
 {
     gnome_cmd_con_set_host_name (con, host.empty() ? NULL : host.c_str());
@@ -335,11 +386,11 @@ void gnome_cmd_con_erase_bookmark (GnomeCmdCon *con);
 
 void gnome_cmd_con_updated (GnomeCmdCon *con);
 
-GnomeVFSResult gnome_cmd_con_get_path_target_type (GnomeCmdCon *con, const gchar *path, GnomeVFSFileType *type);
+gboolean gnome_cmd_con_get_path_target_type (GnomeCmdCon *con, const gchar *path, GFileType *type);
 
-GnomeVFSResult gnome_cmd_con_mkdir (GnomeCmdCon *con, const gchar *path_str);
+gboolean gnome_cmd_con_mkdir (GnomeCmdCon *con, const gchar *path_str, GError *error);
 
-void gnome_cmd_con_add_to_cache (GnomeCmdCon *con, GnomeCmdDir *dir);
+void gnome_cmd_con_add_to_cache (GnomeCmdCon *con, GnomeCmdDir *dir, gchar *uri_str = nullptr);
 
 void gnome_cmd_con_remove_from_cache (GnomeCmdCon *con, GnomeCmdDir *dir);
 
@@ -377,13 +428,38 @@ inline gchar *gnome_cmd_con_get_free_space (GnomeCmdCon *con, GnomeCmdDir *dir,
     return retval;
 }
 
-inline ConnectionMethodID gnome_cmd_con_get_scheme (GnomeVFSURI *uri)
+inline ConnectionMethodID gnome_cmd_con_get_scheme (const gchar *uriString)
 {
-    const gchar *scheme = gnome_vfs_uri_get_scheme (uri);       // do not g_free
-    const gchar *user = gnome_vfs_uri_get_user_name (uri);      // do not g_free
+    gchar *scheme = nullptr;
+    gchar *user = nullptr;
+    GError *error = nullptr;
 
-    return gnome_vfs_uri_is_local (uri) ? CON_LOCAL :
+    g_uri_split_with_user (
+        uriString,
+        G_URI_FLAGS_HAS_PASSWORD,
+        &scheme, //scheme
+        &user, //user
+        nullptr, //password
+        nullptr, //auth_params
+        nullptr, //host
+        nullptr, //port
+        nullptr, //path
+        nullptr, //query
+        nullptr, //fragment
+        &error
+    );
+
+    if(error)
+    {
+        g_warning("gnome_cmd_con_get_scheme - g_uri_split_with_user error: %s", error->message);
+        g_error_free(error);
+        return CON_SSH;
+    }
+
+    ConnectionMethodID retValue = scheme == nullptr ? CON_FILE :
+           g_str_equal (scheme, "file") ? CON_FILE :
            g_str_equal (scheme, "ftp")  ? (user && g_str_equal (user, "anonymous") ? CON_ANON_FTP : CON_FTP) :
+           g_str_equal (scheme, "ftp")  ? CON_FTP :
            g_str_equal (scheme, "sftp") ? CON_SSH :
            g_str_equal (scheme, "dav")  ? CON_DAV :
            g_str_equal (scheme, "davs") ? CON_DAVS :
@@ -391,70 +467,77 @@ inline ConnectionMethodID gnome_cmd_con_get_scheme (GnomeVFSURI *uri)
            g_str_equal (scheme, "smb")  ? CON_SMB :
 #endif
                                           CON_URI;
+
+    g_free(user);
+    g_free(scheme);
+    return retValue;
 }
 
-std::string &__gnome_cmd_con_make_uri (std::string &s, const gchar *method, gboolean use_auth, std::string &server, std::string &port, std::string &folder, std::string &user, std::string &password);
+std::string &__gnome_cmd_con_make_uri (std::string &s, const gchar *method, std::string &server, std::string &port, std::string &folder);
 
 inline std::string &gnome_cmd_con_make_custom_uri (std::string &s, const std::string &uri)
 {
-    stringify (s, gnome_vfs_make_uri_from_input (uri.c_str()));
+    GError *error = nullptr;
+    auto gUri = g_uri_parse (uri.c_str(), G_URI_FLAGS_NONE, &error);
+    if (error != nullptr)
+    {
+        g_warning ("g_uri_parse error of \"%s\": %s", uri.c_str(), error->message);
+        g_error_free(error);
+        return s;
+    }
+    stringify (s, g_uri_to_string(gUri));
 
     if (!uri_is_valid (s))
         s.erase();
-
     return s;
 }
 
-inline std::string &gnome_cmd_con_make_ssh_uri (std::string &s, gboolean use_auth, std::string &server, std::string &port, std::string &folder, std::string &user, std::string &password)
+inline std::string &gnome_cmd_con_make_ssh_uri (std::string &s, std::string &server, std::string &port, std::string &folder)
 {
-    return __gnome_cmd_con_make_uri (s, "sftp://", use_auth, server, port, folder, user, password);
+    return __gnome_cmd_con_make_uri (s, "sftp://", server, port, folder);
 }
 
-inline std::string &gnome_cmd_con_make_ftp_uri (std::string &s, gboolean use_auth, std::string &server, std::string &port, std::string &folder, std::string &user, std::string &password)
+inline std::string &gnome_cmd_con_make_ftp_uri (std::string &s, std::string &server, std::string &port, std::string &folder)
 {
-    if (user=="anonymous")
-    {
-        use_auth = FALSE;
-        password = gnome_cmd_data_get_ftp_anonymous_password ();
-    }
-
-    return __gnome_cmd_con_make_uri (s, "ftp://", use_auth, server, port, folder, user, password);
+    return __gnome_cmd_con_make_uri (s, "ftp://", server, port, folder);
 }
 
 #ifdef HAVE_SAMBA
-std::string &gnome_cmd_con_make_smb_uri (std::string &s, gboolean use_auth, std::string &server, std::string &share, std::string &folder, std::string &domain, std::string &user, std::string &password);
+std::string &gnome_cmd_con_make_smb_uri (std::string &s, std::string &server, std::string &folder, std::string &domain);
 #endif
 
-inline std::string &gnome_cmd_con_make_dav_uri (std::string &s, gboolean use_auth, std::string &server, std::string &port, std::string &folder, std::string &user, std::string &password)
+inline std::string &gnome_cmd_con_make_dav_uri (std::string &s, std::string &server, std::string &port, std::string &folder)
 {
-    return __gnome_cmd_con_make_uri (s, "dav://", use_auth, server, port, folder, user, password);
+    return __gnome_cmd_con_make_uri (s, "dav://", server, port, folder);
 }
 
-inline std::string &gnome_cmd_con_make_davs_uri (std::string &s, gboolean use_auth, std::string &server, std::string &port, std::string &folder, std::string &user, std::string &password)
+inline std::string &gnome_cmd_con_make_davs_uri (std::string &s, std::string &server, std::string &port, std::string &folder)
 {
-    return __gnome_cmd_con_make_uri (s, "davs://", use_auth, server, port, folder, user, password);
+    return __gnome_cmd_con_make_uri (s, "davs://", server, port, folder);
 }
 
-inline std::string &gnome_cmd_con_make_uri (std::string &s, ConnectionMethodID method, gboolean use_auth, std::string &uri, std::string &server, std::string &share, std::string &port, std::string &folder, std::string &domain, std::string &user, std::string &password)
+inline std::string &gnome_cmd_con_make_uri (std::string &s, ConnectionMethodID method, std::string &uri, std::string &server, std::string &port, std::string &folder, std::string &domain)
 {
     switch (method)
     {
         case CON_FTP:
-        case CON_ANON_FTP:  return gnome_cmd_con_make_ftp_uri (s, use_auth, server, port, folder, user, password);
+        case CON_ANON_FTP:  return gnome_cmd_con_make_ftp_uri (s, server, port, folder);
 
-        case CON_SSH:       return gnome_cmd_con_make_ssh_uri (s, use_auth, server, port, folder, user, password);
+        case CON_SSH:       return gnome_cmd_con_make_ssh_uri (s, server, port, folder);
 
 #ifdef HAVE_SAMBA
-        case CON_SMB:       return gnome_cmd_con_make_smb_uri (s, use_auth, server, share, folder, domain, user, password);
+        case CON_SMB:       return gnome_cmd_con_make_smb_uri (s, server, folder, domain);
 #endif
 
-        case CON_DAV:       return gnome_cmd_con_make_dav_uri (s, use_auth, server, port, folder, user, password);
+        case CON_DAV:       return gnome_cmd_con_make_dav_uri (s, server, port, folder);
 
-        case CON_DAVS:      return gnome_cmd_con_make_davs_uri (s, use_auth, server, port, folder, user, password);
+        case CON_DAVS:      return gnome_cmd_con_make_davs_uri (s, server, port, folder);
 
         case CON_URI:       return gnome_cmd_con_make_custom_uri (s, uri);
 
-        case CON_LOCAL:
+        case CON_FILE:
         default:            return s;
     }
 }
+
+void gnome_cmd_con_close_active_or_inactive_connection (GMount *gMount);
