@@ -2,7 +2,7 @@
  * @file gnome-cmd-data.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,6 +66,7 @@ GcmdSettings *gcmd_settings_new (void);
 #define GCMD_SETTINGS_LEFT_MOUSE_BUTTON_UNSELECTS     "left-mouse-btn-unselects"
 #define GCMD_SETTINGS_RIGHT_MOUSE_BUTTON_MODE         "right-mouse-btn-mode"
 #define GCMD_SETTINGS_MIDDLE_MOUSE_BUTTON_MODE        "middle-mouse-btn-mode"
+#define GCMD_SETTINGS_USE_TRASH                       "delete-to-trash"
 #define GCMD_SETTINGS_ICON_SIZE                       "icon-size"
 #define GCMD_SETTINGS_DEV_ICON_SIZE                   "dev-icon-size"
 #define GCMD_SETTINGS_ICON_SCALE_QUALITY              "icon-scale-quality"
@@ -86,7 +87,6 @@ GcmdSettings *gcmd_settings_new (void);
 #define GCMD_SETTINGS_SAVE_TABS_ON_EXIT               "save-tabs-on-exit"
 #define GCMD_SETTINGS_SAVE_DIR_HISTORY_ON_EXIT        "save-dir-history-on-exit"
 #define GCMD_SETTINGS_SAVE_CMDLINE_HISTORY_ON_EXIT    "save-cmdline-history-on-exit"
-#define GCMD_SETTINGS_SAVE_SEARCH_HISTORY_ON_EXIT     "save-search-history-on-exit"
 #define GCMD_SETTINGS_ALWAYS_SHOW_TABS                "always-show-tabs"
 #define GCMD_SETTINGS_TAB_LOCK_INDICATOR              "tab-lock-indicator"
 #define GCMD_SETTINGS_MAIN_WIN_STATE                  "main-win-state"
@@ -95,8 +95,10 @@ GcmdSettings *gcmd_settings_new (void);
 #define GCMD_SETTINGS_MULTIPLE_INSTANCES              "allow-multiple-instances"
 #define GCMD_SETTINGS_QUICK_SEARCH_EXACT_MATCH_BEGIN  "quick-search-exact-match-begin"
 #define GCMD_SETTINGS_QUICK_SEARCH_EXACT_MATCH_END    "quick-search-exact-match-end"
-#define GCMD_SETTINGS_DEV_SKIP_MOUNTING               "dev-skip-mounting"
 #define GCMD_SETTINGS_DEV_ONLY_ICON                   "dev-only-icon"
+#ifdef HAVE_SAMBA
+#define GCMD_SETTINGS_SHOW_SAMBA_WORKGROUP_BUTTON     "show-samba-workgroup-button"
+#endif
 #define GCMD_SETTINGS_MAINMENU_VISIBILITY             "mainmenu-visibility"
 #define GCMD_SETTINGS_QUICK_SEARCH_SHORTCUT           "quick-search"
 #define GCMD_SETTINGS_MAIN_WIN_WIDTH                  "main-win-width"
@@ -125,11 +127,7 @@ GcmdSettings *gcmd_settings_new (void);
 #define GCMD_SETTINGS_FAV_APPS                        "favorite-apps"
 #define GCMD_SETTINGS_FAV_APPS_FORMAT_STRING          "(ssssubbb)"
 #define GCMD_SETTINGS_DIRECTORY_HISTORY               "directory-history"
-#define GCMD_SETTINGS_SEARCH_WIN_WIDTH                "search-win-width"
-#define GCMD_SETTINGS_SEARCH_WIN_HEIGHT               "search-win-height"
-#define GCMD_SETTINGS_SEARCH_WIN_IS_TRANSIENT         "search-win-is-transient"
 #define GCMD_SETTINGS_SEARCH_PATTERN_HISTORY          "search-pattern-history"
-#define GCMD_SETTINGS_SEARCH_TEXT_HISTORY             "search-text-history"
 #define GCMD_SETTINGS_SEARCH_PROFILES                 "search-profiles"
 #define GCMD_SETTINGS_SEARCH_PROFILE_FORMAT_STRING    "(siisbbs)"
 #define GCMD_SETTINGS_SEARCH_PROFILES_FORMAT_STRING   "a(siisbbs)"
@@ -148,13 +146,14 @@ GcmdSettings *gcmd_settings_new (void);
 #define GCMD_SETTINGS_FILTER_HIDE_UNKNOWN             "hide-unknown"
 #define GCMD_SETTINGS_FILTER_HIDE_REGULAR             "hide-regular"
 #define GCMD_SETTINGS_FILTER_HIDE_DIRECTORY           "hide-directory"
-#define GCMD_SETTINGS_FILTER_HIDE_FIFO                "hide-fifo"
-#define GCMD_SETTINGS_FILTER_HIDE_SOCKET              "hide-socket"
-#define GCMD_SETTINGS_FILTER_HIDE_CHARACTER_DEVICE    "hide-char-device"
-#define GCMD_SETTINGS_FILTER_HIDE_BLOCK_DEVICE        "hide-block-device"
-#define GCMD_SETTINGS_FILTER_HIDE_SYMBOLIC_LINK       "hide-symbolic-link"
-#define GCMD_SETTINGS_FILTER_DOTFILE                  "hide-dotfile"
-#define GCMD_SETTINGS_FILTER_BACKUP                   "hide-backup-files"
+#define GCMD_SETTINGS_FILTER_HIDE_SYMLINK             "hide-symlink"
+#define GCMD_SETTINGS_FILTER_HIDE_SPECIAL             "hide-special"
+#define GCMD_SETTINGS_FILTER_HIDE_SHORTCUT            "hide-shortcut"
+#define GCMD_SETTINGS_FILTER_HIDE_MOUNTABLE           "hide-mountable"
+#define GCMD_SETTINGS_FILTER_HIDE_VIRTUAL             "hide-virtual"
+#define GCMD_SETTINGS_FILTER_HIDE_VOLATILE            "hide-volatile"
+#define GCMD_SETTINGS_FILTER_HIDE_HIDDEN              "hide-dotfile"
+#define GCMD_SETTINGS_FILTER_HIDE_BACKUPS             "hide-backupfiles"
 #define GCMD_SETTINGS_FILTER_BACKUP_PATTERN           "backup-pattern"
 
 #define GCMD_PREF_CONFIRM                             "org.gnome.gnome-commander.preferences.confirmations"
@@ -198,6 +197,7 @@ GcmdSettings *gcmd_settings_new (void);
 #define GCMD_SETTINGS_VIEWER_CMD                      "viewer-cmd"
 #define GCMD_SETTINGS_EDITOR_CMD                      "editor-cmd"
 #define GCMD_SETTINGS_DIFFER_CMD                      "differ-cmd"
+#define GCMD_SETTINGS_SEARCH_CMD                      "search-cmd"
 #define GCMD_SETTINGS_SENDTO_CMD                      "sendto-cmd"
 #define GCMD_SETTINGS_TERMINAL_CMD                    "terminal-cmd"
 #define GCMD_SETTINGS_TERMINAL_EXEC_CMD               "terminal-exec-cmd"
@@ -205,7 +205,6 @@ GcmdSettings *gcmd_settings_new (void);
 
 #define GCMD_PREF_NETWORK                             "org.gnome.gnome-commander.preferences.network"
 #define GCMD_SETTINGS_QUICK_CONNECT_URI               "quick-connect-uri"
-#define GCMD_SETTINGS_FTP_ANONYMOUS_PASSWORD          "ftp-anonymous-password"
 
 #define GCMD_PREF_INTERNAL_VIEWER                     "org.gnome.gnome-commander.preferences.internal-viewer"
 #define GCMD_SETTINGS_IV_CASE_SENSITIVE               "case-sensitive-search"
@@ -280,18 +279,28 @@ struct GnomeCmdData
 
     enum {SEARCH_HISTORY_SIZE=10, ADVRENAME_HISTORY_SIZE=10, INTVIEWER_HISTORY_SIZE=16};
 
+    typedef enum
+    {
+        G_FILE_IS_UNKNOWN,
+        G_FILE_IS_REGULAR,
+        G_FILE_IS_DIR,
+        G_FILE_IS_SYMLINK,
+        G_FILE_IS_SPECIAL,
+        G_FILE_IS_SHORTCUT,
+        G_FILE_IS_MOUNTABLE,
+        G_FILE_IS_HIDDEN,
+        G_FILE_IS_BACKUP,
+        G_FILE_IS_VIRTUAL,
+        G_FILE_IS_VOLATILE
+    }GcmdFileType;
+
     struct FilterSettings
     {
-        gboolean file_types[8];
-
-        gboolean hidden;
-        gboolean backup;
+        gboolean file_types[11];
 
         FilterSettings()
         {
             memset(file_types, 0, sizeof(file_types));
-            hidden = TRUE;
-            backup = TRUE;
         }
     };
 
@@ -317,10 +326,9 @@ struct GnomeCmdData
         gboolean                     save_tabs_on_exit;
         gboolean                     save_dir_history_on_exit;
         gboolean                     save_cmdline_history_on_exit;
-        gboolean                     save_search_history_on_exit;
-        gboolean                     search_window_is_transient {true};
         gchar                       *symlink_prefix;
         gint                         main_win_pos[2];
+        gboolean                     deleteToTrash;
         // Format
         GnomeCmdSizeDispMode         size_disp_mode;
         GnomeCmdPermDispMode         perm_disp_mode;
@@ -344,7 +352,7 @@ struct GnomeCmdData
         GtkButtonsType               confirm_delete_default;
         GnomeCmdConfirmOverwriteMode confirm_copy_overwrite;
         GnomeCmdConfirmOverwriteMode confirm_move_overwrite;
-        gboolean                     confirm_mouse_dnd;
+        GnomeCmdDefaultDndMode       mouse_dnd_default;
         //  Filters
         FilterSettings               filter;
         gchar                       *backup_pattern;
@@ -355,13 +363,16 @@ struct GnomeCmdData
         gboolean                     use_internal_viewer;
         gchar                       *editor;
         gchar                       *differ;
+        gchar                       *search;
         gchar                       *sendto;
         gchar                       *termopen;
         gchar                       *termexec;
         GList                       *fav_apps;
         //  Devices
         gboolean                     device_only_icon;
-        gboolean                     skip_mounting;
+#ifdef HAVE_SAMBA
+        gboolean                     show_samba_workgroups_button;
+#endif
 
         Options(): gcmd_settings(nullptr),
                    left_mouse_button_mode(LEFT_BUTTON_OPENS_WITH_DOUBLE_CLICK),
@@ -378,8 +389,9 @@ struct GnomeCmdData
                    save_tabs_on_exit(TRUE),
                    save_dir_history_on_exit(TRUE),
                    save_cmdline_history_on_exit(TRUE),
-                   save_search_history_on_exit(TRUE),
                    symlink_prefix(nullptr),
+                   main_win_pos{0,25},
+                   deleteToTrash(TRUE),
                    size_disp_mode(GNOME_CMD_SIZE_DISP_MODE_POWERED),
                    perm_disp_mode(GNOME_CMD_PERM_DISP_MODE_TEXT),
                    date_format(nullptr),
@@ -398,7 +410,7 @@ struct GnomeCmdData
                    confirm_delete_default(GTK_BUTTONS_OK),
                    confirm_copy_overwrite(GNOME_CMD_CONFIRM_OVERWRITE_QUERY),
                    confirm_move_overwrite(GNOME_CMD_CONFIRM_OVERWRITE_QUERY),
-                   confirm_mouse_dnd(TRUE),
+                   mouse_dnd_default(GNOME_CMD_DEFAULT_DND_QUERY),
                    backup_pattern(nullptr),
                    backup_pattern_list(nullptr),
                    honor_expect_uris(FALSE),
@@ -406,12 +418,17 @@ struct GnomeCmdData
                    use_internal_viewer(TRUE),
                    editor(nullptr),
                    differ(nullptr),
+                   search(nullptr),
                    sendto(nullptr),
                    termopen(nullptr),
                    termexec(nullptr),
                    fav_apps(nullptr),
+#ifdef HAVE_SAMBA
                    device_only_icon(FALSE),
-                   skip_mounting(FALSE)
+                   show_samba_workgroups_button(FALSE)
+#else
+                   device_only_icon(FALSE)
+#endif
         {
             memset(&color_themes, 0, sizeof(color_themes));
             memset(&ls_colors_palette, 0, sizeof(ls_colors_palette));
@@ -430,6 +447,7 @@ struct GnomeCmdData
             g_free (viewer);
             g_free (editor);
             g_free (differ);
+            g_free (search);
             g_free (sendto);
             g_free (termopen);
             g_free (termexec);
@@ -492,6 +510,12 @@ struct GnomeCmdData
             differ = g_strdup (command);
         }
 
+        void set_search(const gchar *command)
+        {
+            g_free (search);
+            search = g_strdup (command);
+        }
+
         void set_sendto(const gchar *command)
         {
             g_free (sendto);
@@ -550,20 +574,14 @@ struct GnomeCmdData
 
     struct SearchConfig
     {
-        gint width, height;
-
         SearchProfile default_profile;
 
         History name_patterns;
-        History content_patterns;
 
         std::vector<SearchProfile> &profiles;
 
         explicit SearchConfig(std::vector<SearchProfile> &searchProfiles):
-            width(600),
-            height(400),
             name_patterns(SEARCH_HISTORY_SIZE),
-            content_patterns(SEARCH_HISTORY_SIZE),
             profiles(searchProfiles)
         {
             default_profile.name = "Default";
@@ -644,7 +662,7 @@ struct GnomeCmdData
     void save_connections();
     void save_cmdline_history();
     void save_directory_history();
-    void save_search_history();
+    void save_search_pattern_history();
     void save_intviewer_defaults();
     void save_devices();
     void save_fav_apps();
@@ -705,7 +723,7 @@ struct GnomeCmdData
     void load();
     void load_colors();
     void load_color_themes();
-    void load_tabs_from_gsettings();
+    void load_tabs();
     void load_devices();
     void load_fav_apps();
     void gsettings_init();
@@ -731,9 +749,6 @@ struct GnomeCmdData
 
 gpointer gnome_cmd_data_get_con_list ();
 
-const gchar *gnome_cmd_data_get_ftp_anonymous_password ();
-void gnome_cmd_data_set_ftp_anonymous_password (const gchar *pw);
-
 GList *gnome_cmd_data_get_auto_load_plugins ();
 void gnome_cmd_data_set_auto_load_plugins (GList *plugins);
 
