@@ -2,7 +2,7 @@
  * @file gnome-cmd-dir-indicator.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ GtkType gnome_cmd_dir_indicator_get_type ();
 
 GtkWidget *gnome_cmd_dir_indicator_new (GnomeCmdFileSelector *fs);
 
-void gnome_cmd_dir_indicator_set_dir (GnomeCmdDirIndicator *indicator, gchar *path);
+void gnome_cmd_dir_indicator_set_dir (GnomeCmdDirIndicator *indicator, GnomeCmdDir *dir);
 
 void gnome_cmd_dir_indicator_set_active (GnomeCmdDirIndicator *indicator, gboolean value);
 
