@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-dir.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ GType gnome_cmd_dir_get_type ();
 struct GnomeCmdDir;
 struct GnomeCmdDirPrivate;
 
-typedef void (* DirListDoneFunc) (GnomeCmdDir *dir, GList *files, GnomeVFSResult result);
+typedef void (* DirListDoneFunc) (GnomeCmdDir *dir, GList *files, GError *error);
 
 #include <string>
 
@@ -59,10 +59,10 @@ struct GnomeCmdDir
 
     gint voffset;
     GList *infolist;
-    GnomeVFSAsyncHandle *list_handle;
-    GnomeVFSResult list_result;
+    GList *gFileInfoList;
     gint list_counter;
     State state;
+    GError *error;
 
     DirListDoneFunc done_func;
 
@@ -80,14 +80,14 @@ struct GnomeCmdDirClass
     void (* file_changed)       (GnomeCmdDir *dir, GnomeCmdFile *file);
     void (* file_renamed)       (GnomeCmdDir *dir, GnomeCmdFile *file);
     void (* list_ok)            (GnomeCmdDir *dir, GList *files);
-    void (* list_failed)        (GnomeCmdDir *dir, GnomeVFSResult result);
+    void (* list_failed)        (GnomeCmdDir *dir, GError *error);
 };
 
 struct GnomeCmdCon;
 
-GnomeCmdDir *gnome_cmd_dir_new_from_info (GnomeVFSFileInfo *info, GnomeCmdDir *parent);
+GnomeCmdDir *gnome_cmd_dir_new_from_gfileinfo (GFileInfo *gFileInfo, GnomeCmdDir *parent);
 GnomeCmdDir *gnome_cmd_dir_new_with_con (GnomeCmdCon *con);
-GnomeCmdDir *gnome_cmd_dir_new (GnomeCmdCon *con, GnomeCmdPath *path);
+GnomeCmdDir *gnome_cmd_dir_new (GnomeCmdCon *con, GnomeCmdPath *path, gboolean isStartup = false);
 GnomeCmdDir *gnome_cmd_dir_get_parent (GnomeCmdDir *dir);
 GnomeCmdDir *gnome_cmd_dir_get_child (GnomeCmdDir *dir, const gchar *child);
 GnomeCmdCon *gnome_cmd_dir_get_connection (GnomeCmdDir *dir);
@@ -95,19 +95,16 @@ Handle *gnome_cmd_dir_get_handle (GnomeCmdDir *dir);
 
 inline GnomeCmdFile *gnome_cmd_dir_new_parent_dir_file (GnomeCmdDir *dir)
 {
-    GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+    auto info = g_file_info_new ();
 
-    memset (info, '\0', sizeof (GnomeVFSFileInfo));
-    info->name = g_strdup ("..");
-    info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
-    info->mime_type = g_strdup ("x-directory/normal");
-    info->size = 0;
-    info->refcount = 1;
-    info->valid_fields = (GnomeVFSFileInfoFields) (GNOME_VFS_FILE_INFO_FIELDS_TYPE |
-                                                   GNOME_VFS_FILE_INFO_FIELDS_SIZE |
-                                                   GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE);
+    g_file_info_set_name(info, "..");
+    g_file_info_set_display_name(info, "..");
+    g_file_info_set_file_type(info, G_FILE_TYPE_DIRECTORY);
+    g_file_info_set_size(info, 0);
 
-    return gnome_cmd_file_new (info, dir);
+    auto gnomeCmdFile = gnome_cmd_file_new (info, dir);
+    gnomeCmdFile->ref();
+    return gnomeCmdFile;
 }
 
 inline GnomeCmdDir *gnome_cmd_dir_ref (GnomeCmdDir *dir)
@@ -128,12 +125,17 @@ void gnome_cmd_dir_set_path (GnomeCmdDir *dir, GnomeCmdPath *path);
 void gnome_cmd_dir_update_path (GnomeCmdDir *dir);
 gchar *gnome_cmd_dir_get_display_path (GnomeCmdDir *dir);
 
-GnomeVFSURI *gnome_cmd_dir_get_uri (GnomeCmdDir *dir);
-gchar       *gnome_cmd_dir_get_uri_str (GnomeCmdDir *dir);
+GFile       *gnome_cmd_dir_get_gfile (GnomeCmdDir *dir);
+gchar       *gnome_cmd_dir_get_uri_str (GnomeCmdDir *dir, gboolean withTrailingSlash = false);
 
-GnomeVFSURI *gnome_cmd_dir_get_child_uri (GnomeCmdDir *dir, const gchar *filename);
+gchar *gnome_cmd_dir_get_relative_path_string(const char* childPathString, const char* basePath);
+GFile *gnome_cmd_dir_get_gfile_for_con_and_filename(GnomeCmdDir *dir, const gchar *filename);
+GFile *gnome_cmd_dir_get_child_gfile (GnomeCmdDir *dir, const gchar *filename);
 
-GnomeVFSURI *gnome_cmd_dir_get_absolute_path_uri (GnomeCmdDir *dir, std::string absolute_filename);
+GFile *gnome_cmd_dir_get_absolute_path_gfile (GnomeCmdDir *dir, std::string absolute_filename);
+
+GnomeCmdDir *gnome_cmd_dir_get_existing_parent(GnomeCmdDir *dir);
+void gnome_cmd_dir_update_file_selector(GnomeCmdDir *dir, const gchar *deletedDirUriString);
 
 void gnome_cmd_dir_file_created (GnomeCmdDir *dir, const gchar *uri_str);
 void gnome_cmd_dir_file_deleted (GnomeCmdDir *dir, const gchar *uri_str);
@@ -153,13 +155,24 @@ inline gchar *gnome_cmd_dir_get_free_space (GnomeCmdDir *dir)
 {
     g_return_val_if_fail (GNOME_CMD_IS_DIR (dir), NULL);
 
-    GnomeVFSFileSize free_space;
-    GnomeVFSURI *uri = GNOME_CMD_FILE (dir)->get_uri();
-    GnomeVFSResult res = gnome_vfs_get_volume_free_space (uri, &free_space);
-    gnome_vfs_uri_unref (uri);
+    GError *error = nullptr;
 
-    if (res!=GNOME_VFS_OK)
-        return NULL;
+    auto gFileInfo = g_file_query_filesystem_info (GNOME_CMD_FILE (dir)->gFile,
+                              G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
+                              nullptr,
+                              &error);
 
-    return gnome_vfs_format_file_size_for_display (free_space);
+    if (error)
+    {
+        g_warning("Could not g_file_query_filesystem_info %s: %s\n",
+            g_file_peek_path(GNOME_CMD_FILE (dir)->gFile), error->message);
+        g_error_free(error);
+        return nullptr;
+    }
+
+    auto freeSpace = g_file_info_get_attribute_uint64(gFileInfo, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+
+    g_object_unref(gFileInfo);
+
+    return g_format_size (freeSpace);
 }
