@@ -2,7 +2,7 @@
  * @file gnome-cmd-file-collection.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,10 @@ void GnomeCmdFileCollection::add(GnomeCmdFile *f)
     list = g_list_append (list, f);
 
     gchar *uri_str = f->get_uri_str();
+    if (!uri_str)
+    {
+         return;
+    }
     g_hash_table_insert (map, uri_str, f);
     f->ref();
 }
