@@ -2,7 +2,7 @@
  * @file gnome-cmd-file-list.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -126,6 +126,8 @@ struct GnomeCmdFileList
     void unselect_file(GnomeCmdFile *f, gint row=-1);
     void select_all();
     void unselect_all();
+    void select_all_files();
+    void unselect_all_files();
 
     void toggle_file(GnomeCmdFile *f);
     void toggle();
@@ -151,7 +153,7 @@ struct GnomeCmdFileList
      * Returns a list with all files shown in the file list. The list is
      * the same as that in the file list it self so make a copy and ref
      * the files if needed
-     */    
+     */
     GList *get_visible_files();
 
     /**
@@ -160,13 +162,13 @@ struct GnomeCmdFileList
      * list is however not refed before returning
      */
     GList *get_selected_files();
-    
+
     /**
      * Returns a collection of all selected files.
      * A marked file is a file that has been selected with ins etc. The file that is currently focused is not marked.
      */
     GnomeCmd::Collection<GnomeCmdFile *> &get_marked_files();
-                                              
+
     /**
      * Returns the currently focused file if any. The returned file is
      * not reffed. The ".." file is returned if focused
@@ -215,8 +217,15 @@ struct GnomeCmdFileList
 
     gboolean key_pressed(GdkEventKey *event);
 
+    enum DndMode
+    {
+        COPY,
+        MOVE,
+        LINK
+    };
+
     void init_dnd();
-    void drop_files(GnomeVFSXferOptions xferOptions, GList *uri_list, GnomeCmdDir *dir);
+    void drop_files(DndMode dndMode, GFileCopyFlags gFileCopyFlags, GList *uri_list, GnomeCmdDir *dir);
 };
 
 
@@ -249,7 +258,7 @@ inline GnomeCmdFile *GnomeCmdFileList::get_selected_file()
     return !f || f->is_dotdot ? nullptr : f;
 }
 
-void gnome_cmd_file_list_show_delete_dialog (GnomeCmdFileList *fl);
+void gnome_cmd_file_list_show_delete_dialog (GnomeCmdFileList *fl, gboolean forceDelete = FALSE);
 void gnome_cmd_file_list_show_properties_dialog (GnomeCmdFileList *fl);
 void gnome_cmd_file_list_show_rename_dialog (GnomeCmdFileList *fl);
 void gnome_cmd_file_list_show_selpat_dialog (GnomeCmdFileList *fl, gboolean mode);
