@@ -2,7 +2,7 @@
  * @file gnome-cmd-file-popmenu.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "gnome-cmd-plain-path.h"
 #include "gnome-cmd-xfer.h"
 #include "imageloader.h"
+#include "dirlist.h"
 
 #include <fnmatch.h>
 
@@ -69,38 +70,37 @@ struct GnomeCmdFilePopmenuPrivate
     GList *data_list;
 };
 
-
 static void do_mime_exec_multiple (gpointer *args)
 {
-    auto app = static_cast<GnomeCmdApp*> (args[0]);
+    auto gnomeCmdApp = static_cast<GnomeCmdApp*> (args[0]);
     auto files = static_cast<GList*> (args[1]);
-
     if (files)
     {
-        string cmdString = gnome_cmd_app_get_command (app);
-
-        set<string> dirs;
-
-        for (; files; files = files->next)
+        if(gnomeCmdApp->gAppInfo != nullptr)
         {
-            cmdString += ' ';
-            cmdString += stringify (g_shell_quote ((gchar *) files->data));
-
-            gchar *dpath = g_path_get_dirname ((gchar *) files->data);
-
-            if (dpath)
-                dirs.insert (stringify (dpath));
+            // gio app default by system
+            DEBUG('g', "Launching \"%s\"\n", g_app_info_get_commandline(gnomeCmdApp->gAppInfo));
+            g_app_info_launch(gnomeCmdApp->gAppInfo, files, nullptr, nullptr);
         }
-
-        if (dirs.size()==1)
-            run_command_indir (cmdString.c_str(), dirs.begin()->c_str(), gnome_cmd_app_get_requires_terminal (app));
         else
-            run_command_indir (cmdString.c_str(), nullptr, gnome_cmd_app_get_requires_terminal (app));
-
+        {
+            // own app by user defined
+            string cmdString = gnome_cmd_app_get_command (gnomeCmdApp);
+            set<string> dirs;
+            for (auto files_tmp = files; files_tmp; files_tmp = files_tmp->next)
+            {
+                auto gFile = (GFile *) files_tmp->data;
+                auto localpath = g_file_get_path(gFile);
+                auto dpath = g_path_get_dirname (localpath);
+                if (dpath)
+                    dirs.insert (stringify (dpath));
+                g_free(localpath);
+            }
+            run_command_indir (cmdString.c_str(), dirs.begin()->c_str(), gnome_cmd_app_get_requires_terminal (gnomeCmdApp));
+        }
         g_list_free (files);
     }
-
-    gnome_cmd_app_free (app);
+    gnome_cmd_app_free (gnomeCmdApp);
     g_free (args);
 }
 
@@ -110,25 +110,28 @@ static void mime_exec_multiple (GList *files, GnomeCmdApp *app)
     g_return_if_fail (files != nullptr);
     g_return_if_fail (app != nullptr);
 
-    GList *src_uri_list = nullptr;
-    GList *dest_uri_list = nullptr;
-    GList *local_files = nullptr;
+    GList *srcGFileList = nullptr;
+    GList *destGFileList = nullptr;
+    GList *localGFileList = nullptr;
     gboolean asked = FALSE;
     guint no_of_remote_files = 0;
     gint retid;
 
     for (; files; files = files->next)
     {
-        auto f = static_cast<GnomeCmdFile*> (files->data);
+        auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
+        auto scheme = g_file_get_uri_scheme (gnomeCmdFile->gFile);
 
-        if (gnome_vfs_uri_is_local (f->get_uri()))
-            local_files = g_list_append (local_files, f->get_real_path());
+        if (g_strcmp0(scheme, "file") == 0)
+        {
+            localGFileList = g_list_append (localGFileList, gnomeCmdFile->gFile);
+        }
         else
         {
             ++no_of_remote_files;
             if (gnome_cmd_app_get_handles_uris (app) && gnome_cmd_data.options.honor_expect_uris)
             {
-                local_files = g_list_append (local_files,  f->get_uri_str());
+                localGFileList = g_list_append (localGFileList, gnomeCmdFile->gFile);
             }
             else
             {
@@ -143,37 +146,37 @@ static void mime_exec_multiple (GList *files, GnomeCmdApp *app)
 
                 if (retid==1)
                 {
-                    gchar *path_str = get_temp_download_filepath (f->get_name());
+                    gchar *path_str = get_temp_download_filepath (gnomeCmdFile->get_name());
 
                     if (!path_str) return;
 
-                    GnomeVFSURI *src_uri = gnome_vfs_uri_dup (f->get_uri());
+                    auto srcGFile = g_file_dup (gnomeCmdFile->get_gfile());
                     GnomeCmdPlainPath path(path_str);
-                    GnomeVFSURI *dest_uri = gnome_cmd_con_create_uri (get_home_con (), &path);
+                    auto destGFile = gnome_cmd_con_create_gfile (get_home_con (), &path);
 
-                    src_uri_list = g_list_append (src_uri_list, src_uri);
-                    dest_uri_list = g_list_append (dest_uri_list, dest_uri);
-                    local_files = g_list_append (local_files, path_str);
+                    srcGFileList = g_list_append (srcGFileList, srcGFile);
+                    destGFileList = g_list_append (destGFileList, destGFile);
+                    localGFileList = g_list_append (localGFileList, destGFile);
                 }
             }
         }
+        g_free(scheme);
     }
 
     g_list_free (files);
 
     gpointer *args = g_new0 (gpointer, 2);
     args[0] = app;
-    args[1] = local_files;
+    args[1] = localGFileList;
 
-    if (src_uri_list)
-        gnome_cmd_xfer_tmp_download_multiple (src_uri_list,
-                                              dest_uri_list,
-                                              GNOME_VFS_XFER_FOLLOW_LINKS,
-                                              GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-                                              GTK_SIGNAL_FUNC (do_mime_exec_multiple),
-                                              args);
+    if (srcGFileList)
+        gnome_cmd_tmp_download(srcGFileList,
+                               destGFileList,
+                               G_FILE_COPY_OVERWRITE,
+                               GTK_SIGNAL_FUNC (do_mime_exec_multiple),
+                               args);
     else
-        do_mime_exec_multiple (args);
+      do_mime_exec_multiple (args);
 }
 
 
@@ -211,26 +214,26 @@ static void cb_exec_default (GtkMenuItem *menu_item, GList *files)
 
     for (; files; files = files->next)
     {
-        auto f = static_cast<GnomeCmdFile*> (files->data);
-        auto vfs_app = f->get_default_application();
+        auto file = static_cast<GnomeCmdFile*> (files->data);
+        auto appInfo = file->GetAppInfoForContentType();
 
-        if (vfs_app)
+        if (g_app_info_get_commandline (appInfo))
         {
-            auto data = static_cast<OpenWithData*> (g_hash_table_lookup (gHashTable, vfs_app->id));
+            auto data = static_cast<OpenWithData*> (g_hash_table_lookup (gHashTable, g_app_info_get_id(appInfo)));
 
             if (!data)
             {
                 data = g_new0 (OpenWithData, 1);
-                data->app = gnome_cmd_app_new_from_vfs_app (vfs_app);
+                data->app = gnome_cmd_app_new_from_app_info (appInfo);
                 data->files = nullptr;
-                g_hash_table_insert (gHashTable, vfs_app->id, data);
+                g_hash_table_insert (gHashTable, (gpointer) g_app_info_get_id(appInfo), data);
             }
-
-            gnome_vfs_mime_application_free (vfs_app);
-            data->files = g_list_append (data->files, f);
+            data->files = g_list_append (data->files, file);
         }
         else
-            gnome_cmd_show_message (nullptr, f->info->name, _("Couldn’t retrieve MIME type of the file."));
+            gnome_cmd_show_message (nullptr,
+                g_file_info_get_display_name(file->gFileInfo),
+                _("Couldn’t retrieve MIME type of the file."));
     }
 
     g_hash_table_foreach (gHashTable, (GHFunc) htcb_exec_with_app, nullptr);
@@ -313,7 +316,7 @@ static void on_execute_script (GtkMenuItem *menuItem, ScriptData *scriptData)
         {
             auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
             string command (scriptData->path);
-            command.append (" ").append (quotationMarks).append (gnomeCmdFile->get_name ()).append (quotationMarks);
+            command.append (" ").append(gnomeCmdFile->get_quoted_name());
 
             dirName = gnomeCmdFile->get_dirname (); // must be g_free'd
             run_command_indir (command.c_str (), dirName, scriptData->inTerminal);
@@ -328,7 +331,7 @@ static void on_execute_script (GtkMenuItem *menuItem, ScriptData *scriptData)
         for (auto files = scriptData->files; files; files = files->next)
         {
             auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
-            commandString.append (quotationMarks).append(gnomeCmdFile->get_name ()).append (quotationMarks).append (" ");
+            commandString.append(gnomeCmdFile->get_quoted_name()).append(" ");
         }
 
         dirName = static_cast<GnomeCmdFile*> (scriptData->files->data)->get_dirname (); // must be g_free'd
@@ -385,13 +388,15 @@ static void add_fav_app_menu_item (GtkUIManager *uiManager, guint mergeIdFavApp,
 
 inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
 {
+
     switch (gnome_cmd_app_get_target (app))
     {
         case APP_TARGET_ALL_DIRS:
             for (; files; files = files->next)
             {
-                auto f = static_cast<GnomeCmdFile*> (files->data);
-                if (f->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
+                auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
+                if (gnomeCmdFile->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE)
+                      != G_FILE_TYPE_DIRECTORY)
                     return FALSE;
             }
             return TRUE;
@@ -399,8 +404,9 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
         case APP_TARGET_ALL_FILES:
             for (; files; files = files->next)
             {
-                auto f = static_cast<GnomeCmdFile*> (files->data);
-                if (f->info->type != GNOME_VFS_FILE_TYPE_REGULAR)
+                auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
+                if (gnomeCmdFile->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE)
+                      != G_FILE_TYPE_REGULAR)
                     return FALSE;
             }
             return TRUE;
@@ -408,9 +414,9 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
         case APP_TARGET_ALL_DIRS_AND_FILES:
             for (; files; files = files->next)
             {
-                auto f = static_cast<GnomeCmdFile*> (files->data);
-                if (f->info->type != GNOME_VFS_FILE_TYPE_REGULAR
-                    && f->info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
+                auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
+                auto gFileType = gnomeCmdFile->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE);
+                if (gFileType != G_FILE_TYPE_REGULAR && gFileType != G_FILE_TYPE_DIRECTORY)
                     return FALSE;
             }
             return TRUE;
@@ -425,8 +431,9 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
                 gint fn_flags = FNM_NOESCAPE;
 #endif
 
-                auto f = static_cast<GnomeCmdFile*> (files->data);
-                if (f->info->type != GNOME_VFS_FILE_TYPE_REGULAR)
+                auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
+                if (gnomeCmdFile->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE)
+                      != G_FILE_TYPE_REGULAR)
                     return FALSE;
 
                 // Check that the file matches at least one pattern
@@ -434,7 +441,7 @@ inline gboolean fav_app_matches_files (GnomeCmdApp *app, GList *files)
                 for (; patterns; patterns = patterns->next)
                 {
                     auto pattern = (gchar *) patterns->data;
-                    ok |= fnmatch (pattern, f->info->name, fn_flags) == 0;
+                    ok |= fnmatch (pattern, g_file_info_get_display_name(gnomeCmdFile->gFileInfo), fn_flags) == 0;
                 }
 
                 if (!ok) return FALSE;
@@ -470,6 +477,8 @@ static void destroy (GtkObject *object)
     g_list_foreach (menu->priv->data_list, (GFunc) g_free, nullptr);
     g_list_free (menu->priv->data_list);
 
+    g_free(menu->scriptsDir);
+
     g_free (menu->priv);
 
     if (GTK_OBJECT_CLASS (parent_class)->destroy)
@@ -500,132 +509,43 @@ static void init (GnomeCmdFilePopmenu *menu)
 {
     menu->priv = g_new0 (GnomeCmdFilePopmenuPrivate, 1);
 
+    menu->scriptsDir = g_build_filename (g_get_user_config_dir (), PACKAGE SCRIPT_DIRECTORY, nullptr);
+
     // Create scripts directory if needed
-    gchar *scripts_dir = g_build_filename (g_get_user_config_dir (), PACKAGE "/scripts", nullptr);
-    create_dir_if_needed(scripts_dir);
-    g_free (scripts_dir);
-}
-
-
-inline gchar *string_double_underscores (const gchar *string)
-{
-    if (!string)
-        return nullptr;
-
-    int underscores = 0;
-
-    for (const gchar *p = string; *p; p++)
-        underscores += (*p == '_');
-
-    if (underscores == 0)
-        return g_strdup (string);
-
-    gchar *escaped = g_new (char, strlen (string) + underscores + 1);
-    gchar *q = escaped;
-
-    for (const gchar *p = string; *p; p++, q++)
+    if (!is_dir_existing(menu->scriptsDir))
     {
-        /* Add an extra underscore. */
-        if (*p == '_')
-            *q++ = '_';
-        *q = *p;
+        create_dir(menu->scriptsDir);
     }
-    *q = '\0';
-
-    return escaped;
 }
 
 
-inline GnomeVFSMimeApplication *get_default_gnome_vfs_app_for_mime_type(GnomeCmdFile *gnomeCmdFile)
-{
-    auto uri_str = gnomeCmdFile->get_uri_str();
-    auto *app = gnome_vfs_mime_get_default_application_for_uri (uri_str, gnomeCmdFile->info->mime_type);
-    g_free (uri_str);
-    return app;
-}
-
-
-inline gchar *get_default_application_action_name (GnomeCmdFile *gnomeCmdFile)
-{
-    auto app = get_default_gnome_vfs_app_for_mime_type(gnomeCmdFile);
-
-    if (!app)
-    {
-        return nullptr;
-    }
-
-    gchar *escaped_app_name = string_double_underscores (app->name);
-    gnome_vfs_mime_application_free (app);
-
-    return escaped_app_name;
-}
-
-
-inline gchar *get_default_application_icon_path (GnomeCmdFile *gnomeCmdFile)
-{
-    gchar* icon_path = nullptr;
-
-    auto app = get_default_gnome_vfs_app_for_mime_type(gnomeCmdFile);
-
-    if (app)
-    {
-        GnomeCmdApp *gapp = gnome_cmd_app_new_from_vfs_app (app);
-        if (gapp)
-        {
-            icon_path = g_strdup (gapp->icon_path);
-            gnome_cmd_app_free (gapp);
-        }
-        gnome_vfs_mime_application_free (app);
-    }
-
-    return icon_path;
-}
-
-
-inline gchar *get_default_application_action_label (GnomeCmdFile *gnomeCmdFile)
-{
-    gchar *escaped_app_name = get_default_application_action_name (gnomeCmdFile);
-    if (escaped_app_name == nullptr)
-    {
-        return g_strdup (_("_Open"));
-    }
-
-    gchar *retval = g_strdup_printf (_("_Open with “%s”"), escaped_app_name);
-    g_free (escaped_app_name);
-
-    return retval;
-}
-
 inline GList *get_list_of_action_script_file_names(const gchar* scriptsDir)
 {
-    DIR *dp = opendir (scriptsDir);
+    g_return_val_if_fail (scriptsDir != nullptr, nullptr);
+
+    auto gFileInfoList = sync_dir_list(scriptsDir);
+
+    if (g_list_length (gFileInfoList) == 0)
+        return nullptr;
+
     GList *scriptList = nullptr;
-    if (dp != nullptr)
+    for (auto gFileInfoListItem = gFileInfoList; gFileInfoListItem; gFileInfoListItem = gFileInfoListItem->next)
     {
-        struct dirent *directoryEntry;
-        while ((directoryEntry = readdir (dp)))
+        auto gFileInfo = (GFileInfo*) gFileInfoListItem->data;
+        if (g_file_info_get_file_type(gFileInfo) == G_FILE_TYPE_REGULAR)
         {
-            char *fileName = directoryEntry->d_name;
-            struct stat buf;
-            string scriptPath (scriptsDir);
-            scriptPath.append ("/").append (fileName);
-            if (stat (scriptPath.c_str(), &buf) == 0)
-            {
-                if (buf.st_mode & S_IFREG)
-                {
-                    DEBUG('p', "Adding \'%s\' to the list of scripts.\n", scriptPath.c_str());
-                    scriptList = g_list_append (scriptList, g_strdup(fileName));
-                }
-            }
+            DEBUG('p', "Adding \'%s\' to the list of scripts.\n", g_file_info_get_name(gFileInfo));
+            scriptList = g_list_append (scriptList, g_strdup(g_file_info_get_name(gFileInfo)));
         }
-        closedir (dp);
+
     }
+    g_list_free_full(gFileInfoList, g_object_unref);
     return scriptList;
 }
 
 guint add_action_script_entries(GtkUIManager *uiManager, GList *files)
 {
-    guint pos = 0;
+    guint position = 0;
     auto scriptsDir = g_build_filename (g_get_user_config_dir (), SCRIPT_DIRECTORY, nullptr);
     auto scriptFileNames = get_list_of_action_script_file_names(scriptsDir);
 
@@ -641,8 +561,8 @@ guint add_action_script_entries(GtkUIManager *uiManager, GList *files)
     }
     mergeIdActionScripts = gtk_ui_manager_new_merge_id (uiManager);
 
-    static GtkActionGroup *dynamicActionGroup = gtk_action_group_new ("actionScripts");
-    gtk_ui_manager_insert_action_group (uiManager, dynamicActionGroup, 0);
+    static auto gtkActionGroup = gtk_action_group_new ("actionScripts");
+    gtk_ui_manager_insert_action_group (uiManager, gtkActionGroup, 0);
 
     for (GList *scriptFileName = scriptFileNames; scriptFileName; scriptFileName = scriptFileName->next)
     {
@@ -657,6 +577,7 @@ guint add_action_script_entries(GtkUIManager *uiManager, GList *files)
         scriptPath.append ("/").append ((char*) scriptFileName->data);
         scriptData->path = g_strdup (scriptPath.c_str());
 
+        // Try to get the scriptname out of the script, otherwise take the filename
         char *scriptName = nullptr;
         FILE *fileStream = fopen (scriptData->path, "r");
         if (fileStream)
@@ -677,27 +598,30 @@ guint add_action_script_entries(GtkUIManager *uiManager, GList *files)
             }
             fclose (fileStream);
         }
+        scriptName = scriptName ? scriptName : g_strdup((char*) scriptFileName->data);
 
         auto dynAction = gtk_action_new (scriptName, scriptName, nullptr, GTK_STOCK_EXECUTE);
 
         // remove action if it was added in a previous run of this method
-        if (gtk_action_group_get_action (dynamicActionGroup, scriptName))
+        if (gtk_action_group_get_action (gtkActionGroup, scriptName))
         {
-            gtk_action_group_remove_action(dynamicActionGroup, dynAction);
+            gtk_action_group_remove_action(gtkActionGroup, dynAction);
         }
 
         g_signal_connect (dynAction, "activate", G_CALLBACK (on_execute_script), scriptData);
-        gtk_action_group_add_action (dynamicActionGroup, dynAction);
+        gtk_action_group_add_action (gtkActionGroup, dynAction);
 
         gtk_ui_manager_add_ui (uiManager, mergeIdActionScripts, "/FilePopup/ActionScripts", scriptName,
                                scriptName, GTK_UI_MANAGER_AUTO, true);
         g_free(scriptName);
-        pos++;
+        g_free(scriptData->path);
+        g_free(scriptData);
+        position++;
     }
     g_free (scriptsDir);
     g_list_free_full(scriptFileNames, g_free);
 
-    return pos;
+    return position;
 }
 
 /***********************************
@@ -708,17 +632,15 @@ GtkWidget *gnome_cmd_file_popmenu_new (GnomeCmdFileList *gnomeCmdFileList)
 {
     g_return_val_if_fail (GNOME_CMD_IS_FILE_LIST (gnomeCmdFileList), nullptr);
 
-    guint pos;
-    static guint mergeIdFavApp = 0;
-
     auto files = gnomeCmdFileList->get_selected_files();
     if (!files) return nullptr;
 
-    GtkUIManager *uiManager = get_file_popup_ui_manager(gnomeCmdFileList);
+    guint position;
+    auto uiManager = get_file_popup_ui_manager(gnomeCmdFileList);
 
     // Add "Open With" popup entries
-    pos = add_open_with_entries(uiManager, gnomeCmdFileList);
-    pos += 2;
+    position = add_open_with_entries(uiManager, gnomeCmdFileList);
+    position += 2;
 
     // Add plugin popup entries
     for (auto plugins = plugin_manager_get_all (); plugins; plugins = plugins->next)
@@ -730,11 +652,12 @@ GtkWidget *gnome_cmd_file_popmenu_new (GnomeCmdFileList *gnomeCmdFileList)
             if (items)
             {
                 GtkWidget *menu = gtk_ui_manager_get_widget (uiManager, "/FilePopup");
-                add_plugin_menu_items (GTK_MENU_SHELL(menu), items, ++pos);
+                add_plugin_menu_items (GTK_MENU_SHELL(menu), items, ++position);
             }
         }
     }
 
+    static guint mergeIdFavApp = 0;
     // Add favorite applications menu entries
     if (mergeIdFavApp != 0)
     {
@@ -748,16 +671,16 @@ GtkWidget *gnome_cmd_file_popmenu_new (GnomeCmdFileList *gnomeCmdFileList)
         if (fav_app_matches_files (app, files))
         {
             add_fav_app_menu_item (uiManager, mergeIdFavApp, app, files);
-            pos++;
+            position++;
         }
     }
 
-    // Add execute menu entry
-    add_execute_entry(uiManager, gnomeCmdFileList);
-
     // Add script action menu entries
     add_action_script_entries(uiManager, files);
 
+    // Add execute menu entry
+    add_execute_entry(uiManager, files);
+
     return GTK_WIDGET (gtk_ui_manager_get_widget (uiManager, "/FilePopup"));
 }
 
@@ -810,6 +733,8 @@ GtkType gnome_cmd_file_popmenu_get_type ()
     static const char *ui_description =
     "<ui>"
     "  <popup action='FilePopup'>"
+    "    <placeholder name='Execute'/>"
+    "    <separator/>"
     "    <placeholder name='OpenWithDefault'/>"
     "    <menu action='OpenWith'>"
     "      <placeholder name='OpenWithPlaceholder'/>"
@@ -882,13 +807,14 @@ guint add_open_with_entries(GtkUIManager *ui_manager, GnomeCmdFileList *gnomeCmd
     gchar *openWithDefaultAppLabel = nullptr;
 
     // Only try to find a default application for the first file in the list of selected files
-    openWithDefaultAppName  = get_default_application_action_name(gnomeCmdFile);
-    defaultAppIconPath      = get_default_application_icon_path(gnomeCmdFile);
-    openWithDefaultAppLabel = get_default_application_action_label(gnomeCmdFile);
+    openWithDefaultAppName  = gnomeCmdFile->GetDefaultApplicationNameString();
+    auto gAppInfo = gnomeCmdFile->GetAppInfoForContentType();
+    openWithDefaultAppLabel = gnomeCmdFile->get_default_application_action_label(gAppInfo);
 
     if (openWithDefaultAppName != nullptr)
     {
         // Add the default "Open" menu entry at the top of the popup
+        defaultAppIconPath = g_strdup(get_default_application_icon_path(gAppInfo));
         appStockId = register_application_stock_icon(openWithDefaultAppName, defaultAppIconPath);
         dynAction = gtk_action_new ("Open", openWithDefaultAppLabel, nullptr, appStockId);
         g_signal_connect (dynAction, "activate", G_CALLBACK (cb_exec_default), files);
@@ -896,7 +822,6 @@ guint add_open_with_entries(GtkUIManager *ui_manager, GnomeCmdFileList *gnomeCmd
         gtk_ui_manager_add_ui (ui_manager, mergeIdOpenWith, "/FilePopup/OpenWithDefault", openWithDefaultAppLabel, "Open",
                                GTK_UI_MANAGER_AUTO, true);
         g_free (defaultAppIconPath);
-        g_free(openWithDefaultAppLabel);
         g_free(appStockId);
     }
     else
@@ -908,26 +833,30 @@ guint add_open_with_entries(GtkUIManager *ui_manager, GnomeCmdFileList *gnomeCmd
                                GTK_UI_MANAGER_SEPARATOR, false);
     }
     newTopMenuItems++;
+    g_free(openWithDefaultAppLabel);
 
     // Add menu items in the "Open with" submenu
     gint ii = -1;
-    GList *vfs_apps, *tmp_list;
-    vfs_apps = tmp_list = gnome_vfs_mime_get_all_applications (gnomeCmdFile->info->mime_type);
-    for (; vfs_apps && ii < MAX_OPEN_WITH_APPS; vfs_apps = vfs_apps->next)
+    GList *gAppInfos, *tmp_list;
+    auto contentTypeString = gnomeCmdFile->GetContentType();
+    gAppInfos = tmp_list = g_app_info_get_all_for_type(contentTypeString);
+    g_free(contentTypeString);
+    for (; gAppInfos && ii < MAX_OPEN_WITH_APPS; gAppInfos = gAppInfos->next)
     {
-        auto vfs_app = (GnomeVFSMimeApplication *) vfs_apps->data;
+        auto gAppInfoItem = (GAppInfo *) gAppInfos->data;
 
-        if (vfs_app)
+        if (gAppInfoItem)
         {
             OpenWithData *data = g_new0 (OpenWithData, 1);
 
             data->files = files;
-            data->app = gnome_cmd_app_new_from_vfs_app (vfs_app);
+            data->app = gnome_cmd_app_new_from_app_info (gAppInfoItem);
 
             gchar* openWithAppName  = g_strdup (gnome_cmd_app_get_name (data->app));
 
             // Only add the entry to the submenu if its name different from the default app
-            if (strcmp(openWithAppName, openWithDefaultAppName) == 0)
+            if (openWithDefaultAppName != nullptr
+                && strcmp(openWithAppName, openWithDefaultAppName) == 0)
             {
                 gnome_cmd_app_free(data->app);
                 g_free(data);
@@ -957,15 +886,14 @@ guint add_open_with_entries(GtkUIManager *ui_manager, GnomeCmdFileList *gnomeCmd
     }
 
     g_free(openWithDefaultAppName);
-    gnome_vfs_mime_application_list_free (tmp_list);
+    g_list_free (tmp_list);
     return newTopMenuItems;
 }
 
-void add_execute_entry(GtkUIManager *ui_manager, GnomeCmdFileList *gnomeCmdFileList)
+void add_execute_entry(GtkUIManager *ui_manager, GList *files)
 {
     static guint mergeIdExecute = 0;
     static GtkActionGroup *dynamicActionGroup = nullptr;
-    auto files = gnomeCmdFileList->get_selected_files();
     auto gnomeCmdFile = static_cast<GnomeCmdFile*> (files->data);
 
     if (mergeIdExecute != 0)
@@ -980,7 +908,7 @@ void add_execute_entry(GtkUIManager *ui_manager, GnomeCmdFileList *gnomeCmdFileL
         dynamicActionGroup = gtk_action_group_new ("executeActionGroup");
         gtk_ui_manager_insert_action_group (ui_manager, dynamicActionGroup, 0);
 
-        gtk_ui_manager_add_ui (ui_manager, mergeIdExecute, "/FilePopup/Cut", "Execute", "Execute",
+        gtk_ui_manager_add_ui (ui_manager, mergeIdExecute, "/FilePopup/Execute", "Execute", "Execute",
                                GTK_UI_MANAGER_AUTO, true);
     }
-}
\ No newline at end of file
+}
