@@ -2,7 +2,7 @@
  * @file gnome-cmd-file-selector.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -98,8 +98,8 @@ inline void GnomeCmdFileSelector::update_selected_files_label()
     if (!all_files)
         return;
 
-    GnomeVFSFileSize sel_bytes = 0;
-    GnomeVFSFileSize total_bytes = 0;
+    guint64 sel_bytes = 0;
+    guint64 total_bytes = 0;
     gint num_files = 0;
     gint num_dirs = 0;
     gint num_sel_files = 0;
@@ -117,9 +117,9 @@ inline void GnomeCmdFileSelector::update_selected_files_label()
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wswitch-enum"
 #endif
-        switch (f->info->type)
+        switch (f->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE))
         {
-            case GNOME_VFS_FILE_TYPE_DIRECTORY:
+            case G_FILE_TYPE_DIRECTORY:
                 if (!f->is_dotdot)
                 {
                     num_dirs++;
@@ -128,9 +128,9 @@ inline void GnomeCmdFileSelector::update_selected_files_label()
                 }
                 break;
 
-            case GNOME_VFS_FILE_TYPE_REGULAR:
+            case G_FILE_TYPE_REGULAR:
                 num_files++;
-                total_bytes += f->info->size;
+                total_bytes += g_file_info_get_size(f->gFileInfo);
                 break;
 
             default:
@@ -151,17 +151,17 @@ inline void GnomeCmdFileSelector::update_selected_files_label()
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wswitch-enum"
 #endif
-        switch (f->info->type)
+        switch (f->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE))
         {
-            case GNOME_VFS_FILE_TYPE_DIRECTORY:
+            case G_FILE_TYPE_DIRECTORY:
                 num_sel_dirs++;
                 if (f->has_tree_size())
                     sel_bytes += f->get_tree_size();
                 break;
 
-            case GNOME_VFS_FILE_TYPE_REGULAR:
+            case G_FILE_TYPE_REGULAR:
                 num_sel_files++;
-                sel_bytes += f->info->size;
+                sel_bytes += f->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_STANDARD_SIZE);
                 break;
 
             default:
@@ -215,13 +215,7 @@ inline void GnomeCmdFileSelector::update_direntry()
     if (!dir)
         return;
 
-    gchar *tmp = gnome_cmd_dir_get_display_path (dir);
-
-    g_return_if_fail (tmp != nullptr);
-
-    gnome_cmd_dir_indicator_set_dir (GNOME_CMD_DIR_INDICATOR (dir_indicator), tmp);
-
-    g_free (tmp);
+    gnome_cmd_dir_indicator_set_dir (GNOME_CMD_DIR_INDICATOR (dir_indicator), dir);
 }
 
 
@@ -244,9 +238,8 @@ void GnomeCmdFileSelector::do_file_specific_action (GnomeCmdFileList *fl, GnomeC
 {
     g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
     g_return_if_fail (f != nullptr);
-    g_return_if_fail (f->info != nullptr);
 
-    if (f->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+    if (f->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
     {
         if (!fl->locked)
         {
@@ -438,6 +431,7 @@ static void on_notebook_switch_page (GtkNotebook *notebook, GtkNotebookPage *pag
     fs->update_direntry();
     fs->update_selected_files_label();
     fs->update_vol_label();
+    fs->update_style();
 
     if (prev_dir!=fs->get_directory())
         g_signal_emit (fs, signals[DIR_CHANGED], 0, fs->get_directory());
@@ -484,7 +478,9 @@ static void on_list_list_clicked (GnomeCmdFileList *fl, GnomeCmdFile *f, GdkEven
                     if (f && f->is_dotdot)
                         fs->new_tab(gnome_cmd_dir_get_parent (fl->cwd));
                     else
-                        fs->new_tab(f && f->info->type==GNOME_VFS_FILE_TYPE_DIRECTORY ? GNOME_CMD_DIR (f) : fl->cwd);
+                        fs->new_tab(f && f->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY
+                                         ? GNOME_CMD_DIR (f)
+                                         : fl->cwd);
                 }
                 break;
 
@@ -523,7 +519,7 @@ static void on_list_dir_changed (GnomeCmdFileList *fl, GnomeCmdDir *dir, GnomeCm
 {
     if (fs->priv->dir_history && !fs->priv->dir_history->locked())
     {
-        gchar *fpath = GNOME_CMD_FILE (dir)->get_path();
+        gchar *fpath = GNOME_CMD_FILE (dir)->GetPathStringThroughParent();
         fs->priv->dir_history->add(fpath);
         g_free (fpath);
     }
@@ -621,6 +617,7 @@ static gboolean on_notebook_button_pressed (GtkWidget *widget, GdkEventButton *e
         case GDK_BUTTON_PRESS:
             switch (event->button)
             {
+                // mid-click
                 case 2:
                     tab_clicked = notebook->find_tab_num_at_pos(event->x_root, event->y_root);
 
@@ -634,6 +631,7 @@ static gboolean on_notebook_button_pressed (GtkWidget *widget, GdkEventButton *e
 
                     return tab_clicked>=0;
 
+                // right-click
                 case 3:
                     tab_clicked = notebook->find_tab_num_at_pos(event->x_root, event->y_root);
 
@@ -705,6 +703,8 @@ static gboolean on_notebook_button_pressed (GtkWidget *widget, GdkEventButton *e
 
                 if (!fl->locked || gnome_cmd_prompt_message (*main_win, GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL, _("The tab is locked, close anyway?"))==GTK_RESPONSE_OK)
                     fs->close_tab(tab_clicked);
+
+
             }
             else
                 fs->new_tab(fs->get_directory());
@@ -1083,7 +1083,14 @@ void GnomeCmdFileSelector::update_style()
     update_connections();
 }
 
-
+/**
+ * @brief This function tries to create a new file.
+ *
+ * @param string_dialog a GnomeCmdStringDialog object reference
+ * @param values a pointer to char pointers
+ * @param fs a GnomeCmdFileSelector object reference
+ * @return TRUE if all went well, FALSE if not
+ */
 static gboolean on_new_textfile_ok (GnomeCmdStringDialog *string_dialog, const gchar **values, GnomeCmdFileSelector *fs)
 {
     g_return_val_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs), TRUE);
@@ -1099,26 +1106,37 @@ static gboolean on_new_textfile_ok (GnomeCmdStringDialog *string_dialog, const g
     GnomeCmdDir *dir = fs->get_directory();
     g_return_val_if_fail (GNOME_CMD_IS_DIR (dir), TRUE);
 
-    gchar *dpath = GNOME_CMD_FILE (dir)->get_real_path();
-    gchar *filepath = g_build_filename (dpath, fname, nullptr);
-    g_free (dpath);
-    g_return_val_if_fail (filepath, TRUE);
+    auto uriBaseString = static_cast<gchar*>(GNOME_CMD_FILE (dir)->get_uri_str());
+    // Let the URI to the current directory end with '/'
+    auto conLastCharacter = uriBaseString[strlen(uriBaseString)-1];
+    auto uriBaseStringSep = conLastCharacter == G_DIR_SEPARATOR
+        ? g_strdup(uriBaseString)
+        : g_strdup_printf("%s%s", uriBaseString, G_DIR_SEPARATOR_S);
+    g_free (uriBaseString);
 
-    gchar *escaped_filepath = g_strdup_printf ("\"%s\"", filepath);
-#if defined (__GNUC__)
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif
-    gchar *cmd = g_strdup_printf (gnome_cmd_data.options.editor, escaped_filepath);
-#if defined (__GNUC__)
-#pragma GCC diagnostic pop
-#endif
+    auto relativeFileNamePath = g_build_filename(".", fname, nullptr);
+    GError *error = nullptr;
+    auto uriString = g_uri_resolve_relative (uriBaseStringSep, relativeFileNamePath,
+                                             G_URI_FLAGS_NONE, &error);
+    g_free(relativeFileNamePath);
+    g_free (uriBaseStringSep);
+    if (error)
+    {
+        gnome_cmd_string_dialog_set_error_desc (string_dialog, g_strdup_printf("%s", error->message));
+        g_error_free(error);
+        return FALSE;
+    }
 
-    g_free (filepath);
-    g_free (escaped_filepath);
-
-    if (cmd)
-        run_command (cmd);
+    auto gFile = g_file_new_for_uri(uriString);
+    g_free (uriString);
+    auto gFileOutputStream = g_file_create(gFile, G_FILE_CREATE_NONE, nullptr, &error);
+    if (error)
+    {
+        gnome_cmd_string_dialog_set_error_desc (string_dialog, g_strdup_printf("%s", error->message));
+        g_error_free(error);
+        return FALSE;
+    }
+    g_object_unref(gFileOutputStream);
 
     return TRUE;
 }
@@ -1128,6 +1146,7 @@ static gboolean on_create_symlink_ok (GnomeCmdStringDialog *string_dialog, const
 {
     g_return_val_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs), TRUE);
     g_return_val_if_fail (fs->priv->sym_file != nullptr, TRUE);
+    GError *error = nullptr;
 
     const gchar *fname = values[0];
 
@@ -1138,21 +1157,26 @@ static gboolean on_create_symlink_ok (GnomeCmdStringDialog *string_dialog, const
         return FALSE;
     }
 
-    GnomeVFSURI *uri = gnome_cmd_dir_get_child_uri (fs->get_directory(), fname);
-    GnomeVFSResult result = gnome_vfs_create_symbolic_link (uri, fs->priv->sym_file->get_uri_str());
+    auto gFile = gnome_cmd_dir_get_child_gfile (fs->get_directory(), fname);
+    auto absolutePath = g_file_get_parse_name(fs->priv->sym_file->gFile);
 
-    if (result == GNOME_VFS_OK)
+    //if (g_file_make_symbolic_link (gFile, fs->priv->sym_file->get_uri_str(), nullptr, &error))
+    if (g_file_make_symbolic_link (gFile, absolutePath, nullptr, &error))
     {
-        gchar *uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
+        gchar *uri_str = g_file_get_uri (gFile);
         gnome_cmd_dir_file_created (fs->get_directory(), uri_str);
         g_free (uri_str);
-        gnome_vfs_uri_unref (uri);
+        g_free(absolutePath);
+        g_object_unref (gFile);
         return TRUE;
     }
 
-    gnome_vfs_uri_unref (uri);
-
-    gnome_cmd_string_dialog_set_error_desc (string_dialog, g_strdup (gnome_vfs_result_to_string (result)));
+    g_object_unref (gFile);
+    auto msg = g_strdup(error->message);
+    gnome_cmd_string_dialog_set_error_desc (string_dialog, msg);
+    g_free(msg);
+    g_free(absolutePath);
+    g_error_free(error);
 
     return FALSE;
 }
@@ -1172,8 +1196,11 @@ void gnome_cmd_file_selector_show_new_textfile_dialog (GnomeCmdFileSelector *fs)
     GnomeCmdFile *f = fs->file_list()->get_selected_file();
 
     if (f)
-        gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 0, f->info->name);
-
+    {
+        gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog),
+            0,
+            g_file_info_get_display_name(f->gFileInfo));
+    }
     g_object_ref (dialog);
     gtk_widget_show (dialog);
 }
@@ -1281,7 +1308,7 @@ gboolean GnomeCmdFileSelector::key_pressed(GdkEventKey *event)
             case GDK_Right:
             case GDK_KP_Right:
                 f = list->get_selected_file();
-                if (f && f->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+                if (f && f->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
                     do_file_specific_action (list, f);
                 g_signal_stop_emission_by_name (list, "key-press-event");
                 return TRUE;
@@ -1313,16 +1340,14 @@ void gnome_cmd_file_selector_create_symlink (GnomeCmdFileSelector *fs, GnomeCmdF
 {
     const gchar *labels[] = {_("Symbolic link name:")};
 
-    gchar *fname = get_utf8 (f->get_name());
 #if defined (__GNUC__)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat-nonliteral"
 #endif
-    gchar *text = g_strdup_printf (gnome_cmd_data_get_symlink_prefix (), fname);
+    gchar *text = g_strdup_printf (gnome_cmd_data_get_symlink_prefix (), f->get_name());
 #if defined (__GNUC__)
 #pragma GCC diagnostic pop
 #endif
-    g_free (fname);
 
     GtkWidget *dialog = gnome_cmd_string_dialog_new (_("Create Symbolic Link"),
                                                      labels,
@@ -1345,45 +1370,46 @@ void gnome_cmd_file_selector_create_symlinks (GnomeCmdFileSelector *fs, GList *f
 
     for (; files; files=files->next)
     {
+        GError *error = nullptr;
         auto f = static_cast<GnomeCmdFile*> (files->data);
-        gchar *fname = get_utf8 (f->get_name());
 #if defined (__GNUC__)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat-nonliteral"
 #endif
-        gchar *symlink_name = g_strdup_printf (gnome_cmd_data_get_symlink_prefix (), fname);
+        gchar *symlink_name = g_strdup_printf (gnome_cmd_data_get_symlink_prefix (), f->get_name());
 #if defined (__GNUC__)
 #pragma GCC diagnostic pop
 #endif
 
-        GnomeVFSURI *uri = gnome_cmd_dir_get_child_uri (fs->get_directory(), symlink_name);
+        auto gFile = gnome_cmd_dir_get_child_gfile (fs->get_directory(), symlink_name);
 
-        g_free (fname);
         g_free (symlink_name);
 
-        GnomeVFSResult result;
+        gboolean result;
 
         do
         {
-            result = gnome_vfs_create_symbolic_link (uri, f->get_uri_str());
-
-            if (result == GNOME_VFS_OK)
+            auto absolutePathName = g_file_get_parse_name(f->gFile);
+            result = g_file_make_symbolic_link (gFile, absolutePathName, nullptr, &error);
+            g_free(absolutePathName);
+            if (!result) // 0 means it worked
             {
-                gchar *uri_str = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_PASSWORD);
+                gchar *uri_str = g_file_get_uri (gFile);
                 gnome_cmd_dir_file_created (fs->get_directory(), uri_str);
                 g_free (uri_str);
             }
             else
-                if (choice != 1)  // choice != SKIP_ALL
+                if (choice != 1 && error)  // choice != SKIP_ALL
                 {
-                    gchar *msg = g_strdup (gnome_vfs_result_to_string (result));
+                    gchar *msg = g_strdup (error->message);
                     choice = run_simple_dialog (*main_win, TRUE, GTK_MESSAGE_QUESTION, msg, _("Create Symbolic Link"), 3, _("Skip"), _("Skip all"), _("Cancel"), _("Retry"), nullptr);
                     g_free (msg);
+                    g_error_free(error);
                 }
         }
-        while (result != GNOME_VFS_OK && choice == 3);  // choice != RETRY
+        while (result != true && choice == 3);  // choice != RETRY
 
-        gnome_vfs_uri_unref (uri);
+        g_object_unref (gFile);
     }
 }
 
@@ -1532,6 +1558,8 @@ GtkWidget *GnomeCmdFileSelector::new_tab(GnomeCmdDir *dir, GnomeCmdFileList::Col
 
 void GnomeCmdFileSelector::update_tab_label(GnomeCmdFileList *fl)
 {
+    g_return_if_fail(fl->cwd != nullptr);
+
     const gchar *name = GNOME_CMD_FILE (fl->cwd)->get_name();
 
     switch (gnome_cmd_data.options.tab_lock_indicator)
