@@ -2,7 +2,7 @@
  * @file gnome-cmd-file.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,15 +39,16 @@ struct GnomeCmdDir;
 
 struct GnomeCmdFile
 {
-    GnomeCmdFileInfo parent;
+    GnomeCmdFileBase parent;
 
     class Private;
 
     Private *priv;
 
-    GnomeVFSFileInfo *info;
+    GFile *gFile;
+    GFileInfo *gFileInfo;
     gboolean is_dotdot;
-    gchar *collate_key;                 // necessary for proper sorting of UTF-8 encoded file names
+    gchar *collate_key;                 // necessary for proper sorting of UTF-8 encoded file names //ToDo: Check if this is really still needed
     GnomeCmdFileMetadata *metadata;
 
     GnomeCmdFile *ref();
@@ -55,42 +56,44 @@ struct GnomeCmdFile
 
     void invalidate_metadata();
 
-    gchar *get_name();
+    const gchar *get_name();
     gchar *get_quoted_name();
-    gchar *get_path();
+    gchar *GetPathStringThroughParent();
     gchar *get_real_path();
     gchar *get_quoted_real_path();
     gchar *get_dirname();
     gchar *get_unescaped_dirname();
-    GnomeVFSURI *get_uri(const gchar *name=NULL);
-    gchar *get_uri_str(GnomeVFSURIHideOptions hide_options=GNOME_VFS_URI_HIDE_PASSWORD);
 
-    char *get_collation_fname() const    {  return collate_key ? collate_key : info->name;  }
+    GFile *get_gfile(const gchar *name=NULL);
+    gchar *get_uri_str();
+
+    char *get_collation_fname() const    {  return collate_key ? collate_key : g_file_get_basename(gFile);  }
 
     const gchar *get_extension();
     const gchar *get_owner();
     const gchar *get_group();
+#ifdef GLIB_2_70
     const gchar *get_adate(gboolean overide_disp_setting);
+#endif
     const gchar *get_mdate(gboolean overide_disp_setting);
     const gchar *get_size();
-    GnomeVFSFileSize get_tree_size();
+    guint64 get_tree_size();
+    guint64 calc_tree_size (gulong *count);
     const gchar *get_tree_size_as_str();
     const gchar *get_perm();
-    const gchar *get_mime_type();
-    const gchar *get_mime_type_desc();
-    gboolean has_mime_type(const gchar *mime_type);
-    gboolean mime_begins_with(const gchar *mime_type_start);
+    gboolean has_content_type(const gchar *contentType);
+    gboolean content_type_begins_with(const gchar *contentTypeStart);
 
     GnomeCmdDir *get_parent_dir();
 
     const gchar *get_type_string();
     gboolean get_type_pixmap_and_mask(GdkPixmap **pixmap, GdkBitmap **mask);
 
-    GnomeVFSResult chmod(GnomeVFSFilePermissions perm);
-    GnomeVFSResult chown(uid_t uid, gid_t gid);
-    GnomeVFSResult rename(const gchar *new_name);
+    gboolean chmod(guint32 permissions, GError **error);
+    gboolean chown(uid_t uid, gid_t gid, GError **error);
+    gboolean rename(const gchar *new_name, GError **error);
 
-    void update_info(GnomeVFSFileInfo *info);
+    void update_gFileInfo(GFileInfo *gFileInfo);
     gboolean is_local();
     gboolean is_executable();
     void is_deleted();
@@ -101,25 +104,34 @@ struct GnomeCmdFile
     void invalidate_tree_size();
     gboolean has_tree_size();
 
-    GnomeVFSMimeApplication *get_default_application();
+    gboolean GetGfileAttributeBoolean(const char *attribute);
+    guint32 GetGfileAttributeUInt32(const char *attribute);
+    guint64 GetGfileAttributeUInt64(const char *attribute);
+    gchar *GetGfileAttributeString(const char *attribute);
+    gchar *GetDefaultApplicationNameString();
+    gchar *GetContentType();
+    gchar *get_default_application_action_label(GAppInfo *gAppInfo);
+    gchar *get_default_application_name(GAppInfo *gAppInfo);
+    GAppInfo *GetAppInfoForContentType();
 };
 
 struct GnomeCmdFileClass
 {
-    GnomeCmdFileInfoClass parent_class;
+    GnomeCmdFileBaseClass parent_class;
 };
 
 
-inline gchar *GnomeCmdFile::get_name()
+inline const gchar *GnomeCmdFile::get_name()
 {
-    g_return_val_if_fail (info != NULL, NULL);
-    return info->name;
+    g_return_val_if_fail (gFileInfo != NULL, NULL);
+    return g_file_info_get_display_name(gFileInfo);
 }
 
-GnomeCmdFile *gnome_cmd_file_new_from_uri (GnomeVFSURI *uri);
+GnomeCmdFile *gnome_cmd_file_new_from_gfile (GFile *gFile);
 GnomeCmdFile *gnome_cmd_file_new (const gchar *local_full_path);
-GnomeCmdFile *gnome_cmd_file_new (GnomeVFSFileInfo *info, GnomeCmdDir *dir);
-void gnome_cmd_file_setup (GnomeCmdFile *f, GnomeVFSFileInfo *info, GnomeCmdDir *dir);
+GnomeCmdFile *gnome_cmd_file_new (GFileInfo *gFileInfo, GnomeCmdDir *dir);
+void gnome_cmd_file_setup (GObject *gObject, GFileInfo *gFileInfo, GnomeCmdDir *dir);
+gboolean gnome_cmd_file_setup (GObject *gObject, GFile *gFile, GError **error);
 
 inline GnomeCmdFile *gnome_cmd_file_ref (GnomeCmdFile *f)
 {
@@ -133,7 +145,7 @@ inline void gnome_cmd_file_unref (GnomeCmdFile *f)
     f->unref();
 }
 
-inline gchar *gnome_cmd_file_get_name (GnomeCmdFile *f)
+inline const gchar *gnome_cmd_file_get_name (GnomeCmdFile *f)
 {
     g_return_val_if_fail (f != NULL, NULL);
     return f->get_name();
@@ -157,10 +169,10 @@ inline gchar *gnome_cmd_file_get_quoted_real_path (GnomeCmdFile *f)
     return f->get_quoted_real_path();
 }
 
-inline gchar *gnome_cmd_file_get_uri_str (GnomeCmdFile *f, GnomeVFSURIHideOptions hide_options=GNOME_VFS_URI_HIDE_PASSWORD)
+inline gchar *gnome_cmd_file_get_uri_str (GnomeCmdFile *f)
 {
     g_return_val_if_fail (f != NULL, NULL);
-    return f->get_uri_str(hide_options);
+    return f->get_uri_str();
 }
 
 void gnome_cmd_file_show_properties (GnomeCmdFile *f);
@@ -181,20 +193,3 @@ inline void gnome_cmd_file_list_unref (GList *files)
 {
     g_list_foreach (files, (GFunc) gnome_cmd_file_unref, NULL);
 }
-
-inline const gchar *GnomeCmdFile::get_mime_type()
-{
-    g_return_val_if_fail (info != NULL, NULL);
-    return gnome_vfs_file_info_get_mime_type (info);
-}
-
-inline const gchar *GnomeCmdFile::get_mime_type_desc()
-{
-    g_return_val_if_fail (info != NULL, NULL);
-    return info->mime_type ? gnome_vfs_mime_get_description (info->mime_type) : NULL;
-}
-
-inline GnomeVFSMimeApplication *GnomeCmdFile::get_default_application()
-{
-    return info->mime_type ? gnome_vfs_mime_get_default_application (info->mime_type) : NULL;
-}
