@@ -2,7 +2,7 @@
  * @file gnome-cmd-main-win.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -152,7 +152,7 @@ static gint gnome_cmd_key_snooper(GtkWidget *grab_widget, GdkEventKey *event, Gn
     {
         return FALSE;
     }
-    
+
     if ((gnome_cmd_data.options.quick_search == GNOME_CMD_QUICK_SEARCH_JUST_A_CHARACTER) &&
         (state_is_ctrl (event->state) || state_is_ctrl_shift (event->state) || state_is_ctrl_alt_shift (event->state)
         || (state_is_alt (event->state) || state_is_alt_shift (event->state))
@@ -479,7 +479,7 @@ static void on_delete_clicked (GtkButton *button, GnomeCmdMainWin *mw)
 
 static void on_search_clicked (GtkButton *button, GnomeCmdMainWin *mw)
 {
-    edit_search (NULL);
+    file_search (NULL);
 }
 
 
@@ -649,8 +649,7 @@ void GnomeCmdMainWin::update_drop_con_button(GnomeCmdFileList *fl)
     if (!con)
         return;
 
-    if (!gnome_cmd_data.show_toolbar
-        || (gnome_cmd_data.options.skip_mounting && GNOME_CMD_IS_CON_DEVICE (con)))
+    if (!gnome_cmd_data.show_toolbar)
         return;
 
     GtkWidget *btn = priv->tb_con_drop_btn;
@@ -659,7 +658,7 @@ void GnomeCmdMainWin::update_drop_con_button(GnomeCmdFileList *fl)
     if (prev_pixmap)
     {
         gtk_widget_destroy (prev_pixmap);
-        prev_pixmap = NULL;
+        prev_pixmap = nullptr;
     }
 
     if (gnome_cmd_con_is_closeable (con))
@@ -680,7 +679,7 @@ void GnomeCmdMainWin::update_drop_con_button(GnomeCmdFileList *fl)
         {
             g_object_ref (pixmap);
             gtk_widget_show (pixmap);
-            gtk_container_add (GTK_CONTAINER (btn), pixmap);
+            gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON(btn), pixmap);
             prev_pixmap = pixmap;
         }
     }
@@ -691,7 +690,7 @@ void GnomeCmdMainWin::update_drop_con_button(GnomeCmdFileList *fl)
         {
             g_object_ref (label);
             gtk_widget_show (label);
-            gtk_container_add (GTK_CONTAINER (btn), label);
+            gtk_tool_button_set_label_widget(GTK_TOOL_BUTTON(btn), label);
             prev_pixmap = label;
         }
     }
@@ -794,9 +793,6 @@ static void destroy (GtkObject *object)
     if (main_win && main_win->advrename_dlg)
         gtk_widget_destroy (*main_win->advrename_dlg);
 
-    if (main_win && main_win->file_search_dlg)
-        gtk_widget_destroy (*main_win->file_search_dlg);
-
     gtk_main_quit ();
 }
 
@@ -839,7 +835,6 @@ static void init (GnomeCmdMainWin *mw)
     main_win = GNOME_CMD_MAIN_WIN (mw);
 
     mw->advrename_dlg = NULL;
-    mw->file_search_dlg = NULL;
     mw->priv = g_new0 (GnomeCmdMainWin::Private, 1);
     mw->priv->current_fs = LEFT;
     mw->priv->accel_group = gtk_accel_group_new ();
@@ -1074,7 +1069,8 @@ gboolean GnomeCmdMainWin::key_pressed(GdkEventKey *event)
         {
             case GDK_H:
             case GDK_h:
-                gnome_cmd_data.options.filter.hidden = !gnome_cmd_data.options.filter.hidden;
+                gnome_cmd_data.options.filter.file_types[GnomeCmdData::G_FILE_IS_HIDDEN] =
+                    !gnome_cmd_data.options.filter.file_types[GnomeCmdData::G_FILE_IS_HIDDEN];
                 gnome_cmd_data.save();
                 return TRUE;
             default:
@@ -1207,17 +1203,61 @@ gboolean GnomeCmdMainWin::key_pressed(GdkEventKey *event)
 
 void GnomeCmdMainWin::open_tabs(FileSelectorID id)
 {
-    GnomeCmdCon *home = get_home_con ();
-
     if (gnome_cmd_data.tabs[id].empty())
         gnome_cmd_data.tabs[id].push_back(make_pair(string(g_get_home_dir ()), make_triple(GnomeCmdFileList::COLUMN_NAME, GTK_SORT_ASCENDING, FALSE)));
 
-    vector<GnomeCmdData::Tab>::const_iterator last_tab = unique(gnome_cmd_data.tabs[id].begin(),gnome_cmd_data.tabs[id].end());
+    auto last_tab = unique(gnome_cmd_data.tabs[id].begin(), gnome_cmd_data.tabs[id].end());
 
-    for (vector<GnomeCmdData::Tab>::const_iterator i=gnome_cmd_data.tabs[id].begin(); i!=last_tab; ++i)
+    for (auto tab=gnome_cmd_data.tabs[id].begin(); tab!=last_tab; ++tab)
     {
-        GnomeCmdDir *dir = gnome_cmd_dir_new (home, gnome_cmd_con_create_path (home, i->first.c_str()));
-        fs(id)->new_tab(dir, i->second.first, i->second.second, i->second.third, TRUE);
+        auto uriString = tab->first;
+        auto uriScheme = g_uri_peek_scheme (uriString.c_str());
+        auto uriIsRelative = false;
+
+        if (!uriScheme)
+        {
+            g_warning("Stored URI is either not absolute or invalid: %s", uriString.c_str());
+            uriScheme = g_strdup("file");
+            uriIsRelative = true;
+        }
+
+        if (strcmp(uriScheme, "file") == 0 || uriIsRelative)
+        {
+            GnomeCmdCon *home = get_home_con ();
+            gchar *path = nullptr;
+            if (uriIsRelative)
+            {
+                path = g_strdup(tab->first.c_str());
+            }
+            else
+            {
+                auto gUri = g_uri_parse(tab->first.c_str(), G_URI_FLAGS_NONE, nullptr);
+                path = g_strdup(g_uri_get_path(gUri));
+            }
+            auto *gnomeCmdDir = gnome_cmd_dir_new (home, gnome_cmd_con_create_path (home, path));
+            fs(id)->new_tab(gnomeCmdDir, tab->second.first, tab->second.second, tab->second.third, TRUE);
+            g_free(path);
+        }
+        else
+        {
+            GError *error = nullptr;
+            gchar *path = nullptr;
+            auto gUri = g_uri_parse(tab->first.c_str(), G_URI_FLAGS_NONE, &error);
+            if (error)
+            {
+                g_warning("Stored URI is invalid: %s", error->message);
+                g_error_free(error);
+                // open home directory instead
+                path = g_strdup(g_get_home_dir());
+            }
+            path = path ? path : g_strdup(g_uri_get_path(gUri));
+
+            GnomeCmdConRemote *con = gnome_cmd_con_remote_new(nullptr, uriString);
+            auto gnomeCmdPath = gnome_cmd_con_create_path((GnomeCmdCon*) con, path);
+            auto gnomeCmdDir = gnome_cmd_dir_new((GnomeCmdCon*) con, gnomeCmdPath, true);
+            fs(id)->new_tab(gnomeCmdDir, tab->second.first, tab->second.second, tab->second.third, TRUE);
+            g_free(path);
+        }
     }
 }
 
@@ -1252,8 +1292,8 @@ void GnomeCmdMainWin::set_fs_directory_to_opposite(FileSelectorID fsID)
     {
         GnomeCmdFile *file = other->file_list()->get_selected_file();
 
-        if (file && file->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
-            dir = GNOME_CMD_IS_DIR (file) ? GNOME_CMD_DIR (file) : gnome_cmd_dir_new_from_info (file->info, dir);
+        if (file && (file->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY))
+            dir = GNOME_CMD_IS_DIR (file) ? GNOME_CMD_DIR (file) : gnome_cmd_dir_new_from_gfileinfo (file->gFileInfo, dir);
     }
 
     if (fselector->file_list()->locked)
@@ -1429,8 +1469,8 @@ GnomeCmdState *GnomeCmdMainWin::get_state() const
     GnomeCmdDir *dir2 = fs2->get_directory();
 
     GnomeCmdState *state = &priv->state;
-    state->active_dir_uri = GNOME_CMD_FILE (dir1)->get_uri();
-    state->inactive_dir_uri = GNOME_CMD_FILE (dir2)->get_uri();
+    state->activeDirGfile = dir1 ? GNOME_CMD_FILE (dir1)->gFile : nullptr;
+    state->inactiveDirGfile = dir2 ? GNOME_CMD_FILE (dir2)->gFile : nullptr;
     state->active_dir_files = fs1->file_list()->get_visible_files();
     state->inactive_dir_files = fs2->file_list()->get_visible_files();
     state->active_dir_selected_files = fs1->file_list()->get_selected_files();
