@@ -2,7 +2,7 @@
  * @file gnome-cmd-main-win.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #include "gnome-cmd-cmdline.h"
 #include "plugin_manager.h"
 #include "dialogs/gnome-cmd-advrename-dialog.h"
-#include "dialogs/gnome-cmd-search-dialog.h"
 
 #define GNOME_CMD_TYPE_MAIN_WIN              (gnome_cmd_main_win_get_type ())
 #define GNOME_CMD_MAIN_WIN(obj)              (G_TYPE_CHECK_INSTANCE_CAST((obj), GNOME_CMD_TYPE_MAIN_WIN, GnomeCmdMainWin))
@@ -62,7 +61,6 @@ struct GnomeCmdMainWin
     operator GtkWindow * () const       {  return GTK_WINDOW (this);       }
 
     GnomeCmdAdvrenameDialog *advrename_dlg;
-    GnomeCmdSearchDialog *file_search_dlg;
 
     FileSelectorID fs() const;
     FileSelectorID fs(GnomeCmdFileSelector *fs) const;
