@@ -1,8 +1,8 @@
-/** 
+/**
  * @file gnome-cmd-plain-path.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,20 +29,24 @@ using namespace std;
 
 GnomeCmdPath *GnomeCmdPlainPath::get_parent()
 {
-    GnomeVFSURI *t = gnome_vfs_uri_new (G_DIR_SEPARATOR_S);
-    GnomeVFSURI *u1 = gnome_vfs_uri_append_path (t, path);
-    gnome_vfs_uri_unref (t);
+    auto fullPath     = *path != '/'
+                        ? g_strconcat(G_DIR_SEPARATOR_S, path, nullptr)
+                        : g_strdup(path);
+    auto gFileForPath = g_file_new_for_path(fullPath);
+    auto gFileParent  = g_file_get_parent(gFileForPath);
+    g_object_unref(gFileForPath);
+    g_free(fullPath);
 
-    GnomeVFSURI *u2 = gnome_vfs_uri_get_parent (u1);
-    gnome_vfs_uri_unref (u1);
+    if (!gFileParent)
+    {
+        return nullptr;
+    }
 
-    if (!u2)  return NULL;
+    auto pathString = g_file_get_path(gFileParent);
+    g_object_unref(gFileParent);
 
-    gchar *s = gnome_vfs_uri_to_string (u2, GNOME_VFS_URI_HIDE_PASSWORD);
-    gnome_vfs_uri_unref (u2);
-
-    GnomeCmdPath *parent_path = new GnomeCmdPlainPath (gnome_vfs_get_local_path_from_uri (s));
-    g_free (s);
+    GnomeCmdPath *parent_path = new GnomeCmdPlainPath (pathString);
+    g_free (pathString);
 
     return parent_path;
 }
@@ -50,22 +54,9 @@ GnomeCmdPath *GnomeCmdPlainPath::get_parent()
 
 GnomeCmdPath *GnomeCmdPlainPath::get_child(const gchar *child)
 {
-    GnomeVFSURI *t = gnome_vfs_uri_new (G_DIR_SEPARATOR_S);
-    GnomeVFSURI *u1 = gnome_vfs_uri_append_path (t, path);
-    gnome_vfs_uri_unref (t);
-
-    GnomeVFSURI *u2 = strchr (child, '/')==NULL ?
-                      gnome_vfs_uri_append_file_name (u1, child) :
-                      gnome_vfs_uri_append_path (u1, child);
-    gnome_vfs_uri_unref (u1);
-
-    if (!u2)  return NULL;
-
-    gchar *path_str = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (u2), 0);
-    gnome_vfs_uri_unref (u2);
-
-    GnomeCmdPath *child_path = new GnomeCmdPlainPath(path_str);
-    g_free (path_str);
-
+    g_return_val_if_fail(child != nullptr, nullptr);
+    auto fullPath = g_build_filename(path && path[0] != '\0' ? path : G_DIR_SEPARATOR_S, child, nullptr);
+    auto child_path = new GnomeCmdPlainPath(fullPath);
+    g_free (fullPath);
     return child_path;
 }
