@@ -2,7 +2,7 @@
  * @file gnome-cmd-quicksearch-popup.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,7 +85,9 @@ static void set_filter (GnomeCmdQuicksearchPopup *popup, const gchar *text)
     {
         auto f = static_cast<GnomeCmdFile*> (files->data);
 
-        if (gnome_cmd_filter_fnmatch (pattern, f->info->name, gnome_cmd_data.options.case_sens_sort))
+        if (gnome_cmd_filter_fnmatch (pattern,
+                g_file_info_get_display_name(f->gFileInfo),
+                gnome_cmd_data.options.case_sens_sort))
         {
             if (first)
             {
