@@ -2,7 +2,7 @@
  * @file gnome-cmd-types.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,14 @@ typedef enum
 }GnomeCmdSizeDispMode;
 
 
+typedef enum
+{
+    COPY,
+    MOVE,
+    LINK
+}GnomeCmdTransferType;
+
+
 typedef enum
 {
     GNOME_CMD_PERM_DISP_MODE_TEXT,
@@ -85,14 +93,23 @@ typedef enum // Watch out for the usage of GNOME_CMD_COLOR_CUSTOM in gnome-cmd-d
 }GnomeCmdColorMode;
 
 
-typedef enum // The (reversed) order of following enums is significant
+typedef enum // The (reversed) order of following enums compared to the occurence in the GUI is significant
 {
+    GNOME_CMD_CONFIRM_OVERWRITE_SILENTLY,
     GNOME_CMD_CONFIRM_OVERWRITE_SKIP_ALL,
-    GNOME_CMD_CONFIRM_OVERWRITE_QUERY,
-    GNOME_CMD_CONFIRM_OVERWRITE_SILENTLY
+    GNOME_CMD_CONFIRM_OVERWRITE_RENAME_ALL,
+    GNOME_CMD_CONFIRM_OVERWRITE_QUERY
 }GnomeCmdConfirmOverwriteMode;
 
 
+typedef enum
+{
+    GNOME_CMD_DEFAULT_DND_QUERY,
+    GNOME_CMD_DEFAULT_DND_COPY,
+    GNOME_CMD_DEFAULT_DND_MOVE
+}GnomeCmdDefaultDndMode;
+
+
 struct GnomeCmdColorTheme
 {
     gboolean respect_theme {TRUE};
