@@ -2,7 +2,7 @@
  * @file gnome-cmd-user-actions.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@
 #include "dialogs/gnome-cmd-make-copy-dialog.h"
 #include "dialogs/gnome-cmd-manage-bookmarks-dialog.h"
 #include "dialogs/gnome-cmd-mkdir-dialog.h"
-#include "dialogs/gnome-cmd-search-dialog.h"
 #include "dialogs/gnome-cmd-options-dialog.h"
 #include "dialogs/gnome-cmd-prepare-copy-dialog.h"
 #include "dialogs/gnome-cmd-prepare-move-dialog.h"
@@ -92,7 +91,7 @@ static void gcmd_user_action_settings_class_init (GcmdUserActionSettingsClass *k
 
 GcmdUserActionSettings *gcmd_user_action_settings_new ()
 {
-    return (GcmdUserActionSettings *) g_object_new (USER_ACTION_SETTINGS, NULL);
+    return (GcmdUserActionSettings *) g_object_new (USER_ACTION_SETTINGS, nullptr);
 }
 
 static void gcmd_user_action_settings_init (GcmdUserActionSettings *gs)
@@ -103,13 +102,13 @@ static void gcmd_user_action_settings_init (GcmdUserActionSettings *gs)
     global_schema_source = GnomeCmdData::GetGlobalSchemaSource();
 
     global_schema = g_settings_schema_source_lookup (global_schema_source, GCMD_PREF_FILTER, FALSE);
-    gs->filter = g_settings_new_full (global_schema, NULL, NULL);
+    gs->filter = g_settings_new_full (global_schema, nullptr, nullptr);
 
     global_schema = g_settings_schema_source_lookup (global_schema_source, GCMD_PREF_GENERAL, FALSE);
-    gs->general = g_settings_new_full (global_schema, NULL, NULL);
+    gs->general = g_settings_new_full (global_schema, nullptr, nullptr);
 
     global_schema = g_settings_schema_source_lookup (global_schema_source, GCMD_PREF_PROGRAMS, FALSE);
-    gs->programs = g_settings_new_full (global_schema, NULL, NULL);
+    gs->programs = g_settings_new_full (global_schema, nullptr, nullptr);
 }
 
 /***********************************
@@ -125,7 +124,7 @@ GnomeCmdFileList *get_fl (const FileSelectorID fsID)
 {
     GnomeCmdFileSelector *fs = get_fs (fsID);
 
-    return fs ? fs->file_list() : NULL;
+    return fs ? fs->file_list() : nullptr;
 }
 
 // The file returned from this function is not to be unrefed
@@ -213,8 +212,8 @@ static UserActionData user_actions_data[] = {
                                              {file_delete, "edit.delete", N_("Delete")},
                                              {edit_filter, "edit.filter", N_("Show user defined files")},
                                              {edit_cap_paste, "edit.paste", N_("Paste")},
-                                             {edit_quick_search, "edit.quick_search", N_("Quick search")},
-                                             {edit_search, "edit.search", N_("Search")},
+                                             {file_quick_search, "file.quick_search", N_("Quick search")},
+                                             {file_search, "file.search", N_("Search")},
                                              {file_advrename, "file.advrename", N_("Advanced rename tool")},
                                              {file_chmod, "file.chmod", N_("Change permissions")},
                                              {file_chown, "file.chown", N_("Change owner/group")},
@@ -245,6 +244,8 @@ static UserActionData user_actions_data[] = {
                                              {mark_compare_directories, "mark.compare_directories", N_("Compare directories")},
                                              {mark_invert_selection, "mark.invert", N_("Invert selection")},
                                              {mark_select_all, "mark.select_all", N_("Select all")},
+                                             {mark_select_all_files, "mark.select_all_files", N_("Select all files")},
+                                             {mark_unselect_all_files, "mark.unselect_all_files", N_("Unselect all files")},
                                              {mark_toggle, "mark.toggle", N_("Toggle selection")},
                                              {mark_toggle_and_step, "mark.toggle_and_step", N_("Toggle selection and move cursor downward")},
                                              {mark_unselect_all, "mark.unselect_all", N_("Unselect all")},
@@ -563,7 +564,7 @@ gboolean GnomeCmdUserActions::handle_key_event(GnomeCmdMainWin *mw, GnomeCmdFile
     DEBUG('u', "Key event:  %s (%#x)\n", key2str(*event).c_str(), event->keyval);
     DEBUG('u', "Handling key event by %s()\n", action_func[pos->second.func].c_str());
 
-    (*pos->second.func) (NULL, (gpointer) (pos->second.user_data.empty() ? NULL : pos->second.user_data.c_str()));
+    (*pos->second.func) (nullptr, (gpointer) (pos->second.user_data.empty() ? nullptr : pos->second.user_data.c_str()));
 
     return TRUE;
 }
@@ -628,8 +629,12 @@ static void get_file_list (string &s, GList *sfl, F f)
     vector<string> a;
 
     for (GList *i = sfl; i; i = i->next)
-        a.push_back ((*f) (GNOME_CMD_FILE (i->data)));
+    {
+        auto value = (*f) (GNOME_CMD_FILE (i->data));
 
+        if (value)
+            a.push_back (value);
+    }
     join (s, a.begin(), a.end());
 }
 
@@ -640,7 +645,12 @@ inline void get_file_list (string &s, GList *sfl, F f, T t)
     vector<string> a;
 
     for (GList *i = sfl; i; i = i->next)
-        a.push_back ((*f) (GNOME_CMD_FILE (i->data), t));
+    {
+        auto value = (*f) (GNOME_CMD_FILE (i->data), t);
+
+        if (value)
+            a.push_back (value);
+    }
 
     join (s, a.begin(), a.end());
 }
@@ -715,42 +725,42 @@ void file_edit (GtkMenuItem *menuitem, gpointer not_used)
 {
     GdkModifierType mask;
 
-    gdk_window_get_pointer (NULL, NULL, NULL, &mask);
+    gdk_window_get_pointer (nullptr, nullptr, nullptr, &mask);
 
     if (mask & GDK_SHIFT_MASK)
         gnome_cmd_file_selector_show_new_textfile_dialog (get_fs (ACTIVE));
     else
     {
-	gchar       *command;
+	    gchar       *command;
 
-	command = g_strdup (gnome_cmd_data.options.editor);
-	g_return_if_fail (command != NULL);
+	    command = g_strdup (gnome_cmd_data.options.editor);
+	    g_return_if_fail (command != nullptr);
 
-	string dir_path;
-	string cmd;
-	
-	cmd.reserve(2000);
-	if (parse_command(&cmd, (const gchar*) command) == 0)
-	{
-	    DEBUG ('g', "Edit file command is not valid.\n");
-	    gnome_cmd_show_message (*main_win, _("No valid command given."));
-	    return;
-	}
-	else
-	{
-        GnomeCmdDir *dir = NULL;
-	    gint     argc;
-	    gchar  **argv  = NULL;
-	    GError  *error = NULL;
-	    DEBUG ('g', "Edit file: %s\n", cmd.c_str());
-	    
-	    g_shell_parse_argv (cmd.c_str(), &argc, &argv, NULL);
-	    if (!g_spawn_async (gnome_cmd_dir_is_local (dir) ? dir_path.c_str() : NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
-		gnome_cmd_error_message (_("Unable to execute command."), error);
-	    
-	    g_strfreev (argv);
-	    g_free (command);
-	}
+	    string dir_path;
+	    string cmd;
+
+	    cmd.reserve(2000);
+	    if (parse_command(&cmd, (const gchar*) command) == 0)
+	    {
+	        DEBUG ('g', "Edit file command is not valid.\n");
+	        gnome_cmd_show_message (*main_win, _("No valid command given."));
+	        return;
+	    }
+	    else
+	    {
+            GnomeCmdDir *dir = nullptr;
+	        gint     argc;
+	        gchar  **argv  = nullptr;
+	        GError  *error = nullptr;
+	        DEBUG ('g', "Edit file: %s\n", cmd.c_str());
+
+	        g_shell_parse_argv (cmd.c_str(), &argc, &argv, nullptr);
+	        if (!g_spawn_async (gnome_cmd_dir_is_local (dir) ? dir_path.c_str() : nullptr, argv, nullptr, G_SPAWN_SEARCH_PATH, nullptr, nullptr, nullptr, &error))
+            gnome_cmd_error_message (_("Unable to execute command."), error);
+
+	        g_strfreev (argv);
+	        g_free (command);
+	    }
     }
 }
 
@@ -760,6 +770,42 @@ void file_edit_new_doc (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
+void file_search (GtkMenuItem *menuitem, gpointer not_used)
+{
+    string commandString;
+    commandString.reserve(2000);
+
+    if (parse_command(&commandString, (const gchar*) gnome_cmd_data.options.search) == 0)
+    {
+	    DEBUG ('g', "Search command is empty.\n");
+	    gnome_cmd_show_message (*main_win, _("No search command given."), _("You can set a command for a search tool in the program options."));
+	    return;
+    }
+    else
+    {
+        gint     argc;
+        gchar  **argv  = nullptr;
+        GError  *error = nullptr;
+
+        DEBUG ('g', "Invoking search: %s\n", commandString.c_str());
+        // put command into argv
+        g_shell_parse_argv (commandString.c_str(), &argc, &argv, nullptr);
+        // execute command
+        if (!g_spawn_async (nullptr, argv, nullptr, G_SPAWN_SEARCH_PATH, nullptr, nullptr, nullptr, &error))
+        {
+            gnome_cmd_error_message (_("Unable to execute command."), error);
+        }
+        g_strfreev (argv);
+    }
+}
+
+
+void file_quick_search (GtkMenuItem *menuitem, gpointer not_used)
+{
+    gnome_cmd_file_list_show_quicksearch (get_fl (ACTIVE), 0);
+}
+
+
 void file_chmod (GtkMenuItem *menuitem, gpointer not_used)
 {
     GList *files = get_fl (ACTIVE)->get_selected_files();
@@ -816,7 +862,7 @@ void file_create_symlink (GtkMenuItem *menuitem, gpointer not_used)
                                                selected_files),
                                       selected_files, gnome_cmd_dir_get_display_path (inactive_fs->get_directory()));
 
-        gint choice = run_simple_dialog (*main_win, TRUE, GTK_MESSAGE_QUESTION, msg, _("Create Symbolic Link"), 1, _("Cancel"), _("Create"), NULL);
+        gint choice = run_simple_dialog (*main_win, TRUE, GTK_MESSAGE_QUESTION, msg, _("Create Symbolic Link"), 1, _("Cancel"), _("Create"), nullptr);
 
         g_free (msg);
 
@@ -1055,21 +1101,6 @@ void edit_cap_paste (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
-void edit_search (GtkMenuItem *menuitem, gpointer not_used)
-{
-    if (!main_win->file_search_dlg)
-        main_win->file_search_dlg = new GnomeCmdSearchDialog(gnome_cmd_data.search_defaults);
-
-    main_win->file_search_dlg->show_and_set_focus();
-}
-
-
-void edit_quick_search (GtkMenuItem *menuitem, gpointer not_used)
-{
-    gnome_cmd_file_list_show_quicksearch (get_fl (ACTIVE), 0);
-}
-
-
 void edit_filter (GtkMenuItem *menuitem, gpointer not_used)
 {
     get_fs (ACTIVE)->show_filter();
@@ -1102,7 +1133,7 @@ void edit_copy_fnames (GtkMenuItem *menuitem, gpointer fileList)
             get_file_list (fnames, sfl, gnome_cmd_file_get_real_path);
         else
             if (state_is_alt (mask))
-                get_file_list (fnames, sfl, gnome_cmd_file_get_uri_str, GNOME_VFS_URI_HIDE_PASSWORD);
+                get_file_list (fnames, sfl, gnome_cmd_file_get_uri_str);
 
     gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), fnames.c_str(), -1);
 
@@ -1113,7 +1144,7 @@ void edit_copy_fnames (GtkMenuItem *menuitem, gpointer fileList)
 /************** Command Menu **************/
 void command_execute (GtkMenuItem *menuitem, gpointer command)
 {
-    g_return_if_fail (command != NULL);
+    g_return_if_fail (command != nullptr);
 
     DEBUG ('g', "invoking: %s\n", command);
 
@@ -1121,7 +1152,7 @@ void command_execute (GtkMenuItem *menuitem, gpointer command)
     string quoted_dir_path;
     string cmd;
 
-    GnomeCmdDir *dir = NULL;
+    GnomeCmdDir *dir = nullptr;
 
     GnomeCmdFileList *fl = get_fl (ACTIVE);
     GList *sfl = fl->get_selected_files();
@@ -1136,7 +1167,7 @@ void command_execute (GtkMenuItem *menuitem, gpointer command)
     for (; i && GNOME_CMD_FILE (i->data)->get_parent_dir()==dir; i=i->next);
 
     if (i)
-        dir = NULL;
+        dir = nullptr;
 
     if (dir)
     {
@@ -1155,12 +1186,12 @@ void command_execute (GtkMenuItem *menuitem, gpointer command)
     {
 	gint argc;
 	gchar **argv;
-	GError *error = NULL;
+	GError *error = nullptr;
 	
 	DEBUG ('g', "Running: %s\n", cmd.c_str());
 
-	g_shell_parse_argv (cmd.c_str(), &argc, &argv, NULL);
-	if (!g_spawn_async (gnome_cmd_dir_is_local (dir) ? dir_path.c_str() : NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
+	g_shell_parse_argv (cmd.c_str(), &argc, &argv, nullptr);
+	if (!g_spawn_async (gnome_cmd_dir_is_local (dir) ? dir_path.c_str() : nullptr, argv, nullptr, G_SPAWN_SEARCH_PATH, nullptr, nullptr, nullptr, &error))
 	    gnome_cmd_error_message (_("Unable to execute command."), error);
 
 	g_strfreev (argv);
@@ -1173,12 +1204,12 @@ void command_open_terminal__internal (GtkMenuItem *menuitem, gpointer not_used)
 {
     GdkModifierType mask;
 
-    gdk_window_get_pointer (NULL, NULL, NULL, &mask);
+    gdk_window_get_pointer (nullptr, nullptr, nullptr, &mask);
 
     if (mask & GDK_SHIFT_MASK)
-        command_open_terminal_as_root (menuitem, NULL);
+        command_open_terminal_as_root (menuitem, nullptr);
     else
-        command_open_terminal (menuitem, NULL);
+        command_open_terminal (menuitem, nullptr);
 }
 
 /** 
@@ -1191,15 +1222,15 @@ void command_open_terminal (GtkMenuItem *menuitem, gpointer not_used)
     gchar **argv;
     gchar *command;
     gchar *dpath = GNOME_CMD_FILE (get_fs (ACTIVE)->get_directory())->get_real_path();
-    GError *error = NULL;
+    GError *error = nullptr;
 
     command = g_strdup (gnome_cmd_data.options.termopen);
     g_return_if_fail (command[0] != '\0');
 
     DEBUG ('g', "running: %s\n", command);
 
-    g_shell_parse_argv (command, &argc, &argv, NULL);
-    if (!g_spawn_async (dpath, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
+    g_shell_parse_argv (command, &argc, &argv, nullptr);
+    if (!g_spawn_async (dpath, argv, nullptr, G_SPAWN_SEARCH_PATH, nullptr, nullptr, nullptr, &error))
         gnome_cmd_error_message (_("Unable to execute command."), error);
 
     g_strfreev (argv);
@@ -1221,7 +1252,7 @@ void command_open_terminal_as_root (GtkMenuItem *menuitem, gpointer not_used)
 
     command = g_strdup (gnome_cmd_data.options.termopen);
     // Parse the termopen string to get the number of arguments in argc
-    g_shell_parse_argv (command, &argc, &argv, NULL);
+    g_shell_parse_argv (command, &argc, &argv, nullptr);
     g_free (argv);
 
     argv = g_new0 (char *, argc+1);
@@ -1230,15 +1261,15 @@ void command_open_terminal_as_root (GtkMenuItem *menuitem, gpointer not_used)
     if (gnome_cmd_prepend_su_to_vector (argc, argv))
     {
         gchar *dpath = GNOME_CMD_FILE (get_fs (ACTIVE)->get_directory())->get_real_path();
-        GError *error = NULL;
+        GError *error = nullptr;
 
-        if (!g_spawn_async (dpath, argv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL, NULL, NULL, NULL, &error))
+        if (!g_spawn_async (dpath, argv, nullptr, G_SPAWN_STDOUT_TO_DEV_NULL, nullptr, nullptr, nullptr, &error))
             gnome_cmd_error_message (_("Unable to open terminal in root mode."), error);
 
         g_free (dpath);
     }
     else
-        gnome_cmd_show_message (NULL, _("gksudo, xdg-su, gksu, gnomesu, kdesu or beesu is not found."));
+        gnome_cmd_show_message (nullptr, _("gksudo, xdg-su, gksu, gnomesu, kdesu or beesu is not found."));
 
     g_free (argv);
     g_free (command);
@@ -1254,13 +1285,13 @@ void command_root_mode (GtkMenuItem *menuitem, gpointer not_used)
 
     if (gnome_cmd_prepend_su_to_vector (argc, argv))
     {
-        GError *error = NULL;
+        GError *error = nullptr;
 
-        if (!g_spawn_async (NULL, argv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL, NULL, NULL, NULL, &error))
+        if (!g_spawn_async (nullptr, argv, nullptr, G_SPAWN_STDOUT_TO_DEV_NULL, nullptr, nullptr, nullptr, &error))
             gnome_cmd_error_message (_("Unable to start GNOME Commander in root mode."), error);
     }
     else
-        gnome_cmd_show_message (NULL, _("gksudo, xdg-su, gksu, gnomesu, kdesu or beesu is not found"));
+        gnome_cmd_show_message (nullptr, _("gksudo, xdg-su, gksu, gnomesu, kdesu or beesu is not found"));
 
     g_strfreev (argv);
 }
@@ -1285,6 +1316,18 @@ void mark_select_all (GtkMenuItem *menuitem, gpointer not_used)
 }
 
 
+void mark_select_all_files (GtkMenuItem *menuitem, gpointer not_used)
+{
+    get_fl (ACTIVE)->select_all_files();
+}
+
+
+void mark_unselect_all_files (GtkMenuItem *menuitem, gpointer not_used)
+{
+    get_fl (ACTIVE)->unselect_all_files();
+}
+
+
 void mark_unselect_all (GtkMenuItem *menuitem, gpointer not_used)
 {
     get_fl (ACTIVE)->unselect_all();
@@ -1382,7 +1425,7 @@ void mark_compare_directories (GtkMenuItem *menuitem, gpointer not_used)
     {
         auto f2 = static_cast<GnomeCmdFile*> (i2->data);
 
-        if (!f2->is_dotdot && f2->info->type!=GNOME_VFS_FILE_TYPE_DIRECTORY)
+        if (!f2->is_dotdot && f2->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) != G_FILE_TYPE_DIRECTORY)
             files2[f2->get_name()] = f2;
     }
 
@@ -1392,7 +1435,7 @@ void mark_compare_directories (GtkMenuItem *menuitem, gpointer not_used)
     {
         auto f1 = static_cast<GnomeCmdFile*> (i1->data);
 
-        if (f1->is_dotdot || f1->info->type==GNOME_VFS_FILE_TYPE_DIRECTORY)
+        if (f1->is_dotdot || f1->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
             continue;
 
         map<const char *,GnomeCmdFile *,ltstr>::iterator i2 = files2.find(f1->get_name());
@@ -1405,16 +1448,19 @@ void mark_compare_directories (GtkMenuItem *menuitem, gpointer not_used)
 
         GnomeCmdFile *f2 = i2->second;
 
-        if (f1->info->mtime > f2->info->mtime)          //  if f1 is newer than f2...
+        if (f1->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_TIME_MODIFIED)
+            > f2->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_TIME_MODIFIED)) //  if f1 is newer than f2...
         {
             new_selection1.insert(f1);                  //  ... add f1 to files to be selected in fl1...
             files2.erase(i2);                           //  ... and remove f2 from fl2 files
             continue;
         }
 
-        if (f1->info->mtime == f2->info->mtime)         //  if the same mtime for f1, f2
+        if (f1->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_TIME_MODIFIED)
+            == f2->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_TIME_MODIFIED)) //  if the same mtime for f1, f2
         {
-            if (f1->info->size == f2->info->size)       //  ... then check f1, f2 sizes
+            if (f1->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_STANDARD_SIZE)
+                == f2->GetGfileAttributeUInt64(G_FILE_ATTRIBUTE_STANDARD_SIZE)) //  ... then check f1, f2 sizes
                 files2.erase(i2);                       //  ... and remove f2 from fl2 files
             else
                 new_selection1.insert(f1);              //  ... or add f1 to files to be selected in fl1
@@ -1487,7 +1533,7 @@ void view_hidden_files (GtkToggleAction *toggleAction, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     auto active = gtk_toggle_action_get_active(toggleAction);
-    g_settings_set_boolean (gcmd_user_actions.settings->filter, GCMD_SETTINGS_FILTER_DOTFILE, !active);
+    g_settings_set_boolean (gcmd_user_actions.settings->filter, GCMD_SETTINGS_FILTER_HIDE_HIDDEN, !active);
 }
 
 
@@ -1496,7 +1542,7 @@ void view_backup_files (GtkToggleAction *toggleAction, gpointer not_used)
     if (!GTK_WIDGET_REALIZED (main_win)) return;
 
     auto active = gtk_toggle_action_get_active(toggleAction);
-    g_settings_set_boolean (gcmd_user_actions.settings->filter, GCMD_SETTINGS_FILTER_BACKUP, !active);
+    g_settings_set_boolean (gcmd_user_actions.settings->filter, GCMD_SETTINGS_FILTER_HIDE_BACKUPS, !active);
 }
 
 
@@ -1513,7 +1559,7 @@ void view_step_up (GtkMenuItem *menuitem, gpointer not_used)
     GnomeCmdFileSelector *fs = get_fs (ACTIVE);
     GnomeCmdFileList *fl = fs->file_list();
 
-    g_signal_emit_by_name (fl, "scroll-vertical", GTK_SCROLL_STEP_BACKWARD, 0.0, NULL);
+    g_signal_emit_by_name (fl, "scroll-vertical", GTK_SCROLL_STEP_BACKWARD, 0.0, nullptr);
 }
 
 void view_step_down (GtkMenuItem *menuitem, gpointer not_used)
@@ -1521,7 +1567,7 @@ void view_step_down (GtkMenuItem *menuitem, gpointer not_used)
     GnomeCmdFileSelector *fs = get_fs (ACTIVE);
     GnomeCmdFileList *fl = fs->file_list();
 
-    g_signal_emit_by_name (fl, "scroll-vertical", GTK_SCROLL_STEP_FORWARD, 0.0, NULL);
+    g_signal_emit_by_name (fl, "scroll-vertical", GTK_SCROLL_STEP_FORWARD, 0.0, nullptr);
 }
 
 void view_up (GtkMenuItem *menuitem, gpointer not_used)
@@ -1619,7 +1665,7 @@ void view_directory (GtkMenuItem *menuitem, gpointer not_used)
     GnomeCmdFileList *fl = fs->file_list();
 
     GnomeCmdFile *f = fl->get_selected_file();
-    if (f && f->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+    if (f && f->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
         fs->do_file_specific_action (fl, f);
 }
 
@@ -1750,7 +1796,7 @@ void view_in_new_tab (GtkMenuItem *menuitem, gpointer not_used)
     GnomeCmdFileSelector *fs = get_fs (ACTIVE);
     GnomeCmdFile *file = fs->file_list()->get_selected_file();
 
-    if (file && file->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+    if (file && file->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
         fs->new_tab(GNOME_CMD_DIR (file), FALSE);
     else
         fs->new_tab(fs->get_directory(), FALSE);
@@ -1762,7 +1808,7 @@ void view_in_inactive_tab (GtkMenuItem *menuitem, gpointer file_list)
     GnomeCmdFileList *fl = file_list ? GNOME_CMD_FILE_LIST (file_list) : get_fl (ACTIVE);
     GnomeCmdFile *file = fl->get_selected_file();
 
-    if (file && file->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
+    if (file && file->GetGfileAttributeUInt32(G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
         get_fs (INACTIVE)->new_tab(GNOME_CMD_DIR (file), FALSE);
     else
         get_fs (INACTIVE)->new_tab(fl->cwd, FALSE);
@@ -1956,18 +2002,18 @@ void help_keyboard (GtkMenuItem *menuitem, gpointer not_used)
 
 void help_web (GtkMenuItem *menuitem, gpointer not_used)
 {
-    GError *error = NULL;
+    GError *error = nullptr;
 
-    if (!gtk_show_uri (NULL, "https://gcmd.github.io/", GDK_CURRENT_TIME, &error))
+    if (!gtk_show_uri (nullptr, PACKAGE_URL, GDK_CURRENT_TIME, &error))
         gnome_cmd_error_message (_("There was an error opening home page."), error);
 }
 
 
 void help_problem (GtkMenuItem *menuitem, gpointer not_used)
 {
-    GError *error = NULL;
+    GError *error = nullptr;
 
-    if (!gtk_show_uri (NULL, "https://gitlab.gnome.org/GNOME/gnome-commander/issues", GDK_CURRENT_TIME, &error))
+    if (!gtk_show_uri (nullptr, "https://gitlab.gnome.org/GNOME/gnome-commander/issues", GDK_CURRENT_TIME, &error))
         gnome_cmd_error_message (_("There was an error reporting problem."), error);
 }
 
@@ -1979,7 +2025,7 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
         "Piotr Eljasiak <epiotr@use.pl>",
         "Assaf Gordon <agordon88@gmail.com>",
         "Uwe Scholz <u.scholz83@gmx.de>",
-        NULL
+        nullptr
     };
 
     static const gchar *documenters[] = {
@@ -1987,12 +2033,12 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
         "Piotr Eljasiak <epiotr@use.pl>",
         "Laurent Coudeur <laurentc@eircom.net>",
         "Uwe Scholz <u.scholz83@gmx.de>",
-        NULL
+        nullptr
     };
 
     static const gchar copyright[] = "Copyright \xc2\xa9 2001-2006 Marcus Bjurman\n"
                                      "Copyright \xc2\xa9 2007-2012 Piotr Eljasiak\n"
-                                     "Copyright \xc2\xa9 2013-2020 Uwe Scholz";
+                                     "Copyright \xc2\xa9 2013-2022 Uwe Scholz";
 
     static const gchar comments[] = N_("A fast and powerful file manager for the GNOME desktop");
 
@@ -2011,7 +2057,7 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
            "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.")
     };
 
-    gchar *license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]), _(license[2]), NULL);
+    gchar *license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]), _(license[2]), nullptr);
 
     gtk_show_about_dialog (*main_win,
                            "name", "GNOME Commander",
@@ -2026,7 +2072,7 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
                            "translator-credits", _("translator-credits"),
                            "website", "https://gcmd.github.io",
                            "website-label", "GNOME Commander Website",
-                           NULL);
+                           nullptr);
 
     g_free (license_trans);
 }
@@ -2045,7 +2091,7 @@ void help_about (GtkMenuItem *menuitem, gpointer not_used)
  * \li \%D: quoted full path to the directory containg file
  * \li \%%: percent sign
  * Trailing blanks are removed.
- * 
+ *
  * \param[in] command A char array to be parsed
  * \param[out] cmd A string with parsed symbols listed above
  * \returns Length of the cmd string
@@ -2067,38 +2113,51 @@ int parse_command(string *cmd, const gchar *command)
     GnomeCmdFileList *fl = get_fl (ACTIVE);
     GList *sfl = fl->get_selected_files();
 
+    if (sfl)
+    {
+        auto gnomeCmdFile = (GnomeCmdFile*) sfl->data;
+        auto gFileParent = g_file_get_parent(gnomeCmdFile->gFile);
+        dir_path = g_file_get_path(gFileParent);
+        g_object_unref(gFileParent);
+    }
+    else
+    {
+        // ToDo: Fix this misleading usage of the home directory
+        dir_path = g_get_home_dir();
+    }
+
+
     cmdcap = cmd->capacity();
     cmdlen = cmd->length();
 
-    for (auto s = command; *s; ++s)
+    for (auto s = command; *s; ++s) // loop over chars of command-string set by user
     {
-        if (!percent)
+        if (!percent) // check if % not found
         {
-            percent = (*s == '%');
-
-            if (!percent)
-	    {
-		if (cmdcap < cmdlen + 1)
-		{
-		    cmd->reserve(cmdlen +1);
-		    cmdcap = cmd->capacity();
-		}
-		if (blcheck) // copy letter by letter, but remove trailing blanks
-		    *cmd += *s;
-		else
-		{
-		    if (*s == ' ' || *s == '\t') 
-		    {
-			continue;
-		    }
-		    else
-		    {
-			blcheck = TRUE;
-			*cmd += *s;
-		    }
-		}
-		cmdlen = cmd->length();
-	    }
+            percent = (*s == '%'); // true: if s=%, false: if s!=%
+            if (!percent) // check if % not found
+            {
+                if (cmdcap < cmdlen + 1)
+                {
+                    cmd->reserve(cmdlen +1);
+                    cmdcap = cmd->capacity();
+                }
+                if (blcheck) // copy letter by letter, but remove trailing blanks
+                    *cmd += *s;
+                else
+                {
+                    if (*s == ' ' || *s == '\t')
+                    {
+                    continue;
+                    }
+                    else
+                    {
+                    blcheck = TRUE;
+                    *cmd += *s;
+                    }
+                }
+                cmdlen = cmd->length();
+            }
 
             continue;
         }
@@ -2156,7 +2215,7 @@ int parse_command(string *cmd, const gchar *command)
 
             case 'u':           // %u  fully qualified URI for the file (or list for multiple selections)
                 if (uri.empty())
-                    get_file_list (uri, sfl, gnome_cmd_file_get_uri_str, GNOME_VFS_URI_HIDE_PASSWORD);
+                    get_file_list (uri, sfl, gnome_cmd_file_get_uri_str);
 		if (cmdcap < cmdlen + uri.length())
 		{
 		    cmd->reserve(cmdlen + uri.length());
