@@ -2,7 +2,7 @@
  * @file gnome-cmd-user-actions.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -234,6 +234,8 @@ GNOME_CMD_USER_ACTION(file_internal_view);
 GNOME_CMD_USER_ACTION(file_external_view);
 GNOME_CMD_USER_ACTION(file_edit);
 GNOME_CMD_USER_ACTION(file_edit_new_doc);
+GNOME_CMD_USER_ACTION(file_search);
+GNOME_CMD_USER_ACTION(file_quick_search);
 GNOME_CMD_USER_ACTION(file_chmod);
 GNOME_CMD_USER_ACTION(file_chown);
 GNOME_CMD_USER_ACTION(file_mkdir);
@@ -253,6 +255,8 @@ GNOME_CMD_USER_ACTION(mark_toggle);
 GNOME_CMD_USER_ACTION(mark_toggle_and_step);
 GNOME_CMD_USER_ACTION(mark_select_all);
 GNOME_CMD_USER_ACTION(mark_unselect_all);
+GNOME_CMD_USER_ACTION(mark_select_all_files);
+GNOME_CMD_USER_ACTION(mark_unselect_all_files);
 GNOME_CMD_USER_ACTION(mark_select_with_pattern);
 GNOME_CMD_USER_ACTION(mark_unselect_with_pattern);
 GNOME_CMD_USER_ACTION(mark_invert_selection);
@@ -265,8 +269,6 @@ GNOME_CMD_USER_ACTION(mark_compare_directories);
 GNOME_CMD_USER_ACTION(edit_cap_cut);
 GNOME_CMD_USER_ACTION(edit_cap_copy);
 GNOME_CMD_USER_ACTION(edit_cap_paste);
-GNOME_CMD_USER_ACTION(edit_search);
-GNOME_CMD_USER_ACTION(edit_quick_search);
 GNOME_CMD_USER_ACTION(edit_filter);
 GNOME_CMD_USER_ACTION(edit_copy_fnames);
 
