@@ -2,7 +2,7 @@
  * @file imageloader.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,6 +82,7 @@ static CacheEntry file_type_pixmaps[NUM_FILE_TYPE_PIXMAPS];
 static GHashTable *mime_cache = nullptr;
 static GdkPixbuf *symlink_pixbuf = nullptr;
 
+static const gint ICON_SIZE = 16;
 
 static gboolean load_icon (const gchar *icon_path, GdkPixmap **pm, GdkBitmap **bm, GdkPixmap **lpm, GdkBitmap **lbm);
 
@@ -179,15 +180,11 @@ inline char *get_mime_icon_name (const gchar *mime_type)
  * Returns the filename that an image representing the given filetype
  * should have.
  */
-static const gchar *get_type_icon_name (GnomeVFSFileType type)
+static const gchar *get_type_icon_name (guint32 type)
 {
     static const gchar *names[] = {
         "i-directory.png",
         "i-regular.png",
-        "i-chardev.png",
-        "i-blockdev.png",
-        "i-fifo.png",
-        "i-socket.png",
         "i-symlink.png"
     };
 
@@ -197,20 +194,13 @@ static const gchar *get_type_icon_name (GnomeVFSFileType type)
 #endif
     switch (type)
     {
-        case GNOME_VFS_FILE_TYPE_DIRECTORY:
+        case G_FILE_TYPE_DIRECTORY:
             return names[0];
-        case GNOME_VFS_FILE_TYPE_REGULAR:
+        case G_FILE_TYPE_REGULAR:
             return names[1];
-        case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
-            return names[2];
-        case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
+        case G_FILE_TYPE_SYMBOLIC_LINK:
             return names[3];
-        case GNOME_VFS_FILE_TYPE_FIFO:
-            return names[4];
-        case GNOME_VFS_FILE_TYPE_SOCKET:
-            return names[5];
-        case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
-            return names[6];
+        //TODO: Add filetype names for G_FILE_TYPE_SHORTCUT and G_FILE_TYPE_MOUNTABLE
 
         default:
             return names[1];
@@ -225,9 +215,8 @@ static const gchar *get_type_icon_name (GnomeVFSFileType type)
 
 /**
  * Returns the full path to an image for the given filetype in the given directory.
- *
  */
-inline gchar *get_mime_file_type_icon_path (GnomeVFSFileType type, const gchar *icon_dir)
+inline gchar *get_mime_file_type_icon_path (guint32 type, const gchar *icon_dir)
 {
     return g_build_filename (icon_dir, get_type_icon_name (type), nullptr);
 }
@@ -327,7 +316,7 @@ static gboolean load_icon (const gchar *icon_path, GdkPixmap **pm, GdkBitmap **b
  * If symlink is true a smaller symlink image is painted over the image to indicate this.
  */
 static gboolean get_mime_icon_in_dir (const gchar *icon_dir,
-                                      GnomeVFSFileType type,
+                                      guint32 type,
                                       const gchar *mime_type,
                                       gboolean symlink,
                                       GdkPixmap **pixmap,
@@ -336,7 +325,7 @@ static gboolean get_mime_icon_in_dir (const gchar *icon_dir,
     if (!mime_type)
         return FALSE;
 
-    if (type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK)
+    if (type == G_FILE_TYPE_SYMBOLIC_LINK)
         return FALSE;
 
     auto entry = static_cast<CacheEntry*> (g_hash_table_lookup (mime_cache, mime_type));
@@ -399,7 +388,7 @@ static gboolean get_mime_icon_in_dir (const gchar *icon_dir,
 }
 
 
-static gboolean get_mime_icon (GnomeVFSFileType type,
+static gboolean get_mime_icon (guint32 type,
                                const gchar *mime_type,
                                gboolean symlink,
                                GdkPixmap **pixmap,
@@ -412,7 +401,7 @@ static gboolean get_mime_icon (GnomeVFSFileType type,
 }
 
 
-inline gboolean get_type_icon (GnomeVFSFileType type,
+inline gboolean get_type_icon (guint32 type,
                                gboolean symlink,
                                GdkPixmap **pixmap,
                                GdkBitmap **mask)
@@ -426,7 +415,7 @@ inline gboolean get_type_icon (GnomeVFSFileType type,
 }
 
 
-gboolean IMAGE_get_pixmap_and_mask (GnomeVFSFileType type,
+gboolean IMAGE_get_pixmap_and_mask (guint32 type,
                                     const gchar *mime_type,
                                     gboolean symlink,
                                     GdkPixmap **pixmap,
@@ -583,4 +572,25 @@ char* register_application_stock_icon(const char* applicationName, const char* d
     g_object_unref (icon_factory);
 
     return stockId;
-}
\ No newline at end of file
+}
+
+/**
+ * This method returns the path of the default application icon for a given GAppInfo object.
+ * The returned char must not be free'ed.
+ */
+const gchar* get_default_application_icon_path(GAppInfo* appInfo)
+{
+    g_return_val_if_fail (appInfo, nullptr);
+
+    auto appIconName = g_icon_to_string (g_app_info_get_icon(appInfo));
+    auto gtkIconInfo = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), appIconName, ICON_SIZE, GTK_ICON_LOOKUP_FORCE_SIZE);
+
+    g_free(appIconName);
+
+    if (gtkIconInfo == nullptr)
+    {
+        return nullptr;
+    }
+
+    return gtk_icon_info_get_filename(gtkIconInfo);
+}
