@@ -4,7 +4,7 @@
  *
  * @copyright (C) 2006 Assaf Gordon\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@ using namespace std;
 
 #define IMAGE_RENDER_DEFAULT_WIDTH      100
 #define IMAGE_RENDER_DEFAULT_HEIGHT     200
+#define INC_VALUE 25.0
 
 
 enum {
@@ -65,6 +66,8 @@ struct ImageRenderClass
 struct ImageRender::Private
 {
     guint8 button; // The button pressed in "button_press_event"
+    gint mouseX;   // Old x position before mouse move
+    gint mouseY;   // Old y position before mouse move
 
     GtkAdjustment *h_adjustment;
     // Old values from h_adjustment stored so we know when something changes
@@ -97,6 +100,7 @@ G_DEFINE_TYPE (ImageRender, image_render, GTK_TYPE_WIDGET)
 static void image_render_redraw (ImageRender *w);
 
 static gboolean image_render_key_press (GtkWidget *widget, GdkEventKey *event);
+static gboolean image_render_scroll(GtkWidget *widget, GdkEventScroll *event);
 
 static void image_render_realize (GtkWidget *widget);
 static void image_render_size_request (GtkWidget *widget, GtkRequisition *requisition);
@@ -124,6 +128,17 @@ static void image_render_update_adjustments (ImageRender *obj);
     public functions
     (defined in the header file)
 *****************************************/
+GtkAdjustment *image_render_get_h_adjustment (ImageRender *obj)
+{
+    g_return_val_if_fail (IS_IMAGE_RENDER(obj), nullptr);
+
+    if (obj->priv->h_adjustment)
+    {
+        return obj->priv->h_adjustment;
+    }
+    return nullptr;
+}
+
 void image_render_set_h_adjustment (ImageRender *obj, GtkAdjustment *adjustment)
 {
     g_return_if_fail (IS_IMAGE_RENDER(obj));
@@ -148,6 +163,17 @@ void image_render_set_h_adjustment (ImageRender *obj, GtkAdjustment *adjustment)
 }
 
 
+GtkAdjustment *image_render_get_v_adjustment (ImageRender *obj)
+{
+    g_return_val_if_fail (IS_IMAGE_RENDER(obj), nullptr);
+
+    if (obj->priv->v_adjustment)
+    {
+        return obj->priv->v_adjustment;
+    }
+    return nullptr;
+}
+
 void image_render_set_v_adjustment (ImageRender *obj, GtkAdjustment *adjustment)
 {
     g_return_if_fail (IS_IMAGE_RENDER(obj));
@@ -245,6 +271,7 @@ static void image_render_class_init (ImageRenderClass *klass)
 
     object_class->finalize = image_render_finalize;
 
+    widget_class->scroll_event = image_render_scroll;
     widget_class->key_press_event = image_render_key_press;
     widget_class->button_press_event = image_render_button_press;
     widget_class->button_release_event = image_render_button_release;
@@ -293,10 +320,172 @@ void image_render_notify_status_changed (ImageRender *w)
 
 static gboolean image_render_key_press (GtkWidget *widget, GdkEventKey *event)
 {
+    g_return_val_if_fail (IS_IMAGE_RENDER (widget), FALSE);
+
+    auto imageRender = IMAGE_RENDER (widget);
+
+    switch (event->keyval)
+    {
+        case GDK_KEY_Up:
+        {
+            auto vAdjustment = image_render_get_v_adjustment(imageRender);
+            auto current = gtk_adjustment_get_value(vAdjustment);
+            auto lower = gtk_adjustment_get_lower(vAdjustment);
+            if (current - INC_VALUE > lower)
+            {
+                gtk_adjustment_set_value(vAdjustment, current - INC_VALUE);
+            }
+            else
+            {
+                gtk_adjustment_set_value(vAdjustment, lower);
+            }
+
+            return TRUE;
+        }
+        case GDK_KEY_Down:
+        {
+            auto vAdjustment = image_render_get_v_adjustment(imageRender);
+            auto current = gtk_adjustment_get_value(vAdjustment);
+            auto upper = gtk_adjustment_get_upper(vAdjustment);
+            auto page_size = gtk_adjustment_get_page_size(vAdjustment);
+            if (current + INC_VALUE < upper - page_size)
+            {
+                gtk_adjustment_set_value(vAdjustment, current + INC_VALUE);
+            }
+            else
+            {
+                gtk_adjustment_set_value(vAdjustment, upper - page_size);
+            }
+            return TRUE;
+        }
+        case GDK_KEY_Left:
+        {
+            auto hAdjustment = image_render_get_h_adjustment(imageRender);
+            auto current = gtk_adjustment_get_value(hAdjustment);
+            auto lower = gtk_adjustment_get_lower(hAdjustment);
+            if (current - INC_VALUE > lower)
+            {
+                gtk_adjustment_set_value(hAdjustment, current - INC_VALUE);
+            }
+            else
+            {
+                gtk_adjustment_set_value(hAdjustment, lower);
+            }
+
+            return TRUE;
+        }
+        case GDK_KEY_Right:
+        {
+            auto hAdjustment = image_render_get_h_adjustment(imageRender);
+            auto current = gtk_adjustment_get_value(hAdjustment);
+            auto upper = gtk_adjustment_get_upper(hAdjustment);
+            auto page_size = gtk_adjustment_get_page_size(hAdjustment);
+            if (current + INC_VALUE < upper - page_size)
+            {
+                gtk_adjustment_set_value(hAdjustment, current + INC_VALUE);
+            }
+            else
+            {
+                gtk_adjustment_set_value(hAdjustment, upper - page_size);
+            }
+
+            return TRUE;
+        }
+
+        default:
+           break;
+    }
+
+    switch (state_is_blank(event->keyval))
+    {
+        default:
+           break;
+    }
+
     return FALSE;
 }
 
 
+static gboolean image_render_scroll(GtkWidget *widget, GdkEventScroll *event)
+{
+    g_return_val_if_fail (IS_IMAGE_RENDER (widget), FALSE);
+    g_return_val_if_fail (event != NULL, FALSE);
+
+    auto imageRender = IMAGE_RENDER (widget);
+
+    // Mouse scroll wheel
+#if defined (__GNUC__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wswitch-enum"
+#endif
+    if (state_is_ctrl(event->state))
+    {
+        switch (event->direction)
+        {
+            case GDK_SCROLL_UP:
+            {
+                auto hAdjustment = image_render_get_h_adjustment(imageRender);
+                auto current = gtk_adjustment_get_value(hAdjustment);
+                auto lower = gtk_adjustment_get_lower(hAdjustment);
+                if (current > lower)
+                {
+                    gtk_adjustment_set_value(hAdjustment, current - INC_VALUE);
+                }
+                return TRUE;
+            }
+            case GDK_SCROLL_DOWN:
+            {
+                auto hAdjustment = image_render_get_h_adjustment(imageRender);
+                auto current = gtk_adjustment_get_value(hAdjustment);
+                auto upper = gtk_adjustment_get_upper(hAdjustment);
+                auto page_size = gtk_adjustment_get_page_size(hAdjustment);
+                if (current < upper - page_size)
+                {
+                    gtk_adjustment_set_value(hAdjustment, current + INC_VALUE);
+                }
+                return TRUE;
+            }
+            default:
+                return FALSE;
+        }
+    }
+
+    switch (event->direction)
+    {
+        case GDK_SCROLL_UP:
+        {
+            auto vAdjustment = image_render_get_v_adjustment(imageRender);
+            auto current = gtk_adjustment_get_value(vAdjustment);
+            auto lower = gtk_adjustment_get_lower(vAdjustment);
+            if (current > lower)
+            {
+                gtk_adjustment_set_value(vAdjustment, current - INC_VALUE);
+            }
+            return TRUE;
+        }
+        case GDK_SCROLL_DOWN:
+        {
+            auto vAdjustment = image_render_get_v_adjustment(imageRender);
+            auto current = gtk_adjustment_get_value(vAdjustment);
+            auto upper = gtk_adjustment_get_upper(vAdjustment);
+            auto page_size = gtk_adjustment_get_page_size(vAdjustment);
+            if (current < upper - page_size)
+            {
+                gtk_adjustment_set_value(vAdjustment, current + INC_VALUE);
+            }
+            return TRUE;
+        }
+        default:
+            return FALSE;
+    }
+#if defined (__GNUC__)
+#pragma GCC diagnostic pop
+#endif
+
+    return TRUE;
+}
+
+
 static void image_render_realize (GtkWidget *widget)
 {
     g_return_if_fail (IS_IMAGE_RENDER (widget));
@@ -478,6 +667,10 @@ static gboolean image_render_button_press (GtkWidget *widget, GdkEventButton *ev
         w->priv->button = event->button;
 
         // gtk_dial_update_mouse (dial, event->x, event->y);
+
+        // Store current mouse position
+        w->priv->mouseX = event->x;
+        w->priv->mouseY = event->y;
     }
 
     return FALSE;
@@ -507,9 +700,9 @@ static gboolean image_render_motion_notify (GtkWidget *widget, GdkEventMotion *e
     g_return_val_if_fail (IS_IMAGE_RENDER (widget), FALSE);
     g_return_val_if_fail (event != NULL, FALSE);
 
-    ImageRender *w = IMAGE_RENDER (widget);
+    auto imageRender = IMAGE_RENDER (widget);
 
-    if (w->priv->button != 0)
+    if (imageRender->priv->button != 0)
     {
         GdkModifierType mods;
 
@@ -520,7 +713,33 @@ static gboolean image_render_motion_notify (GtkWidget *widget, GdkEventMotion *e
 
         if (event->is_hint || (event->window != window))
             gdk_window_get_pointer (window, &x, &y, &mods);
-        // TODO: respond to motion event
+
+        // Update X position
+        auto hAdjustment = image_render_get_h_adjustment(imageRender);
+        auto currentX = gtk_adjustment_get_value(hAdjustment);
+        auto lowerX = gtk_adjustment_get_lower(hAdjustment);
+        auto upperX = gtk_adjustment_get_upper(hAdjustment);
+        auto pageSizeX = gtk_adjustment_get_page_size(hAdjustment);
+        if (currentX + (imageRender->priv->mouseX - x)    < upperX - pageSizeX
+            && currentX + (imageRender->priv->mouseX - x) > lowerX - pageSizeX)
+        {
+            gtk_adjustment_set_value(hAdjustment, currentX + (imageRender->priv->mouseX - x));
+        }
+
+        // Update Y position
+        auto vAdjustment = image_render_get_v_adjustment(imageRender);
+        auto currentY = gtk_adjustment_get_value(vAdjustment);
+        auto lowerY = gtk_adjustment_get_lower(vAdjustment);
+        auto upperY = gtk_adjustment_get_upper(vAdjustment);
+        auto pageSizeY = gtk_adjustment_get_page_size(vAdjustment);
+        if (currentY + (imageRender->priv->mouseY - y)    < upperY - pageSizeY
+            && currentY + (imageRender->priv->mouseY - y) > lowerY - pageSizeY)
+        {
+            gtk_adjustment_set_value(vAdjustment, currentY + (imageRender->priv->mouseY - y));
+        }
+
+        imageRender->priv->mouseX = x;
+        imageRender->priv->mouseY = y;
     }
 
     return FALSE;
