@@ -2,7 +2,7 @@
  * @file ls_colors.h
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 
 struct LsColor
 {
-    GnomeVFSFileType type;
+    GFileType type;
     gchar *ext;
     GdkColor *fg, *bg;
 };
