@@ -2,7 +2,7 @@
  * @file main.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-main-win.h"
-#include "gnome-cmd-mime-config.h"
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-user-actions.h"
 #include "gnome-cmd-owner.h"
@@ -102,19 +101,11 @@ int main (int argc, char *argv[])
 
     main_win = nullptr;
 
-    if (!g_thread_supported ())
-    {
-        g_printerr ("GNOME Commander needs thread support in glib to work, bailing out...\n");
-        return 0;
-    }
-
     setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, DATADIR "/locale");
     bind_textdomain_codeset (PACKAGE, "UTF-8");
     textdomain (PACKAGE);
 
-    gnome_cmd_mime_config();
-
     if (!gtk_init_with_args (&argc, &argv,
                              _("File Manager"),
                              options,
@@ -129,22 +120,12 @@ int main (int argc, char *argv[])
         debug_flags = g_strdup("cdfgiklmnpstuvwyzx");
 
     gdk_rgb_init ();
-    gnome_vfs_init ();
-
-
-    // ToDo: Remove somewhen after 1.10 release:
-    ///////////////////////////////////
-    auto userConfigDirOld = g_build_filename (g_get_home_dir (), "." PACKAGE, nullptr);
-
-    if (is_dir_existing(userConfigDirOld) == 1)
-    {
-        auto userPluginConfigDirNew = get_package_config_dir();
-        move_old_to_new_location(userConfigDirOld, userPluginConfigDirNew);
-    }
-    ///////////////////////////////////
 
     gchar *conf_dir = get_package_config_dir();
-    create_dir_if_needed (conf_dir);
+    if (!is_dir_existing(conf_dir))
+    {
+        create_dir (conf_dir);
+    }
     g_free (conf_dir);
 
     /* Load Settings */
@@ -164,10 +145,24 @@ int main (int argc, char *argv[])
     {
 #endif
         if (start_dir_left)
+        {
+            if (!strcmp(start_dir_left, "."))
+            {
+                g_free(start_dir_left);
+                start_dir_left = g_get_current_dir ();
+            }
             gnome_cmd_data.tabs[LEFT].push_back(make_pair(string(start_dir_left),make_triple(GnomeCmdFileList::COLUMN_NAME,GTK_SORT_ASCENDING,FALSE)));
+        }
 
         if (start_dir_right)
+        {
+            if (!strcmp(start_dir_right, "."))
+            {
+                g_free(start_dir_right);
+                start_dir_right = g_get_current_dir ();
+            }
             gnome_cmd_data.tabs[RIGHT].push_back(make_pair(string(start_dir_right),make_triple(GnomeCmdFileList::COLUMN_NAME,GTK_SORT_ASCENDING,FALSE)));
+        }
 
         gcmd_user_actions.set_defaults();
         ls_colors_init ();
@@ -204,12 +199,13 @@ int main (int argc, char *argv[])
     }
 #endif
 
-    gnome_vfs_shutdown ();
-
 #ifdef HAVE_UNIQUE
     g_object_unref (app);
 #endif
     g_free (debug_flags);
+    g_free (start_dir_left);
+    g_free (start_dir_right);
+    g_free (config_dir);
 
     DEBUG ('c', "dirs total: %d remaining: %d\n", created_dirs_cnt, created_dirs_cnt - deleted_dirs_cnt);
     DEBUG ('c', "files total: %d remaining: %d\n", created_files_cnt, created_files_cnt - deleted_files_cnt);
