@@ -2,7 +2,7 @@
  * @file plugin_manager.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "imageloader.h"
 #include "gnome-cmd-main-win.h"
 #include "gnome-cmd-about-plugin.h"
+#include "dirlist.h"
 
 using namespace std;
 
@@ -144,70 +145,38 @@ static void inactivate_plugin (PluginData *data)
 
 static void scan_plugins_in_dir (const gchar *dpath)
 {
-    DIR *dir = opendir (dpath);
-    char buff[256];
-    char *prev_dir;
-    struct dirent *ent;
+    auto gFileInfoList = sync_dir_list(dpath);
 
-    if (!dir)
-    {
-        g_warning ("Could not list files in %s: %s", dpath, strerror (errno));
+    if (g_list_length(gFileInfoList) == 0)
         return;
-    }
 
-    prev_dir = getcwd (buff, sizeof(buff));
-    if (chdir (dpath))
+    for (auto gFileInfoListItem = gFileInfoList; gFileInfoListItem; gFileInfoListItem = gFileInfoListItem->next)
     {
-        g_warning ("Could not change directory to %s: %s", dpath, strerror (errno));
-        return;
-    }
-
-    while ((ent = readdir (dir)) != nullptr)
-    {
-        struct stat buf;
-
-        if (strcmp (ent->d_name+strlen(ent->d_name)-strlen("." G_MODULE_SUFFIX), "." G_MODULE_SUFFIX) != 0)
+        auto gFileInfo = (GFileInfo*) gFileInfoListItem->data;
+        if (g_file_info_get_file_type(gFileInfo) != G_FILE_TYPE_REGULAR)
+            continue;
+        if (!g_str_has_suffix (g_file_info_get_name(gFileInfo), "." G_MODULE_SUFFIX))
             continue;
 
-        if (stat (ent->d_name, &buf) == 0)
+        // the direntry has the correct extension and is a regular file, let's accept it
+        PluginData *data = g_new0 (PluginData, 1);
+        data->fname = g_strdup (g_file_info_get_name(gFileInfo));
+        data->fpath = g_build_filename (dpath, g_file_info_get_name(gFileInfo), nullptr);
+        data->loaded = FALSE;
+        data->active = FALSE;
+        data->menu = nullptr;
+        data->autoload = FALSE;
+        activate_plugin (data);
+        if (!data->loaded)
         {
-            if (buf.st_mode & S_IFREG)
-            {
-                // the direntry has the correct extension and is a regular file, let's accept it
-                PluginData *data = g_new0 (PluginData, 1);
-                data->fname = g_strdup (ent->d_name);
-                data->fpath = g_build_filename (dpath, ent->d_name, nullptr);
-                data->loaded = FALSE;
-                data->active = FALSE;
-                data->menu = nullptr;
-                data->autoload = FALSE;
-                activate_plugin (data);
-                if (!data->loaded)
-                {
-                    g_free (data->fname);
-                    g_free (data->fpath);
-                    g_free (data);
-                }
-                else
-                    plugins = g_list_append (plugins, data);
-            }
-            else
-                printf ("%s is not a regular file\n", ent->d_name);
+            g_free (data->fname);
+            g_free (data->fpath);
+            g_free (data);
         }
         else
-            printf ("Failed to stat %s\n", ent->d_name);
-    }
-
-    closedir (dir);
-
-    if (prev_dir)
-    {
-        if (chdir (prev_dir))
-        {
-            g_warning ("Could not change directory back to %s: %s", prev_dir, strerror (errno));
-            return;
-        }
+            plugins = g_list_append (plugins, data);
     }
+    g_list_free_full(gFileInfoList, g_object_unref);
 }
 
 
@@ -221,10 +190,13 @@ void plugin_manager_init ()
 
     // find user plugins
     gchar *user_dir = get_plugin_config_location();
-    if (create_dir_if_needed (user_dir))
+    if (!is_dir_existing(user_dir))
     {
-        scan_plugins_in_dir (user_dir);
+        create_dir (user_dir);
     }
+
+    scan_plugins_in_dir (user_dir);
+
     g_free (user_dir);
 
     // find system plugins
@@ -472,29 +444,12 @@ gchar* get_plugin_config_location()
     string userPluginConfigDir = get_package_config_dir();
     userPluginConfigDir += (char*) "/plugins";
 
-    auto dir_exists = is_dir_existing(userPluginConfigDir.c_str());
-
-    switch (dir_exists)
+    if (!is_dir_existing(userPluginConfigDir.c_str()))
     {
-        case 0:
-        {
-            if (create_dir_if_needed (userPluginConfigDir.c_str()))
-            {
-                returnString = g_strdup(userPluginConfigDir.c_str());
-            }
-            break;
-        }
-        case 1:
-        {
-            returnString = g_strdup(userPluginConfigDir.c_str());
-            break;
-        }
-        case -1:
-        default:
-        {
-            break;
-        }
+        create_dir (userPluginConfigDir.c_str());
     }
 
+    returnString = g_strdup(userPluginConfigDir.c_str());
+
     return returnString;
 }
