@@ -2,7 +2,7 @@
  * @file gnome-cmd-tags-exiv2.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ static DICT<GnomeCmdTag> exiv2_tags(TAG_NONE);
 
 
 template <typename T>
-inline void readTags(GnomeCmdFileMetadata *metadata, const T &data)
+void readTags(GnomeCmdFileMetadata *metadata, const T &data)
 {
     if (data.empty())  return;
 
@@ -74,7 +74,7 @@ inline void readTags(GnomeCmdFileMetadata *metadata, const T &data)
             case TAG_EXIF_FLASHPIXVERSION:
             case TAG_EXIF_INTEROPERABILITYVERSION:
                 {
-                    vector<byte> buff(i->value().size()+1);
+                    vector<Exiv2::byte> buff(i->value().size()+1);
 
                     i->value().copy(&buff[0],invalidByteOrder);
                     metadata->add(tag,(char *) &buff[0]);
@@ -374,7 +374,7 @@ void gcmd_tags_exiv2_init()
 void gcmd_tags_exiv2_load_metadata(GnomeCmdFile *f)
 {
     g_return_if_fail (f != NULL);
-    g_return_if_fail (f->info != NULL);
+    g_return_if_fail (f->gFileInfo != NULL);
 
     if (f->metadata && f->metadata->is_accessed(TAG_IMAGE))  return;
 
