@@ -2,7 +2,7 @@
  * @file gnome-cmd-tags-taglib.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,13 +47,15 @@
 
 using namespace std;
 
-
 #ifdef HAVE_ID3
+bool getAudioProperties(GnomeCmdFileMetadata &metadata, const TagLib::AudioProperties *properties);
+bool readTags(GnomeCmdFileMetadata &metadata, const TagLib::Ogg::XiphComment *oggTag);
+
 static DICT<GnomeCmdTag> id3v2tags(TAG_NONE);
 static DICT<GnomeCmdTag> oggtags(TAG_NONE);
 
 
-inline bool getAudioProperties(GnomeCmdFileMetadata &metadata, const TagLib::AudioProperties *properties)
+bool getAudioProperties(GnomeCmdFileMetadata &metadata, const TagLib::AudioProperties *properties)
 {
     if (!properties)
         return false;
@@ -158,7 +160,7 @@ inline bool getAudioProperties(GnomeCmdFileMetadata &metadata, const TagLib::Aud
 
     if (mpcProperties)
     {
-        if (oggProperties->vorbisVersion())
+        if (oggProperties && oggProperties->vorbisVersion())
             cout << "Audio.MPC.Version  " << mpcProperties->mpcVersion() << endl;
 
         return true;
@@ -247,7 +249,7 @@ static void readTags(GnomeCmdFileMetadata &metadata, const TagLib::ID3v2::Tag *i
 }
 
 
-inline bool readTags(GnomeCmdFileMetadata &metadata, const TagLib::Ogg::XiphComment *oggTag)
+bool readTags(GnomeCmdFileMetadata &metadata, const TagLib::Ogg::XiphComment *oggTag)
 {
     metadata.add(TAG_VORBIS_VENDOR, oggTag->vendorID().to8Bit(true));
 
@@ -436,28 +438,28 @@ void gcmd_tags_taglib_init()
 }
 
 
-void gcmd_tags_taglib_load_metadata(GnomeCmdFile *finfo)
+void gcmd_tags_taglib_load_metadata(GnomeCmdFile *gnomeCmdFile)
 {
-    g_return_if_fail (finfo != NULL);
-    g_return_if_fail (finfo->info != NULL);
-
 #ifdef HAVE_ID3
-    if (finfo->metadata && finfo->metadata->is_accessed(TAG_AUDIO))  return;
+    g_return_if_fail (gnomeCmdFile != nullptr);
+    g_return_if_fail (gnomeCmdFile->gFileInfo != nullptr);
 
-    if (!finfo->metadata)
-        finfo->metadata = new GnomeCmdFileMetadata;
+    if (gnomeCmdFile->metadata && gnomeCmdFile->metadata->is_accessed(TAG_AUDIO))  return;
 
-    if (!finfo->metadata)  return;
+    if (!gnomeCmdFile->metadata)
+        gnomeCmdFile->metadata = new GnomeCmdFileMetadata;
 
-    finfo->metadata->mark_as_accessed(TAG_AUDIO);
-    finfo->metadata->mark_as_accessed(TAG_APE);
-    finfo->metadata->mark_as_accessed(TAG_FLAC);
-    finfo->metadata->mark_as_accessed(TAG_ID3);
-    finfo->metadata->mark_as_accessed(TAG_VORBIS);
+    if (!gnomeCmdFile->metadata)  return;
 
-    if (!finfo->is_local())  return;
+    gnomeCmdFile->metadata->mark_as_accessed(TAG_AUDIO);
+    gnomeCmdFile->metadata->mark_as_accessed(TAG_APE);
+    gnomeCmdFile->metadata->mark_as_accessed(TAG_FLAC);
+    gnomeCmdFile->metadata->mark_as_accessed(TAG_ID3);
+    gnomeCmdFile->metadata->mark_as_accessed(TAG_VORBIS);
 
-    const gchar *fname = finfo->get_real_path();
+    if (!gnomeCmdFile->is_local())  return;
+
+    const gchar *fname = gnomeCmdFile->get_real_path();
 
     DEBUG('t', "Loading audio metadata for '%s'\n", fname);
 
@@ -466,7 +468,7 @@ void gcmd_tags_taglib_load_metadata(GnomeCmdFile *finfo)
     if (f.isNull())
         return;
 
-    getAudioProperties(*finfo->metadata, f.audioProperties());
-    getTag(*finfo->metadata, f.file(), f.tag());
+    getAudioProperties(*gnomeCmdFile->metadata, f.audioProperties());
+    getTag(*gnomeCmdFile->metadata, f.file(), f.tag());
 #endif
 }
