@@ -2,7 +2,7 @@
  * @file utils.cc
  * @copyright (C) 2001-2006 Marcus Bjurman\n
  * @copyright (C) 2007-2012 Piotr Eljasiak\n
- * @copyright (C) 2013-2020 Uwe Scholz\n
+ * @copyright (C) 2013-2022 Uwe Scholz\n
  *
  * @copyright This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "gnome-cmd-data.h"
 #include "imageloader.h"
 #include "gnome-cmd-main-win.h"
+#include "gnome-cmd-user-actions.h"
 
 using namespace std;
 
@@ -83,7 +84,7 @@ void DEBUG (gchar flag, const gchar *format, ...)
  * terminal window, if desired.
  * \param in_command Command to be executed.
  * \param dpath Directory in which the command should be executed.
- * \param term If TRUE, the command is executed in a terminal window. 
+ * \param term If TRUE, the command is executed in a terminal window.
  * \sa GnomeCmdData::Options::termexec
  */
 void run_command_indir (const gchar *in_command, const gchar *dpath, gboolean term)
@@ -126,7 +127,18 @@ void run_command_indir (const gchar *in_command, const gchar *dpath, gboolean te
     gchar **argv;
     GError *error = NULL;
 
-    g_shell_parse_argv (command, &argc, &argv, NULL);
+    // check if command includes % and replace
+    string cmd;
+	cmd.reserve(2000);
+	if (parse_command(&cmd, (const gchar*) command) == 0)
+	{
+	    DEBUG ('g', "run_command_indir: command is not valid.\n");
+	    gnome_cmd_show_message (*main_win, _("No valid command given."));
+	    return;
+	}
+
+    //g_shell_parse_argv (command, &argc, &argv, NULL);
+    g_shell_parse_argv (cmd.c_str(), &argc, &argv, NULL); // include parse_command
     if (!g_spawn_async (dpath, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
         gnome_cmd_error_message (_("Unable to execute command."), error);
 
@@ -228,35 +240,32 @@ gint run_simple_dialog (GtkWidget *parent, gboolean ignore_close_box,
     return result;
 }
 
-const gchar *type2string (GnomeVFSFileType type, gchar *buf, guint max)
+const gchar *type2string (guint32 type, gchar *buf, guint max)
 {
     const char *s;
 
     switch (type)
     {
-        case GNOME_VFS_FILE_TYPE_UNKNOWN:
+        case G_FILE_TYPE_UNKNOWN:
             s = "?";
             break;
-        case GNOME_VFS_FILE_TYPE_REGULAR:
+        case G_FILE_TYPE_REGULAR:
             s = " ";
             break;
-        case GNOME_VFS_FILE_TYPE_DIRECTORY:
+        case G_FILE_TYPE_DIRECTORY:
             s = G_DIR_SEPARATOR_S;
             break;
-        case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
+        case G_FILE_TYPE_SYMBOLIC_LINK:
             s = "@";
             break;
-        case GNOME_VFS_FILE_TYPE_FIFO:
-            s = "F";
-            break;
-        case GNOME_VFS_FILE_TYPE_SOCKET:
+        case G_FILE_TYPE_SPECIAL:
             s = "S";
             break;
-        case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
-            s = "C";
+        case G_FILE_TYPE_SHORTCUT:
+            s = "K"; // "Keyboard shortcut"
             break;
-        case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
-            s = "B";
+        case G_FILE_TYPE_MOUNTABLE:
+            s = "M";
             break;
 
         default:
@@ -269,65 +278,64 @@ const gchar *type2string (GnomeVFSFileType type, gchar *buf, guint max)
 }
 
 
-const gchar *perm2string (GnomeVFSFilePermissions p, gchar *buf, guint max)
+const gchar *perm2string (guint32 permissions, gchar *buf, guint max)
 {
     switch (gnome_cmd_data.options.perm_disp_mode)
     {
         case GNOME_CMD_PERM_DISP_MODE_TEXT:
-            return perm2textstring (p, buf, max);
+            return perm2textstring (permissions, buf, max);
 
         case GNOME_CMD_PERM_DISP_MODE_NUMBER:
-            return perm2numstring (p, buf, max);
+            return perm2numstring (permissions, buf, max);
 
         default:
-            return perm2textstring (p, buf, max);
+            return perm2textstring (permissions, buf, max);
     }
 }
 
 
-const gchar *perm2textstring (GnomeVFSFilePermissions p, gchar *buf, guint max)
+const gchar *perm2textstring (guint32 permissions, gchar *buf, guint max)
 {
     g_snprintf (buf, max, "%s%s%s%s%s%s%s%s%s",
-                (p & GNOME_VFS_PERM_USER_READ) ? "r" : "-",
-                (p & GNOME_VFS_PERM_USER_WRITE) ? "w" : "-",
-                (p & GNOME_VFS_PERM_USER_EXEC) ? "x" : "-",
-                (p & GNOME_VFS_PERM_GROUP_READ) ? "r" : "-",
-                (p & GNOME_VFS_PERM_GROUP_WRITE) ? "w" : "-",
-                (p & GNOME_VFS_PERM_GROUP_EXEC) ? "x" : "-",
-                (p & GNOME_VFS_PERM_OTHER_READ) ? "r" : "-",
-                (p & GNOME_VFS_PERM_OTHER_WRITE) ? "w" : "-",
-                (p & GNOME_VFS_PERM_OTHER_EXEC) ? "x" : "-");
+                (permissions & GNOME_CMD_PERM_USER_READ)   ? "r" : "-",
+                (permissions & GNOME_CMD_PERM_USER_WRITE)  ? "w" : "-",
+                (permissions & GNOME_CMD_PERM_USER_EXEC)   ? "x" : "-",
+                (permissions & GNOME_CMD_PERM_GROUP_READ)  ? "r" : "-",
+                (permissions & GNOME_CMD_PERM_GROUP_WRITE) ? "w" : "-",
+                (permissions & GNOME_CMD_PERM_GROUP_EXEC)  ? "x" : "-",
+                (permissions & GNOME_CMD_PERM_OTHER_READ)  ? "r" : "-",
+                (permissions & GNOME_CMD_PERM_OTHER_WRITE) ? "w" : "-",
+                (permissions & GNOME_CMD_PERM_OTHER_EXEC)  ? "x" : "-");
 
     return buf;
 }
 
 
-const gchar *perm2numstring (GnomeVFSFilePermissions p, gchar *buf, guint max)
+const gchar *perm2numstring (guint32 permissions, gchar *buf, guint max)
 {
     gint i = 0;
 
-    if (p & GNOME_VFS_PERM_USER_READ) i += 400;
-    if (p & GNOME_VFS_PERM_USER_WRITE) i += 200;
-    if (p & GNOME_VFS_PERM_USER_EXEC) i += 100;
-    if (p & GNOME_VFS_PERM_GROUP_READ) i += 40;
-    if (p & GNOME_VFS_PERM_GROUP_WRITE) i += 20;
-    if (p & GNOME_VFS_PERM_GROUP_EXEC) i += 10;
-    if (p & GNOME_VFS_PERM_OTHER_READ) i += 4;
-    if (p & GNOME_VFS_PERM_OTHER_WRITE) i += 2;
-    if (p & GNOME_VFS_PERM_OTHER_EXEC) i += 1;
+    if (permissions & GNOME_CMD_PERM_USER_READ)   i += 400;
+    if (permissions & GNOME_CMD_PERM_USER_WRITE)  i += 200;
+    if (permissions & GNOME_CMD_PERM_USER_EXEC)   i += 100;
+    if (permissions & GNOME_CMD_PERM_GROUP_READ)  i += 40;
+    if (permissions & GNOME_CMD_PERM_GROUP_WRITE) i += 20;
+    if (permissions & GNOME_CMD_PERM_GROUP_EXEC)  i += 10;
+    if (permissions & GNOME_CMD_PERM_OTHER_READ)  i += 4;
+    if (permissions & GNOME_CMD_PERM_OTHER_WRITE) i += 2;
+    if (permissions & GNOME_CMD_PERM_OTHER_EXEC)  i += 1;
 
-    g_snprintf (buf, max, "%d", i);
+    g_snprintf (buf, max, "%i", i);
 
     return buf;
 }
 
 
-const gchar *size2string (GnomeVFSFileSize size, GnomeCmdSizeDispMode size_disp_mode)
+const gchar *size2string (guint64 size, GnomeCmdSizeDispMode size_disp_mode)
 {
     static gchar buf0[64];
     static gchar buf1[128];
 
-
     switch (size_disp_mode)
     {
         case GNOME_CMD_SIZE_DISP_MODE_POWERED:
@@ -342,13 +350,13 @@ const gchar *size2string (GnomeVFSFileSize size, GnomeCmdSizeDispMode size_disp_
                 if (i)
                     g_snprintf (buf0, sizeof(buf0), "%.1f %s ", dsize, prefixes[i]);
                 else
-                    g_snprintf (buf0, sizeof(buf0), "%" GNOME_VFS_SIZE_FORMAT_STR " %s ", size, prefixes[0]);
+                    g_snprintf (buf0, sizeof(buf0), "%" G_GUINT64_FORMAT " %s ", size, prefixes[0]);
             }
             break;
 
         case GNOME_CMD_SIZE_DISP_MODE_GROUPED:
             {
-                gint len = g_snprintf (buf0, sizeof(buf0), "%" GNOME_VFS_SIZE_FORMAT_STR " ", size);
+                gint len = g_snprintf (buf0, sizeof(buf0), "%" G_GUINT64_FORMAT " ", size);
 
                 if (len < 5)
                     return buf0;
@@ -371,11 +379,11 @@ const gchar *size2string (GnomeVFSFileSize size, GnomeCmdSizeDispMode size_disp_
             return buf1;
 
         case GNOME_CMD_SIZE_DISP_MODE_LOCALE:
-            g_snprintf (buf0, sizeof(buf0), "%'" GNOME_VFS_SIZE_FORMAT_STR " ", size);
+            g_snprintf (buf0, sizeof(buf0), "%'" G_GUINT64_FORMAT " ", size);
             break;
 
         case GNOME_CMD_SIZE_DISP_MODE_PLAIN:
-            g_snprintf (buf0, sizeof(buf0), "%" GNOME_VFS_SIZE_FORMAT_STR " ", size);
+            g_snprintf (buf0, sizeof(buf0), "%" G_GUINT64_FORMAT " ", size);
             break;
 
         default:
@@ -386,29 +394,23 @@ const gchar *size2string (GnomeVFSFileSize size, GnomeCmdSizeDispMode size_disp_
 }
 
 
-const gchar *time2string (time_t t, const gchar *date_format)
+const gchar *time2string (GDateTime *gDateTime, const gchar *dateFormat)
 {
     // NOTE: date_format is passed in current locale format
 
+    g_return_val_if_fail (gDateTime != nullptr, nullptr);
+    g_return_val_if_fail (dateFormat != nullptr, nullptr);
+
     static gchar buf[64];
-    struct tm lt;
 
-    localtime_r (&t, &lt);
-#if defined (__GNUC__)
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif
-    strftime (buf, sizeof(buf), date_format, &lt);
-#if defined (__GNUC__)
-#pragma GCC diagnostic pop
-#endif
+    auto localGDateTime = g_date_time_to_local (gDateTime);
 
-    // convert formatted date from current locale to UTF8
-    gchar *loc_date = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-    if (loc_date)
-        strncpy (buf, loc_date, sizeof(buf)-1);
+    auto dateString = g_date_time_format (localGDateTime, dateFormat);
 
-    g_free (loc_date);
+    strncpy (buf, dateString, sizeof(buf)-1);
+
+    g_date_time_unref(localGDateTime);
+    g_free (dateString);
 
     return buf;
 }
@@ -425,98 +427,36 @@ void clear_event_key (GdkEventKey *event)
 
 
 /**
- * Transform a "\r\n" separated string into a GList with GnomeVFSURI's
+ * Transform a "\r\n" separated string into a GList with GFiles's
  */
-GList *strings_to_uris (gchar *data)
+GList *uri_strings_to_gfiles (gchar *data)
 {
-    GList *uri_list = NULL;
+    GList *gFileGList = NULL;
     gchar **filenames = g_strsplit (data, "\r\n", STRINGS_TO_URIS_CHUNK);
 
     for (gint i=0; filenames[i] != NULL; i++)
     {
         if (i == STRINGS_TO_URIS_CHUNK)
         {
-            uri_list = g_list_concat (uri_list, strings_to_uris (filenames[i]));
+            gFileGList = g_list_concat (gFileGList, uri_strings_to_gfiles (filenames[i]));
             break;
         }
 
         gchar *fn = g_strdup (filenames[i]);
-        GnomeVFSURI *uri = gnome_vfs_uri_new (fn);
-        fix_uri (uri);
-        if (uri)
-            uri_list = g_list_append (uri_list, uri);
+        if (!fn || strcmp(fn, "") == 0)
+        {
+            g_free(fn);
+            continue;
+        }
+        auto gFile = g_file_new_for_uri(fn);
+            gFileGList = g_list_append (gFileGList, gFile);
         g_free (fn);
     }
 
     g_strfreev (filenames);
-    return uri_list;
+    return gFileGList;
 }
 
-
-GnomeVFSFileSize calc_tree_size (const GnomeVFSURI *dir_uri, gulong *count)
-{
-    if (!dir_uri)
-        return -1;
-
-    gchar *dir_uri_str = gnome_vfs_uri_to_string (dir_uri, GNOME_VFS_URI_HIDE_PASSWORD);
-
-    g_return_val_if_fail (dir_uri_str != NULL, -1);
-
-    GList *list = NULL;
-    GnomeVFSFileSize size = 0;
-
-    GnomeVFSResult result = gnome_vfs_directory_list_load (&list, dir_uri_str, GNOME_VFS_FILE_INFO_DEFAULT);
-
-    if (result==GNOME_VFS_OK && list)
-    {
-        if (count!=NULL) {
-            (*count)++; // Count the directory too
-        }
-        for (GList *i = list; i; i = i->next)
-        {
-            GnomeVFSFileInfo *info = (GnomeVFSFileInfo *) i->data;
-            if (strcmp (info->name, ".") != 0 && strcmp (info->name, "..") != 0)
-            {
-                if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY)
-                {
-                    GnomeVFSURI *new_dir_uri = gnome_vfs_uri_append_file_name (dir_uri, info->name);
-                    size += calc_tree_size (new_dir_uri,count);
-                    gnome_vfs_uri_unref (new_dir_uri);
-                }
-                else
-		{
-                    size += info->size;
-                    if (count!=NULL) {
-                        (*count)++;
-                    }
-		}
-            }
-        }
-
-        for (GList *i = list; i; i = i->next)
-            gnome_vfs_file_info_unref ((GnomeVFSFileInfo *) i->data);
-
-        g_list_free (list);
-
-    } else if (result==GNOME_VFS_ERROR_NOT_A_DIRECTORY)
-    {
-        // A file
-        GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
-        gnome_vfs_get_file_info (dir_uri_str, info, GNOME_VFS_FILE_INFO_DEFAULT);
-        size += info->size;
-        if (count!=NULL) {
-            (*count)++;
-        }
-        gnome_vfs_file_info_unref (info);
-
-   }
-
-    g_free (dir_uri_str);
-
-    return size;
-}
-
-
 GList *string_history_add (GList *in, const gchar *value, guint maxsize)
 {
     GList *tmp = g_list_find_custom (in, (gchar *) value, (GCompareFunc) strcmp);
@@ -547,7 +487,7 @@ GList *string_history_add (GList *in, const gchar *value, guint maxsize)
 }
 
 
-gchar *create_nice_size_str (GnomeVFSFileSize size)
+gchar *create_nice_size_str (guint64 size)
 {
     GString *s = g_string_sized_new (64);
 
@@ -701,32 +641,43 @@ GList *app_get_linked_libs (GnomeCmdFile *f)
 
 gboolean app_needs_terminal (GnomeCmdFile *f)
 {
-    gboolean need_term = TRUE;
+    gboolean needTerminal;
 
-    if (strcmp (f->info->mime_type, "application/x-executable") && strcmp (f->info->mime_type, "application/x-executable-binary"))
-        return need_term;
-
-    GList *libs = app_get_linked_libs (f);
-    if  (!libs) return FALSE;
-
-    for (GList *i = libs; i; i = i->next)
+    auto contentType = f->GetGfileAttributeString(G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+    if (strcmp (contentType, "application/x-executable") && strcmp (contentType, "application/x-executable-binary"))
     {
-        gchar *lib = (gchar *) i->data;
-        lib = g_strstrip (lib);
-        if (g_str_has_prefix (lib, "libX11"))
+        needTerminal = TRUE;
+    }
+    else
+    {
+        GList *libs = app_get_linked_libs (f);
+        if  (!libs)
         {
-            need_term = FALSE;
-            break;
+            needTerminal = FALSE;
         }
+        else
+        {
+            for (GList *i = libs; i; i = i->next)
+            {
+                gchar *lib = (gchar *) i->data;
+                lib = g_strstrip (lib);
+                if (g_str_has_prefix (lib, "libX11"))
+                {
+                    needTerminal = FALSE;
+                    break;
+                }
+            }
+        }
+        g_list_foreach (libs, (GFunc) g_free, NULL);
+        g_list_free (libs);
     }
 
-    g_list_foreach (libs, (GFunc) g_free, NULL);
-    g_list_free (libs);
+    g_free(contentType);
 
-    return need_term;
+    return needTerminal;
 }
 
-
+// ToDo: This function should be reworked using GIO / GFile
 gchar *get_temp_download_filepath (const gchar *fname)
 {
     const gchar *tmp_dir = g_get_tmp_dir ();
@@ -834,120 +785,60 @@ GdkColor *gdk_color_new (gushort r, gushort g, gushort b)
 }
 
 
-GList *file_list_to_uri_list (GList *files)
+GList *gnome_cmd_file_list_to_gfile_list (GList *files)
 {
-    GList *uris = NULL;
+    GList *gFiles = nullptr;
 
     for (; files; files = files->next)
     {
         GnomeCmdFile *f = GNOME_CMD_FILE (files->data);
-        GnomeVFSURI *uri = f->get_uri();
+        auto gFile = f->get_gfile();
 
-        if (!uri)
-            g_warning ("NULL uri!!!");
+        if (!gFile)
+            g_warning ("gnome_cmd_file_list_to_gfile_list: no gFile!!!");
         else
-            uris = g_list_append (uris, uri);
+            gFiles = g_list_append (gFiles, gFile);
     }
 
-    return uris;
+    return gFiles;
 }
 
 
-/**
- * returns  1 if dir is existing,
- * returns  0 if dir is not existing,
- * returns -1 if dir is not readable
- */
-int is_dir_existing(const gchar *dpath)
+gboolean is_dir_existing(const gchar *dpath)
+{
+    g_return_val_if_fail (dpath, FALSE);
+    auto gFile = g_file_new_for_path(dpath);
+
+    auto returnValue = false;
+
+    if (g_file_query_exists(gFile, nullptr))
+    {
+        returnValue = true;
+    }
+    g_object_unref(gFile);
+    return returnValue;
+}
+
+
+gboolean create_dir (const gchar *dpath)
 {
     g_return_val_if_fail (dpath, FALSE);
 
-    DIR *dir = opendir (dpath);
-
-    if (!dir)
+    g_print (_("Creating directory %s… "), dpath);
+    GError *error = nullptr;
+    auto gFile = g_file_new_for_path(dpath);
+    if (g_file_make_directory (gFile, nullptr, &error))
     {
-        if (errno == ENOENT)
-        {
-            return 0;
-        }
-        else
-            g_warning (_("Couldn’t read from the directory %s: %s"), dpath, strerror (errno));
-
-        return -1;
+        g_object_unref(gFile);
+        return TRUE;
     }
-
-    closedir (dir);
-    return 1;
-}
-
-
-gboolean create_dir_if_needed (const gchar *dpath)
-{
-    g_return_val_if_fail (dpath, FALSE);
-
-    auto dir_exists = is_dir_existing(dpath);
-
-    switch (dir_exists)
+    else
     {
-        case -1:
-        {
-            return FALSE;
-        }
-        case 0:
-        {
-            g_print (_("Creating directory %s… "), dpath);
-            if (mkdir (dpath, S_IRUSR|S_IWUSR|S_IXUSR) == 0)
-            {
-                return TRUE;
-            }
-            else
-            {
-                gchar *msg = g_strdup_printf (_("Failed to create the directory %s"), dpath);
-                perror (msg);
-                g_free (msg);
-                return FALSE;
-            }
-            break;
-        }
-        case 1:
-        default:
-        {
-            return TRUE;
-        }
+        g_object_unref(gFile);
+        g_warning (_("Failed to create the directory %s: %s"), dpath, error->message);
+        g_error_free (error);
+        return FALSE;
     }
-
-    // never reached
-    return TRUE;
-}
-
-
-void fix_uri (GnomeVFSURI *uri)
-{
-#ifdef FIX_PW_HACK
-    gchar *p, *t;
-    const gchar *hn, *pw;
-
-    hn = gnome_vfs_uri_get_host_name (uri);
-    if (!hn) return;
-
-    pw = gnome_vfs_uri_get_password (uri);
-
-    t = g_strdup (hn);
-    p = strrchr(t,'@');
-    if (p && p[1] != '\0')
-    {
-        gchar *hn2;
-        gchar *pw2;
-        *p = '\0';
-        hn2 = g_strdup (p+1);
-        pw2 = g_strdup_printf ("%s@%s", pw, t);
-        gnome_vfs_uri_set_host_name (uri, hn2);
-        gnome_vfs_uri_set_password (uri, pw2);
-        g_free (hn2);
-        g_free (pw2);
-    }
-    g_free (t);
-#endif
 }
 
 
@@ -1052,6 +943,13 @@ void gnome_cmd_help_display (const gchar *file_name, const gchar *link_id)
 }
 
 
+void gnome_cmd_error_message (const gchar *title, GError *error)
+{
+    gnome_cmd_prompt_message (NULL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, title, error->message);
+    g_error_free (error);
+}
+
+
 gboolean gnome_cmd_prepend_su_to_vector (int &argc, char **&argv)
 {
     // sanity
@@ -1096,8 +994,8 @@ gboolean gnome_cmd_prepend_su_to_vector (int &argc, char **&argv)
     int real_argc = su_argc + argc;
     char **real_argv = g_new0 (char *, real_argc+1);
 
-    g_memmove (real_argv, su_argv, su_argc*sizeof(char *));
-    g_memmove (real_argv+su_argc, argv, argc*sizeof(char *));
+    memmove (real_argv, su_argv, su_argc*sizeof(char *));
+    memmove (real_argv+su_argc, argv, argc*sizeof(char *));
 
     g_free (argv);
 
@@ -1164,18 +1062,178 @@ gint get_string_pixel_size (const char *s, int len)
 }
 
 
-gboolean move_old_to_new_location(const gchar* oldPath, const gchar* newPath)
-{
-    if (rename(oldPath, newPath) == 0)
-    {
-        return TRUE;
-    }
-    g_warning (_("Couldn’t move path from “%s” to “%s”: %s"), oldPath, newPath, strerror (errno));
-
-    return FALSE;
-}
-
 gchar* get_package_config_dir()
 {
     return g_build_filename (g_get_user_config_dir(), PACKAGE, NULL);
-}
\ No newline at end of file
+}
+
+
+gchar *string_double_underscores (const gchar *string)
+{
+    if (!string)
+        return nullptr;
+
+    int underscores = 0;
+
+    for (const gchar *p = string; *p; p++)
+        underscores += (*p == '_');
+
+    if (underscores == 0)
+        return g_strdup (string);
+
+    gchar *escaped = g_new (char, strlen (string) + underscores + 1);
+    gchar *q = escaped;
+
+    for (const gchar *p = string; *p; p++, q++)
+    {
+        /* Add an extra underscore. */
+        if (*p == '_')
+            *q++ = '_';
+        *q = *p;
+    }
+    *q = '\0';
+
+    return escaped;
+}
+
+guint32 get_gfile_attribute_uint32(GFileInfo *gFileInfo, const char *attribute)
+{
+    g_return_val_if_fail(gFileInfo != nullptr, 0);
+    g_return_val_if_fail(G_IS_FILE_INFO(gFileInfo), 0);
+
+    return g_file_info_get_attribute_uint32 (gFileInfo, attribute);
+}
+
+guint32 get_gfile_attribute_uint32(GFile *gFile, const char *attribute)
+{
+    GError *error;
+    error = nullptr;
+    guint32 gFileAttributeUInt32 = 0;
+
+    auto gcmdFileInfo = g_file_query_info(gFile,
+                                   attribute,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   nullptr,
+                                   &error);
+    if (error)
+    {
+        auto gFileUri = g_file_get_uri(gFile);
+        g_message ("get_gfile_attribute_uint32: retrieving file info for %s failed: %s",
+                    gFileUri, error->message);
+        g_free(gFileUri);
+        g_error_free (error);
+    }
+    else
+    {
+        gFileAttributeUInt32 = get_gfile_attribute_uint32 (gcmdFileInfo, attribute);
+        g_object_unref(gcmdFileInfo);
+    }
+
+    return gFileAttributeUInt32;
+}
+
+guint64 get_gfile_attribute_uint64(GFileInfo *gFileInfo, const char *attribute)
+{
+    g_return_val_if_fail(gFileInfo != nullptr, 0);
+    g_return_val_if_fail(G_IS_FILE_INFO(gFileInfo), 0);
+
+    return g_file_info_get_attribute_uint64 (gFileInfo, attribute);
+}
+
+guint64 get_gfile_attribute_uint64(GFile *gFile, const char *attribute)
+{
+    GError *error;
+    error = nullptr;
+    guint64 gFileAttributeUInt64 = 0;
+
+    auto gcmdFileInfo = g_file_query_info(gFile,
+                                   attribute,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   nullptr,
+                                   &error);
+    if (error)
+    {
+        auto gFileUri = g_file_get_uri(gFile);
+        g_message ("get_gfile_attribute_uint64: retrieving file info for %s failed: %s",
+                    gFileUri, error->message);
+        g_free(gFileUri);
+        g_error_free (error);
+    }
+    else
+    {
+        gFileAttributeUInt64 = get_gfile_attribute_uint64 (gcmdFileInfo, attribute);
+        g_object_unref(gcmdFileInfo);
+    }
+
+    return gFileAttributeUInt64;
+}
+
+
+gboolean get_gfile_attribute_boolean(GFileInfo *gFileInfo, const char *attribute)
+{
+    g_return_val_if_fail(gFileInfo != nullptr, FALSE);
+    g_return_val_if_fail(G_IS_FILE_INFO(gFileInfo), FALSE);
+
+    return g_file_info_get_attribute_boolean (gFileInfo, attribute);
+}
+
+gboolean get_gfile_attribute_boolean(GFile *gFile, const char *attribute)
+{
+    GError *error;
+    error = nullptr;
+    gboolean gFileAttributeBoolean = 0;
+
+    auto gcmdFileInfo = g_file_query_info(gFile,
+                                   attribute,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   nullptr,
+                                   &error);
+    if (error)
+    {
+        auto gFileUri = g_file_get_uri(gFile);
+        g_message ("get_gfile_attribute_boolean: retrieving file info for %s failed: %s",
+                    gFileUri, error->message);
+        g_free(gFileUri);
+        g_error_free (error);
+    }
+    else
+    {
+        gFileAttributeBoolean = get_gfile_attribute_boolean (gcmdFileInfo, attribute);
+        g_object_unref(gcmdFileInfo);
+    }
+
+    return gFileAttributeBoolean;
+}
+
+gchar *get_gfile_attribute_string(GFileInfo *gFileInfo, const char *attribute)
+{
+    g_return_val_if_fail(gFileInfo != nullptr, nullptr);
+    g_return_val_if_fail(G_IS_FILE_INFO(gFileInfo), nullptr);
+
+    return g_strdup(g_file_info_get_attribute_string (gFileInfo, attribute));
+}
+
+gchar *get_gfile_attribute_string(GFile *gFile, const char *attribute)
+{
+    GError *error;
+    error = nullptr;
+    auto gcmdFileInfo = g_file_query_info(gFile,
+                                   attribute,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   nullptr,
+                                   &error);
+    if (gcmdFileInfo && error)
+    {
+        auto gFileUri = g_file_get_uri(gFile);
+        g_message ("get_gfile_attribute_string: retrieving file info for %s failed: %s",
+                    gFileUri, error->message);
+        g_free(gFileUri);
+        g_error_free (error);
+        return nullptr;
+    }
+
+    auto gFileAttributeString = get_gfile_attribute_string (gcmdFileInfo, attribute);
+    g_object_unref(gcmdFileInfo);
+
+    return gFileAttributeString;
+}
